/*
 * Z80 disassembler for	Z80-CPU	simulator
 *
 * Copyright (C) 1989-2006 by Udo Munk
 *
 * History:
 * 06-DEC-89 Development on TARGON/35 with AT&T Unix System V.3
 * 07-APR-92 forget to implement Op-Codes LD A,R and LD R,A, added
 * 25-JUN-92 comments in english
 * 03-OCT-06 changed to ANSI C for modern POSIX OS's
 * 27-MAR-07 Release 2.0 Ported to CPP by J.Sievers <JSievers@nadhh.Hanse.de> 
 */


#include "stdafx.h"
#include "zilog.h"
#ifdef WANT_DISASS
#include <stdio.h>

const struct z80::opt z80::optab[256] = {
	{ &z80::opout,  "NOP"			},	/* 0x00	*/
	{ &z80::nnout,  "LD\tBC,"		},	/* 0x01	*/
	{ &z80::opout,  "LD\t(BC),A"	},	/* 0x02	*/
	{ &z80::opout,  "INC\tBC"		},	/* 0x03	*/
	{ &z80::opout,  "INC\tB"		},	/* 0x04	*/
	{ &z80::opout,  "DEC\tB"		},	/* 0x05	*/
	{ &z80::nout,	"LD\tB,"		},	/* 0x06	*/
	{ &z80::opout,  "RLCA"			},	/* 0x07	*/
	{ &z80::opout,  "EX\tAF,AF'"	},	/* 0x08	*/
	{ &z80::opout,  "ADD\tHL,BC"	},	/* 0x09	*/
	{ &z80::opout,  "LD\tA,(BC)"	},	/* 0x0a	*/
	{ &z80::opout,  "DEC\tBC"		},	/* 0x0b	*/
	{ &z80::opout,  "INC\tC"		},	/* 0x0c	*/
	{ &z80::opout,  "DEC\tC"		},	/* 0x0d	*/
	{ &z80::nout,	"LD\tC,"		},	/* 0x0e	*/
	{ &z80::opout,  "RRCA"			},	/* 0x0f	*/
	{ &z80::rout,	"DJNZ\t"		},	/* 0x10	*/
	{ &z80::nnout,  "LD\tDE,"		},	/* 0x11	*/
	{ &z80::opout,  "LD\t(DE),A"	},	/* 0x12	*/
	{ &z80::opout,  "INC\tDE"		},	/* 0x13	*/
	{ &z80::opout,  "INC\tD"		},	/* 0x14	*/
	{ &z80::opout,  "DEC\tD"		},	/* 0x15	*/
	{ &z80::nout,	"LD\tD,"		},	/* 0x16	*/
	{ &z80::opout,  "RLA"			},	/* 0x17	*/
	{ &z80::rout,	"JR\t"			},	/* 0x18	*/
	{ &z80::opout,  "ADD\tHL,DE"	},	/* 0x19	*/
	{ &z80::opout,  "LD\tA,(DE)"	},	/* 0x1a	*/
	{ &z80::opout,  "DEC\tDE"		},	/* 0x1b	*/
	{ &z80::opout,  "INC\tE"		},	/* 0x1c	*/
	{ &z80::opout,  "DEC\tE"		},	/* 0x1d	*/
	{ &z80::nout,	"LD\tE,"		},	/* 0x1e	*/
	{ &z80::opout,  "RRA"			},	/* 0x1f	*/
	{ &z80::rout,	"JR\tNZ,"		},	/* 0x20	*/
	{ &z80::nnout,  "LD\tHL,"		},	/* 0x21	*/
	{ &z80::inout,  "LD\t(%04x),HL"	},	/* 0x22	*/
	{ &z80::opout,  "INC\tHL"		},	/* 0x23	*/
	{ &z80::opout,  "INC\tH"		},	/* 0x24	*/
	{ &z80::opout,  "DEC\tH"		},	/* 0x25	*/
	{ &z80::nout,	"LD\tH,"		},	/* 0x26	*/
	{ &z80::opout,  "DAA"			},	/* 0x27	*/
	{ &z80::rout,	"JR\tZ,"		},	/* 0x28	*/
	{ &z80::opout,  "ADD\tHL,HL"	},	/* 0x29	*/
	{ &z80::inout,  "LD\tHL,(%04x)"	},	/* 0x2a	*/
	{ &z80::opout,  "DEC\tHL"		},	/* 0x2b	*/
	{ &z80::opout,  "INC\tL"		},	/* 0x2c	*/
	{ &z80::opout,  "DEC\tL"		},	/* 0x2d	*/
	{ &z80::nout,	"LD\tL,"		},	/* 0x2e	*/
	{ &z80::opout,  "CPL"			},	/* 0x2f	*/
	{ &z80::rout,	"JR\tNC,"		},	/* 0x30	*/
	{ &z80::nnout,  "LD\tSP,"		},	/* 0x31	*/
	{ &z80::inout,  "LD\t(%04x),A"	},	/* 0x32	*/
	{ &z80::opout,  "INC\tSP"		},	/* 0x33	*/
	{ &z80::opout,  "INC\t(HL)"		},	/* 0x34	*/
	{ &z80::opout,  "DEC\t(HL)"		},	/* 0x35	*/
	{ &z80::nout,	"LD\t(HL),"		},	/* 0x36	*/
	{ &z80::opout,  "SCF"			},	/* 0x37	*/
	{ &z80::rout,	"JR\tC,"		},	/* 0x38	*/
	{ &z80::opout,  "ADD\tHL,SP"	},	/* 0x39	*/
	{ &z80::inout,  "LD\tA,(%04x)"	},	/* 0x3a	*/
	{ &z80::opout,  "DEC\tSP"		},	/* 0x3b	*/
	{ &z80::opout,  "INC\tA"		},	/* 0x3c	*/
	{ &z80::opout,  "DEC\tA"		},	/* 0x3d	*/
	{ &z80::nout,	"LD\tA,"		},	/* 0x3e	*/
	{ &z80::opout,  "CCF"			},	/* 0x3f	*/
	{ &z80::opout,  "LD\tB,B"		},	/* 0x40	*/
	{ &z80::opout,  "LD\tB,C"		},	/* 0x41	*/
	{ &z80::opout,  "LD\tB,D"		},	/* 0x42	*/
	{ &z80::opout,  "LD\tB,E"		},	/* 0x43	*/
	{ &z80::opout,  "LD\tB,H"		},	/* 0x44	*/
	{ &z80::opout,  "LD\tB,L"		},	/* 0x45	*/
	{ &z80::opout,  "LD\tB,(HL)"	},	/* 0x46	*/
	{ &z80::opout,  "LD\tB,A"		},	/* 0x47	*/
	{ &z80::opout,  "LD\tC,B"		},	/* 0x48	*/
	{ &z80::opout,  "LD\tC,C"		},	/* 0x49	*/
	{ &z80::opout,  "LD\tC,D"		},	/* 0x4a	*/
	{ &z80::opout,  "LD\tC,E"		},	/* 0x4b	*/
	{ &z80::opout,  "LD\tC,H"		},	/* 0x4c	*/
	{ &z80::opout,  "LD\tC,L"		},	/* 0x4d	*/
	{ &z80::opout,  "LD\tC,(HL)"	},	/* 0x4e	*/
	{ &z80::opout,  "LD\tC,A"		},	/* 0x4f	*/
	{ &z80::opout,  "LD\tD,B"		},	/* 0x50	*/
	{ &z80::opout,  "LD\tD,C"		},	/* 0x51	*/
	{ &z80::opout,  "LD\tD,D"		},	/* 0x52	*/
	{ &z80::opout,  "LD\tD,E"		},	/* 0x53	*/
	{ &z80::opout,  "LD\tD,H"		},	/* 0x54	*/
	{ &z80::opout,  "LD\tD,L"		},	/* 0x55	*/
	{ &z80::opout,  "LD\tD,(HL)"	},	/* 0x56	*/
	{ &z80::opout,  "LD\tD,A"		},	/* 0x57	*/
	{ &z80::opout,  "LD\tE,B"		},	/* 0x58	*/
	{ &z80::opout,  "LD\tE,C"		},	/* 0x59	*/
	{ &z80::opout,  "LD\tE,D"		},	/* 0x5a	*/
	{ &z80::opout,  "LD\tE,E"		},	/* 0x5b	*/
	{ &z80::opout,  "LD\tE,H"		},	/* 0x5c	*/
	{ &z80::opout,  "LD\tE,L"		},	/* 0x5d	*/
	{ &z80::opout,  "LD\tE,(HL)"	},	/* 0x5e	*/
	{ &z80::opout,  "LD\tE,A"		},	/* 0x5f	*/
	{ &z80::opout,  "LD\tH,B"		},	/* 0x60	*/
	{ &z80::opout,  "LD\tH,C"		},	/* 0x61	*/
	{ &z80::opout,  "LD\tH,D"		},	/* 0x62	*/
	{ &z80::opout,  "LD\tH,E"		},	/* 0x63	*/
	{ &z80::opout,  "LD\tH,H"		},	/* 0x64	*/
	{ &z80::opout,  "LD\tH,L"		},	/* 0x65	*/
	{ &z80::opout,  "LD\tH,(HL)"	},	/* 0x66	*/
	{ &z80::opout,  "LD\tH,A"		},	/* 0x67	*/
	{ &z80::opout,  "LD\tL,B"		},	/* 0x68	*/
	{ &z80::opout,  "LD\tL,C"		},	/* 0x69	*/
	{ &z80::opout,  "LD\tL,D"		},	/* 0x6a	*/
	{ &z80::opout,  "LD\tL,E"		},	/* 0x6b	*/
	{ &z80::opout,  "LD\tL,H"		},	/* 0x6c	*/
	{ &z80::opout,  "LD\tL,L"		},	/* 0x6d	*/
	{ &z80::opout,  "LD\tL,(HL)"	},	/* 0x6e	*/
	{ &z80::opout,  "LD\tL,A"		},	/* 0x6f	*/
	{ &z80::opout,  "LD\t(HL),B"	},	/* 0x70	*/
	{ &z80::opout,  "LD\t(HL),C"	},	/* 0x71	*/
	{ &z80::opout,  "LD\t(HL),D"	},	/* 0x72	*/
	{ &z80::opout,  "LD\t(HL),E"	},	/* 0x73	*/
	{ &z80::opout,  "LD\t(HL),H"	},	/* 0x74	*/
	{ &z80::opout,  "LD\t(HL),L"	},	/* 0x75	*/
	{ &z80::opout,  "HALT"			},	/* 0x76	*/
	{ &z80::opout,  "LD\t(HL),A"	},	/* 0x77	*/
	{ &z80::opout,  "LD\tA,B"		},	/* 0x78	*/
	{ &z80::opout,  "LD\tA,C"		},	/* 0x79	*/
	{ &z80::opout,  "LD\tA,D"		},	/* 0x7a	*/
	{ &z80::opout,  "LD\tA,E"		},	/* 0x7b	*/
	{ &z80::opout,  "LD\tA,H"		},	/* 0x7c	*/
	{ &z80::opout,  "LD\tA,L"		},	/* 0x7d	*/
	{ &z80::opout,  "LD\tA,(HL)"	},	/* 0x7e	*/
	{ &z80::opout,  "LD\tA,A"		},	/* 0x7f	*/
	{ &z80::opout,  "ADD\tA,B"		},	/* 0x80	*/
	{ &z80::opout,  "ADD\tA,C"		},	/* 0x81	*/
	{ &z80::opout,  "ADD\tA,D"		},	/* 0x82	*/
	{ &z80::opout,  "ADD\tA,E"		},	/* 0x83	*/
	{ &z80::opout,  "ADD\tA,H"		},	/* 0x84	*/
	{ &z80::opout,  "ADD\tA,L"		},	/* 0x85	*/
	{ &z80::opout,  "ADD\tA,(HL)"	},	/* 0x86	*/
	{ &z80::opout,  "ADD\tA,A"		},	/* 0x87	*/
	{ &z80::opout,  "ADC\tA,B"		},	/* 0x88	*/
	{ &z80::opout,  "ADC\tA,C"		},	/* 0x89	*/
	{ &z80::opout,  "ADC\tA,D"		},	/* 0x8a	*/
	{ &z80::opout,  "ADC\tA,E"		},	/* 0x8b	*/
	{ &z80::opout,  "ADC\tA,H"		},	/* 0x8c	*/
	{ &z80::opout,  "ADC\tA,L"		},	/* 0x8d	*/
	{ &z80::opout,  "ADC\tA,(HL)"	},	/* 0x8e	*/
	{ &z80::opout,  "ADC\tA,A"		},	/* 0x8f	*/
	{ &z80::opout,  "SUB\tB"		},	/* 0x90	*/
	{ &z80::opout,  "SUB\tC"		},	/* 0x91	*/
	{ &z80::opout,  "SUB\tD"		},	/* 0x92	*/
	{ &z80::opout,  "SUB\tE"		},	/* 0x93	*/
	{ &z80::opout,  "SUB\tH"		},	/* 0x94	*/
	{ &z80::opout,  "SUB\tL"		},	/* 0x95	*/
	{ &z80::opout,  "SUB\t(HL)"		},	/* 0x96	*/
	{ &z80::opout,  "SUB\tA"		},	/* 0x97	*/
	{ &z80::opout,  "SBC\tA,B"		},	/* 0x98	*/
	{ &z80::opout,  "SBC\tA,C"		},	/* 0x99	*/
	{ &z80::opout,  "SBC\tA,D"		},	/* 0x9a	*/
	{ &z80::opout,  "SBC\tA,E"		},	/* 0x9b	*/
	{ &z80::opout,  "SBC\tA,H"		},	/* 0x9c	*/
	{ &z80::opout,  "SBC\tA,L"		},	/* 0x9d	*/
	{ &z80::opout,  "SBC\tA,(HL)"	},	/* 0x9e	*/
	{ &z80::opout,  "SBC\tA,A"		},	/* 0x9f	*/
	{ &z80::opout,  "AND\tB"		},	/* 0xa0	*/
	{ &z80::opout,  "AND\tC"		},	/* 0xa1	*/
	{ &z80::opout,  "AND\tD"		},	/* 0xa2	*/
	{ &z80::opout,  "AND\tE"		},	/* 0xa3	*/
	{ &z80::opout,  "AND\tH"		},	/* 0xa4	*/
	{ &z80::opout,  "AND\tL"		},	/* 0xa5	*/
	{ &z80::opout,  "AND\t(HL)"		},	/* 0xa6	*/
	{ &z80::opout,  "AND\tA"		},	/* 0xa7	*/
	{ &z80::opout,  "XOR\tB"		},	/* 0xa8	*/
	{ &z80::opout,  "XOR\tC"		},	/* 0xa9	*/
	{ &z80::opout,  "XOR\tD"		},	/* 0xaa	*/
	{ &z80::opout,  "XOR\tE"		},	/* 0xab	*/
	{ &z80::opout,  "XOR\tH"		},	/* 0xac	*/
	{ &z80::opout,  "XOR\tL"		},	/* 0xad	*/
	{ &z80::opout,  "XOR\t(HL)"		},	/* 0xae	*/
	{ &z80::opout,  "XOR\tA"		},	/* 0xaf	*/
	{ &z80::opout,  "OR\tB"			},	/* 0xb0	*/
	{ &z80::opout,  "OR\tC"			},	/* 0xb1	*/
	{ &z80::opout,  "OR\tD"			},	/* 0xb2	*/
	{ &z80::opout,  "OR\tE"			},	/* 0xb3	*/
	{ &z80::opout,  "OR\tH"			},	/* 0xb4	*/
	{ &z80::opout,  "OR\tL"			},	/* 0xb5	*/
	{ &z80::opout,  "OR\t(HL)"		},	/* 0xb6	*/
	{ &z80::opout,  "OR\tA"			},	/* 0xb7	*/
	{ &z80::opout,  "CP\tB"			},	/* 0xb8	*/
	{ &z80::opout,  "CP\tC"			},	/* 0xb9	*/
	{ &z80::opout,  "CP\tD"			},	/* 0xba	*/
	{ &z80::opout,  "CP\tE"			},	/* 0xbb	*/
	{ &z80::opout,  "CP\tH"			},	/* 0xbc	*/
	{ &z80::opout,  "CP\tL"			},	/* 0xbd	*/
	{ &z80::opout,  "CP\t(HL)"		},	/* 0xbe	*/
	{ &z80::opout,  "CP\tA"			},	/* 0xbf	*/
	{ &z80::opout,  "RET\tNZ"		},	/* 0xc0	*/
	{ &z80::opout,  "POP\tBC"		},	/* 0xc1	*/
	{ &z80::nnout,  "JP\tNZ,"		},	/* 0xc2	*/
	{ &z80::nnout,  "JP\t"			},	/* 0xc3	*/
	{ &z80::nnout,  "CALL\tNZ,"		},	/* 0xc4	*/
	{ &z80::opout,  "PUSH\tBC"		},	/* 0xc5	*/
	{ &z80::nout,	"ADD\tA,"		},	/* 0xc6	*/
	{ &z80::opout,  "RST\t0"		},	/* 0xc7	*/
	{ &z80::opout,  "RET\tZ"		},	/* 0xc8	*/
	{ &z80::opout,  "RET"			},	/* 0xc9	*/
	{ &z80::nnout,  "JP\tZ,"		},	/* 0xca	*/
	{ &z80::cbop,	""				},	/* 0xcb	*/
	{ &z80::nnout,  "CALL\tZ,"		},	/* 0xcc	*/
	{ &z80::nnout,  "CALL\t"		},	/* 0xcd	*/
	{ &z80::nout,	"ADC\tA,"		},	/* 0xce	*/
	{ &z80::opout,  "RST\t8"		},	/* 0xcf	*/
	{ &z80::opout,  "RET\tNC"		},	/* 0xd0	*/
	{ &z80::opout,  "POP\tDE"		},	/* 0xd1	*/
	{ &z80::nnout,  "JP\tNC,"		},	/* 0xd2	*/
	{ &z80::iout,	"OUT\t(%02x),A"	},	/* 0xd3	*/
	{ &z80::nnout,  "CALL\tNC,"		},	/* 0xd4	*/
	{ &z80::opout,  "PUSH\tDE"		},	/* 0xd5	*/
	{ &z80::nout,	"SUB\t"			},	/* 0xd6	*/
	{ &z80::opout,  "RST\t10"		},	/* 0xd7	*/
	{ &z80::opout,  "RET\tC"		},	/* 0xd8	*/
	{ &z80::opout,  "EXX"			},	/* 0xd9	*/
	{ &z80::nnout,  "JP\tC,"		},	/* 0xda	*/
	{ &z80::iout,	"IN\tA,(%02x)"	},	/* 0xdb	*/
	{ &z80::nnout,  "CALL\tC,"		},	/* 0xdc	*/
	{ &z80::ddfd,	""				},	/* 0xdd	*/
	{ &z80::nout,	"SBC\tA,"		},	/* 0xde	*/
	{ &z80::opout,  "RST\t18"		},	/* 0xdf	*/
	{ &z80::opout,  "RET\tPO"		},	/* 0xe0	*/
	{ &z80::opout,  "POP\tHL"		},	/* 0xe1	*/
	{ &z80::nnout,  "JP\tPO,"		},	/* 0xe2	*/
	{ &z80::opout,  "EX\t(SP),HL"	},	/* 0xe3	*/
	{ &z80::nnout,  "CALL\tPO,"		},	/* 0xe4	*/
	{ &z80::opout,  "PUSH\tHL"		},	/* 0xe5	*/
	{ &z80::nout,	"AND\t"			},	/* 0xe6	*/
	{ &z80::opout,  "RST\t20"		},	/* 0xe7	*/
	{ &z80::opout,  "RET\tPE"		},	/* 0xe8	*/
	{ &z80::opout,  "JP\t(HL)"		},	/* 0xe9	*/
	{ &z80::nnout,  "JP\tPE,"		},	/* 0xea	*/
	{ &z80::opout,  "EX\tDE,HL"		},	/* 0xeb	*/
	{ &z80::nnout,  "CALL\tPE,"		},	/* 0xec	*/
	{ &z80::edop,	""				},	/* 0xed	*/
	{ &z80::nout,	"XOR\t"			},	/* 0xee	*/
	{ &z80::opout,  "RST\t28"		},	/* 0xef	*/
	{ &z80::opout,  "RET\tP"		},	/* 0xf0	*/
	{ &z80::opout,  "POP\tAF"		},	/* 0xf1	*/
	{ &z80::nnout,  "JP\tP,"		},	/* 0xf2	*/
	{ &z80::opout,  "DI"			},	/* 0xf3	*/
	{ &z80::nnout,  "CALL\tP,"		},	/* 0xf4	*/
	{ &z80::opout,  "PUSH\tAF"		},	/* 0xf5	*/
	{ &z80::nout,	"OR\t"			},	/* 0xf6	*/
	{ &z80::opout,  "RST\t30"		},	/* 0xf7	*/
	{ &z80::opout,  "RET\tM"		},	/* 0xf8	*/
	{ &z80::opout,  "LD\tSP,HL"		},	/* 0xf9	*/
	{ &z80::nnout,  "JP\tM,"		},	/* 0xfa	*/
	{ &z80::opout,  "EI"			},	/* 0xfb	*/
	{ &z80::nnout,  "CALL\tM,"		},	/* 0xfc	*/
	{ &z80::ddfd,	""				},	/* 0xfd	*/
	{ &z80::nout,	"CP\t"			},	/* 0xfe	*/
	{ &z80::opout,  "RST\t38"		}	/* 0xff	*/
};

const char *z80::unkown = "???";
const char *z80::reg[] = { "B", "C", "D", "E", "H",	"L", "(HL)", "A" };
const char *z80::regix = "IX";
const char *z80::regiy = "IY";

/*
 *	The function disass() is the only global function of
 *	this module. The first argument is a pointer to a
 *	unsigned char pointer, which points to the op-code
 *	to disassemble. The output of the disassembly goes
 *	to stdout, terminated by a newline. After the
 *	disassembly the pointer to the op-code will be
 *	increased by the size of the op-code, so that
 *	disass() can be called again.
 *	The secound argument is the (Z80) address of the
 *	op-code to disassemble. It is used to calculate the
 *	destination address of relative jumps.
 */
WORD z80::disass(WORD pc)
{
	addr = pc;
	WORD ret;
	fprintf(fdis,"\n%04X:\t",pc);
	ret = pc + (this->*optab[(*this)[pc]].fun)(optab[(*this)[pc]].text, pc);
	fflush(fdis);
	return ret;
}

/*
 *	disassemble 1 byte op-codes
 */
int z80::opout(const char *s, WORD p)
{
	fprintf(fdis,"%s",s);
	return(1);
}

/*
 *	disassemble 2 byte op-codes of type "Op n"
 */
int z80::nout(const char *s, WORD p)
{
	fprintf(fdis,"%s%02x", s, (*this)[p + 1]);
	return(2);
}

/*
 *	disassemble 2 byte op-codes with indirect addressing
 */
int z80::iout(const char *s, WORD p)
{
	fprintf(fdis,s, (*this)[p+1]);
	fprintf(fdis,"");
	return(2);
}

/*
 *	disassemble 2 byte op-codes with relative addressing
 */
int z80::rout(const char *s, WORD p)
{
	fprintf(fdis,"%s%04x", s, addr + (*this)[p+1] + 2);
	return(2);
}

/*
 *	disassemble 3 byte op-codes of type "Op nn"
 */
int z80::nnout(const char *s, WORD p)
{
	register int i;

	i = (*this)[p+1] +	((*this)[p+2] << 8);
	fprintf(fdis,"%s%04x", s, i);
	return(3);
}

/*
 *	disassemble 3 byte op-codes with indirect addressing
 */
int z80::inout(const char *s, WORD p)
{
	register int i;

	i = (*this)[p+1] +	((*this)[p+2] << 8);
	fprintf(fdis,s, i);
	fprintf(fdis,"");
	return(3);
}

/*
 *	disassemble multi byte op-codes with prefix 0xcb
 */
int z80::cbop(const char *s, WORD p)
{
	register int b2;

	b2 = (*this)[p+1];
	if (b2 >= 0x00 && b2 <=	0x07) {
		fprintf(fdis,"RLC\t");
		fprintf(fdis,"%s", reg[b2 &	7]);
		return(2);
	}
	if (b2 >= 0x08 && b2 <=	0x0f) {
		fprintf(fdis,"RRC\t");
		fprintf(fdis,"%s", reg[b2 &	7]);
		return(2);
	}
	if (b2 >= 0x10 && b2 <=	0x17) {
		fprintf(fdis,"RL\t");
		fprintf(fdis,"%s", reg[b2 &	7]);
		return(2);
	}
	if (b2 >= 0x18 && b2 <=	0x1f) {
		fprintf(fdis,"RR\t");
		fprintf(fdis,"%s", reg[b2 &	7]);
		return(2);
	}
	if (b2 >= 0x20 && b2 <=	0x27) {
		fprintf(fdis,"SLA\t");
		fprintf(fdis,"%s", reg[b2 &	7]);
		return(2);
	}
	if (b2 >= 0x28 && b2 <=	0x2f) {
		fprintf(fdis,"SRA\t");
		fprintf(fdis,"%s", reg[b2 &	7]);
		return(2);
	}
	if (b2 >= 0x38 && b2 <=	0x3f) {
		fprintf(fdis,"SRL\t");
		fprintf(fdis,"%s", reg[b2 &	7]);
		return(2);
	}
	if (b2 >= 0x40 && b2 <=	0x7f) {
		fprintf(fdis,"BIT\t");
		fprintf(fdis,"%c,", ((b2 >> 3) & 7) +	'0');
		fprintf(fdis,"%s", reg[b2 &	7]);
		return(2);
	}
	if (b2 >= 0x80 && b2 <=	0xbf) {
		fprintf(fdis,"RES\t");
		fprintf(fdis,"%c,", ((b2 >> 3) & 7) +	'0');
		fprintf(fdis,"%s", reg[b2 &	7]);
		return(2);
	}
	if (b2 >= 0xc0)	{
		fprintf(fdis,"SET\t");
		fprintf(fdis,"%c,", ((b2 >> 3) & 7) +	'0');
		fprintf(fdis,"%s", reg[b2 &	7]);
		return(2);
	}
	fprintf(fdis,"%s",unkown);
	return(2);
}

/*
 *	disassemble multi byte op-codes with prefix 0xed
 */
int z80::edop(const char *s, WORD p)
{
	register int b2, i;
	int len	= 2;

	b2 = (*this)[p+1];
	switch (b2) {
	case 0x40:
		fprintf(fdis,"%s","IN\tB,(C)");
		break;
	case 0x41:
		fprintf(fdis,"%s","OUT\t(C),B");
		break;
	case 0x42:
		fprintf(fdis,"%s","SBC\tHL,BC");
		break;
	case 0x43:
		i = (*this)[p + 2] +((*this)[p +	3] << 8);
		fprintf(fdis,"LD\t(%04x),BC", i);
		len = 4;
		break;
	case 0x44:
		fprintf(fdis,"%s","NEG");
		break;
	case 0x45:
		fprintf(fdis,"%s","RETN");
		break;
	case 0x46:
		fprintf(fdis,"%s","IM\t0");
		break;
	case 0x47:
		fprintf(fdis,"%s","LD\tI,A");
		break;
	case 0x48:
		fprintf(fdis,"%s","IN\tC,(C)");
		break;
	case 0x49:
		fprintf(fdis,"%s","OUT\t(C),C");
		break;
	case 0x4a:
		fprintf(fdis,"%s","ADC\tHL,BC");
		break;
	case 0x4b:
		i = (*this)[p + 2] +((*this)[p + 3] << 8);
		fprintf(fdis,"LD\tBC,(%04x)", i);
		len = 4;
		break;
	case 0x4d:
		fprintf(fdis,"%s","RETI");
		break;
	case 0x4f:
		fprintf(fdis,"%s","LD\tR,A");
		break;
	case 0x50:
		fprintf(fdis,"%s","IN\tD,(C)");
		break;
	case 0x51:
		fprintf(fdis,"%s","OUT\t(C),D");
		break;
	case 0x52:
		fprintf(fdis,"%s","SBC\tHL,DE");
		break;
	case 0x53:
		i = (*this)[p + 2] +((*this)[p + 3] << 8);
		fprintf(fdis,"LD\t(%04x),DE\n", i);
		len = 4;
		break;
	case 0x56:
		fprintf(fdis,"%s","IM\t1");
		break;
	case 0x57:
		fprintf(fdis,"%s","LD\tA,I");
		break;
	case 0x58:
		fprintf(fdis,"%s","IN\tE,(C)");
		break;
	case 0x59:
		fprintf(fdis,"%s","OUT\t(C),E");
		break;
	case 0x5a:
		fprintf(fdis,"%s","ADC\tHL,DE");
		break;
	case 0x5b:
		i = (*this)[p + 2] +((*this)[p + 3] << 8);
		fprintf(fdis,"LD\tDE,(%04x)", i);
		len = 4;
		break;
	case 0x5e:
		fprintf(fdis,"%s","IM\t2");
		break;
	case 0x5f:
		fprintf(fdis,"%s","LD\tA,R");
		break;
	case 0x60:
		fprintf(fdis,"%s","IN\tH,(C)");
		break;
	case 0x61:
		fprintf(fdis,"%s","OUT\t(C),H");
		break;
	case 0x62:
		fprintf(fdis,"%s","SBC\tHL,HL");
		break;
	case 0x67:
		fprintf(fdis,"%s","RRD");
		break;
	case 0x68:
		fprintf(fdis,"%s","IN\tL,(C)");
		break;
	case 0x69:
		fprintf(fdis,"%s","OUT\t(C),L");
		break;
	case 0x6a:
		fprintf(fdis,"%s","ADC\tHL,HL");
		break;
	case 0x6f:
		fprintf(fdis,"%s","RLD");
		break;
	case 0x72:
		fprintf(fdis,"%s","SBC\tHL,SP");
		break;
	case 0x73:
		i = (*this)[p + 2] +((*this)[p + 3] << 8);
		fprintf(fdis,"LD\t(%04x),SP", i);
		len = 4;
		break;
	case 0x78:
		fprintf(fdis,"%s","IN\tA,(C)");
		break;
	case 0x79:
		fprintf(fdis,"%s","OUT\t(C),A");
		break;
	case 0x7a:
		fprintf(fdis,"%s","ADC\tHL,SP");
		break;
	case 0x7b:
		i = (*this)[p + 2] +((*this)[p + 3] << 8);
		fprintf(fdis,"LD\tSP,(%04x)", i);
		len = 4;
		break;
	case 0xa0:
		fprintf(fdis,"%s","LDI");
		break;
	case 0xa1:
		fprintf(fdis,"%s","CPI");
		break;
	case 0xa2:
		fprintf(fdis,"%s","INI");
		break;
	case 0xa3:
		fprintf(fdis,"%s","OUTI");
		break;
	case 0xa8:
		fprintf(fdis,"%s","LDD");
		break;
	case 0xa9:
		fprintf(fdis,"%s","CPD");
		break;
	case 0xaa:
		fprintf(fdis,"%s","IND");
		break;
	case 0xab:
		fprintf(fdis,"%s","OUTD");
		break;
	case 0xb0:
		fprintf(fdis,"%s","LDIR");
		break;
	case 0xb1:
		fprintf(fdis,"%s","CPIR");
		break;
	case 0xb2:
		fprintf(fdis,"%s","INIR");
		break;
	case 0xb3:
		fprintf(fdis,"%s","OTIR");
		break;
	case 0xb8:
		fprintf(fdis,"%s","LDDR");
		break;
	case 0xb9:
		fprintf(fdis,"%s","CPDR");
		break;
	case 0xba:
		fprintf(fdis,"%s","INDR");
		break;
	case 0xbb:
		fprintf(fdis,"%s","OTDR");
		break;
	default:
		fprintf(fdis,"%s",unkown);
	}
	return(len);
}

/*
 *	disassemble multi byte op-codes with prefix 0xdd and 0xfd
 */
int z80::ddfd(const char *s, WORD p)
{
	register int b2;
	register const char *ireg;
	int len	= 3;

	if ((*this)[p]	== 0xdd)
		ireg = regix;
	else
		ireg = regiy;
	b2 = (*this)[p+1];
	if (b2 >= 0x70 && b2 <=	0x77) {
		fprintf(fdis,"LD\t(%s+%02x),%s", ireg, (*this)[p+ 2], reg[b2 & 7]);
		return(3);
	}
	switch (b2) {
	case 0x09:
		fprintf(fdis,"ADD\t%s,BC", ireg);
		len = 2;
		break;
	case 0x19:
		fprintf(fdis,"ADD\t%s,DE", ireg);
		len = 2;
		break;
	case 0x21:
		fprintf(fdis,"LD\t%s,%04x",	ireg, (*this)[p+2]	+ ((*this)[p+3] <<	8));
		len = 4;
		break;
	case 0x22:
		fprintf(fdis,"LD\t(%04x),%s", (*this)[p+2] + ((*this)[p+3] << 8),	ireg);
		len = 4;
		break;
	case 0x23:
		fprintf(fdis,"INC\t%s", ireg);
		len = 2;
		break;
	case 0x29:
		if ((*this)[p]	== 0xdd)
			fprintf(fdis,"ADD\tIX,IX");
		else
			fprintf(fdis,"ADD\tIY,IY");
		len = 2;
		break;
	case 0x2a:
		fprintf(fdis,"LD\t%s,(%04x)", ireg,	(*this)[p+2] + ((*this)[p+3] << 8));
		len = 4;
		break;
	case 0x2b:
		fprintf(fdis,"DEC\t%s", ireg);
		len = 2;
		break;
	case 0x34:
		fprintf(fdis,"INC\t(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x35:
		fprintf(fdis,"DEC\t(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x36:
		fprintf(fdis,"LD\t(%s+%02x),%02x", ireg, (*this)[p+2],	(*this)[p+3]);
		len = 4;
		break;
	case 0x39:
		fprintf(fdis,"ADD\t%s,SP", ireg);
		len = 2;
		break;
	case 0x46:
		fprintf(fdis,"LD\tB,(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x4e:
		fprintf(fdis,"LD\tC,(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x56:
		fprintf(fdis,"LD\tD,(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x5e:
		fprintf(fdis,"LD\tE,(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x66:
		fprintf(fdis,"LD\tH,(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x6e:
		fprintf(fdis,"LD\tL,(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x7e:
		fprintf(fdis,"LD\tA,(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x86:
		fprintf(fdis,"ADD\tA,(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x8e:
		fprintf(fdis,"ADC\tA,(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x96:
		fprintf(fdis,"SUB\t(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0x9e:
		fprintf(fdis,"SBC\tA,(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0xa6:
		fprintf(fdis,"AND\t(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0xae:
		fprintf(fdis,"XOR\t(%s+%02x)", ireg, (*this)[p+2]);
		break;
	case 0xb6:
		fprintf(fdis,"OR\t(%s+%02x)", ireg,	(*this)[p+2]);
		break;
	case 0xbe:
		fprintf(fdis,"CP\t(%s+%02x)", ireg,	(*this)[p+2]);
		break;
	case 0xcb:
		switch ((*this)[p+3]) {
		case 0x06:
			fprintf(fdis,"RLC\t(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x0e:
			fprintf(fdis,"RRC\t(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x16:
			fprintf(fdis,"RL\t(%s+%02x)", ireg,	(*this)[p+2]);
			break;
		case 0x1e:
			fprintf(fdis,"RR\t(%s+%02x)", ireg,	(*this)[p+2]);
			break;
		case 0x26:
			fprintf(fdis,"SLA\t(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x2e:
			fprintf(fdis,"SRA\t(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x3e:
			fprintf(fdis,"SRL\t(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x46:
			fprintf(fdis,"BIT\t0,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x4e:
			fprintf(fdis,"BIT\t1,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x56:
			fprintf(fdis,"BIT\t2,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x5e:
			fprintf(fdis,"BIT\t3,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x66:
			fprintf(fdis,"BIT\t4,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x6e:
			fprintf(fdis,"BIT\t5,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x76:
			fprintf(fdis,"BIT\t6,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x7e:
			fprintf(fdis,"BIT\t7,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x86:
			fprintf(fdis,"RES\t0,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x8e:
			fprintf(fdis,"RES\t1,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x96:
			fprintf(fdis,"RES\t2,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0x9e:
			fprintf(fdis,"RES\t3,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xa6:
			fprintf(fdis,"RES\t4,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xae:
			fprintf(fdis,"RES\t5,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xb6:
			fprintf(fdis,"RES\t6,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xbe:
			fprintf(fdis,"RES\t7,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xc6:
			fprintf(fdis,"SET\t0,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xce:
			fprintf(fdis,"SET\t1,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xd6:
			fprintf(fdis,"SET\t2,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xde:
			fprintf(fdis,"SET\t3,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xe6:
			fprintf(fdis,"SET\t4,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xee:
			fprintf(fdis,"SET\t5,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xf6:
			fprintf(fdis,"SET\t6,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		case 0xfe:
			fprintf(fdis,"SET\t7,(%s+%02x)", ireg, (*this)[p+2]);
			break;
		default:
			fprintf(fdis,"%s",unkown);
		}
		len = 4;
		break;
	case 0xe1:
		fprintf(fdis,"POP\t%s", ireg);
		len = 2;
		break;
	case 0xe3:
		fprintf(fdis,"EX\t(SP),%s",	ireg);
		len = 2;
		break;
	case 0xe5:
		fprintf(fdis,"PUSH\t%s", ireg);
		len = 2;
		break;
	case 0xe9:
		fprintf(fdis,"JP\t(%s)", ireg);
		len = 2;
		break;
	case 0xf9:
		fprintf(fdis,"LD\tSP,%s", ireg);
		len = 2;
		break;
	default:
		fprintf(fdis,"%s",unkown);
	}
	return(len);
}

#endif /* WANT_DISASS */
 