/*
 * Z80SIM  -  a	Z80-CPU	simulator
 *
 * Copyright (C) 1987-2006 by Udo Munk
 *
 * History:
 * 28-SEP-87 Development on TARGON/35 with AT&T Unix System V.3
 * 11-JAN-89 Release 1.1
 * 08-FEB-89 Release 1.2
 * 13-MAR-89 Release 1.3
 * 09-FEB-90 Release 1.4 Ported to TARGON/31 M10/30
 * 20-DEC-90 Release 1.5 Ported to COHERENT 3.0
 * 10-JUN-92 Release 1.6 long casting problem solved with COHERENT 3.2
 *			 and some optimization
 * 25-JUN-92 Release 1.7 comments in english
 * 07-OCT-06 Release 1.8 modified to compile on modern POSIX OS's
 * 27-MAR-07 Release 2.0 Ported to CPP by J.Sievers <JSievers@nadhh.Hanse.de> 
 */

/*
 *	Like the function "cpu()" this one emulates multi byte opcodes
 *	starting with 0xcb
 */
#include "stdafx.h"
#include "zilog.h"


int z80::op_cb_handel(void)
{
	register int t;
	t = (this->*op_cb[(*this)[PC++]]) ();		/* execute next opcode */
	return(t);
}


int z80::op_srla(void)		/* SRL A */
{
	(A & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A >>= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srlb(void)		/* SRL B */
{
	(B & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	B >>= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(B & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[B]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srlc(void)		/* SRL C */
{
	(C & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	C >>= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(C) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(C & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[C]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srld(void)		/* SRL D */
{
	(D & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	D >>= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(D) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(D & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[D]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srle(void)		/* SRL E */
{
	(E & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	E >>= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(E) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(E & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[E]) ? (F &= ~P_FLAG) :(F |= P_FLAG);
	return(8);
}

int z80::op_srlh(void)		/* SRL H */
{
	(H & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	H >>= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(H) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[H]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srll(void)		/* SRL L */
{
	(L & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	L >>= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(L) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(L & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[L]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srlhl(void)		/* SRL (HL) */
{
	register WORD p;

	p = (H << 8) + L;
	((*this)[p] & 1) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	(*this)[p] >>= 1;
	F &= ~(H_FLAG |	N_FLAG);
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(15);
}

int z80::op_slaa(void)		/* SLA A */
{
	(A & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	A <<= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_slab(void)		/* SLA B */
{
	(B & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	B <<= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(B & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[B]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_slac(void)		/* SLA C */
{
	(C & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	C <<= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(C) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(C & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[C]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_slad(void)		/* SLA D */
{
	(D & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	D <<= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(D) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(D & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[D]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_slae(void)		/* SLA E */
{
	(E & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	E <<= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(E) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(E & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[E]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_slah(void)		/* SLA H */
{
	(H & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	H <<= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(H) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[H]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_slal(void)		/* SLA L */
{
	(L & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	L <<= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(L) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(L & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[L]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_slahl(void)		/* SLA (HL) */
{
	register WORD p;

	p = (H <<	8) + L;
	((*this)[p] & 128) ? (F	|= C_FLAG) : (F	&= ~C_FLAG);
	(*this)[p] <<= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(*this)[p] ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	(*this)[p] & 128 ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	parrity[(*this)[p]] ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(15);
}

int z80::op_rlra(void)		/* RL A */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(A & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	A <<= 1;
	if (old_c_flag)	A |= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlb(void)			/* RL B */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(B & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	B <<= 1;
	if (old_c_flag)	B |= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(B & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[B]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlc(void)			/* RL C */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(C & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	C <<= 1;
	if (old_c_flag)	C |= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(C) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(C & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[C]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rld(void)			/* RL D */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(D & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	D <<= 1;
	if (old_c_flag)	D |= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(D) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(D & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[D]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rle(void)			/* RL E */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(E & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	E <<= 1;
	if (old_c_flag)	E |= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(E) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(E & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[E]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlh(void)			/* RL H */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(H & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	H <<= 1;
	if (old_c_flag)	H |= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(H) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[H]) ? (F &= ~P_FLAG) :(F |= P_FLAG);
	return(8);
}

int z80::op_rll(void)			/* RL L */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(L & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	L <<= 1;
	if (old_c_flag)	L |= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(L) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(L & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[L]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlhl(void)		/* RL (HL) */
{
	register int old_c_flag;
	register WORD p;

	p = (H <<8) + L;
	old_c_flag = F & C_FLAG;
	(*this)[p & 128] ? (F	|= C_FLAG) : (F	&= ~C_FLAG);
	(*this)[p] <<= 1;
	if (old_c_flag)	(*this)[p] |= 1;
	F &= ~(H_FLAG |	N_FLAG);
	(*this)[p] ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(15);
}

int z80::op_rrra(void)		/* RR A */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(A & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A >>= 1;
	if (old_c_flag)	A |= 128;
	F &= ~(H_FLAG |	N_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrb(void)			/* RR B */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(B & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	B >>= 1;
	if (old_c_flag)	B |= 128;
	F &= ~(H_FLAG |	N_FLAG);
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(B & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[B]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrc(void)			/* RR C */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(C & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	C >>= 1;
	if (old_c_flag)	C |= 128;
	F &= ~(H_FLAG |	N_FLAG);
	(C) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(C & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[C]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrd(void)			/* RR D */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(D & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	D >>= 1;
	if (old_c_flag)	D |= 128;
	F &= ~(H_FLAG |	N_FLAG);
	(D) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(D & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[D]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rre(void)			/* RR E */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(E & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	E >>= 1;
	if (old_c_flag)	E |= 128;
	F &= ~(H_FLAG |	N_FLAG);
	(E) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(E & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[E]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrh(void)			/* RR H */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(H & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	H >>= 1;
	if (old_c_flag)	H |= 128;
	F &= ~(H_FLAG |	N_FLAG);
	(H) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[H]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrl(void)			/* RR L */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(L & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	L >>= 1;
	if (old_c_flag)	L |= 128;
	F &= ~(H_FLAG |	N_FLAG);
	(L) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(L & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[L]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrhl(void)		/* RR (HL) */
{
	register int old_c_flag;
	register WORD p;

	old_c_flag = F & C_FLAG;
	p = (H <<	8) + L;
	((*this)[p] & 1) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	(*this)[p] >>= 1;
	if (old_c_flag)	(*this)[p] |= 128;
	F &= ~(H_FLAG |	N_FLAG);
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(15);
}

int z80::op_rrcra(void)		/* RRC A */
{
	register int i;

	i = A &	1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	A >>= 1;
	if (i) A |= 128;
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrcb(void)		/* RRC B */
{
	register int i;

	i = B &	1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	B >>= 1;
	if (i) B |= 128;
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(B & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[B]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrcc(void)		/* RRC C */
{
	register int i;

	i = C &	1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	C >>= 1;
	if (i) C |= 128;
	(C) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(C & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[C]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrcd(void)		/* RRC D */
{
	register int i;

	i = D &	1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	D >>= 1;
	if (i) D |= 128;
	(D) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(D & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[D]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrce(void)		/* RRC E */
{
	register int i;

	i = E &	1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	E >>= 1;
	if (i) E |= 128;
	(E) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(E & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[E]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrch(void)		/* RRC H */
{
	register int i;

	i = H &	1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	H >>= 1;
	if (i) H |= 128;
	(H) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[H]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrcl(void)		/* RRC L */
{
	register int i;

	i = L &	1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	L >>= 1;
	if (i) L |= 128;
	(L) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(L & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[L]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rrchl(void)		/* RRC (HL) */
{
	register int i;
	register WORD p;

	p = (H <<	8) + L;
	i = (*this)[p] & 1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	(*this)[p] >>= 1;
	if (i) (*this)[p] |= 128;
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(15);
}

int z80::op_rlcra(void)		/* RLC A */
{
	register int i;

	i = A &	128;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	A <<= 1;
	if (i) A |= 1;
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlcb(void)		/* RLC B */
{
	register int i;

	i = B &	128;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	B <<= 1;
	if (i) B |= 1;
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(B & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[B]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlcc(void)		/* RLC C */
{
	register int i;

	i = C &	128;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	C <<= 1;
	if (i) C |= 1;
	(C) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(C & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[C]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlcd(void)		/* RLC D */
{
	register int i;

	i = D &	128;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	D <<= 1;
	if (i) D |= 1;
	(D) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(D & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[D]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlce(void)		/* RLC E */
{
	register int i;

	i = E &	128;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	E <<= 1;
	if (i) E |= 1;
	(E) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(E & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[E]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlch(void)		/* RLC H */
{
	register int i;

	i = H &	128;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	H <<= 1;
	if (i) H |= 1;
	(H) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[H]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlcl(void)		/* RLC L */
{
	register int i;

	i = L &	128;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	L <<= 1;
	if (i) L |= 1;
	(L) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(L & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[L]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_rlchl(void)		/* RLC (HL) */
{
	register int i;
	register WORD p;

	p = (H <<	8) + L;
	i = (*this)[p] & 128;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	(*this)[p] <<= 1;
	if (i) (*this)[p] |= 1;
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(15);
}

int z80::op_sraa(void)		/* SRA A */
{
	register int i;

	i = A &	128;
	(A & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A >>= 1;
	A |= i;
	F &= ~(H_FLAG |	N_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srab(void)		/* SRA B */
{
	register int i;

	i = B &	128;
	(B & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	B >>= 1;
	B |= i;
	F &= ~(H_FLAG |	N_FLAG);
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(B & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[B]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srac(void)		/* SRA C */
{
	register int i;

	i = C &	128;
	(C & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	C >>= 1;
	C |= i;
	F &= ~(H_FLAG |	N_FLAG);
	(C) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(C & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[C]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srad(void)		/* SRA D */
{
	register int i;

	i = D &	128;
	(D & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	D >>= 1;
	D |= i;
	F &= ~(H_FLAG |	N_FLAG);
	(D) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(D & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[D]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srae(void)		/* SRA E */
{
	register int i;

	i = E &	128;
	(E & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	E >>= 1;
	E |= i;
	F &= ~(H_FLAG |	N_FLAG);
	(E) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(E & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[E]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srah(void)		/* SRA H */
{
	register int i;

	i = H &	128;
	(H & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	H >>= 1;
	H |= i;
	F &= ~(H_FLAG |	N_FLAG);
	(H) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[H]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_sral(void)		/* SRA L */
{
	register int i;

	i = L &	128;
	(L & 1)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	L >>= 1;
	L |= i;
	F &= ~(H_FLAG |	N_FLAG);
	(L) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(L & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[L]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(8);
}

int z80::op_srahl(void)		/* SRA (HL) */
{
	register int i;
	register WORD p;

	p = (H <<	8) + L;
	i = (*this)[p] & 128;
	((*this)[p] & 1) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	(*this)[p] >>= 1;
	(*this)[p] |= i;
	F &= ~(H_FLAG |	N_FLAG);
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(15);
}

int z80::op_sb0a(void)		/* SET 0,A */
{
	A |= 1;
	return(8);
}

int z80::op_sb1a(void)		/* SET 1,A */
{
	A |= 2;
	return(8);
}

int z80::op_sb2a(void)		/* SET 2,A */
{
	A |= 4;
	return(8);
}

int z80::op_sb3a(void)		/* SET 3,A */
{
	A |= 8;
	return(8);
}

int z80::op_sb4a(void)		/* SET 4,A */
{
	A |= 16;
	return(8);
}

int z80::op_sb5a(void)		/* SET 5,A */
{
	A |= 32;
	return(8);
}

int z80::op_sb6a(void)		/* SET 6,A */
{
	A |= 64;
	return(8);
}

int z80::op_sb7a(void)		/* SET 7,A */
{
	A |= 128;
	return(8);
}

int z80::op_sb0b(void)		/* SET 0,B */
{
	B |= 1;
	return(8);
}

int z80::op_sb1b(void)		/* SET 1,B */
{
	B |= 2;
	return(8);
}

int z80::op_sb2b(void)		/* SET 2,B */
{
	B |= 4;
	return(8);
}

int z80::op_sb3b(void)		/* SET 3,B */
{
	B |= 8;
	return(8);
}

int z80::op_sb4b(void)		/* SET 4,B */
{
	B |= 16;
	return(8);
}

int z80::op_sb5b(void)		/* SET 5,B */
{
	B |= 32;
	return(8);
}

int z80::op_sb6b(void)		/* SET 6,B */
{
	B |= 64;
	return(8);
}

int z80::op_sb7b(void)		/* SET 7,B */
{
	B |= 128;
	return(8);
}

int z80::op_sb0c(void)		/* SET 0,C */
{
	C |= 1;
	return(8);
}

int z80::op_sb1c(void)		/* SET 1,C */
{
	C |= 2;
	return(8);
}

int z80::op_sb2c(void)		/* SET 2,C */
{
	C |= 4;
	return(8);
}

int z80::op_sb3c(void)		/* SET 3,C */
{
	C |= 8;
	return(8);
}

int z80::op_sb4c(void)		/* SET 4,C */
{
	C |= 16;
	return(8);
}

int z80::op_sb5c(void)		/* SET 5,C */
{
	C |= 32;
	return(8);
}

int z80::op_sb6c(void)		/* SET 6,C */
{
	C |= 64;
	return(8);
}

int z80::op_sb7c(void)		/* SET 7,C */
{
	C |= 128;
	return(8);
}

int z80::op_sb0d(void)		/* SET 0,D */
{
	D |= 1;
	return(8);
}

int z80::op_sb1d(void)		/* SET 1,D */
{
	D |= 2;
	return(8);
}

int z80::op_sb2d(void)		/* SET 2,D */
{
	D |= 4;
	return(8);
}

int z80::op_sb3d(void)		/* SET 3,D */
{
	D |= 8;
	return(8);
}

int z80::op_sb4d(void)		/* SET 4,D */
{
	D |= 16;
	return(8);
}

int z80::op_sb5d(void)		/* SET 5,D */
{
	D |= 32;
	return(8);
}

int z80::op_sb6d(void)		/* SET 6,D */
{
	D |= 64;
	return(8);
}

int z80::op_sb7d(void)		/* SET 7,D */
{
	D |= 128;
	return(8);
}

int z80::op_sb0e(void)		/* SET 0,E */
{
	E |= 1;
	return(8);
}

int z80::op_sb1e(void)		/* SET 1,E */
{
	E |= 2;
	return(8);
}

int z80::op_sb2e(void)		/* SET 2,E */
{
	E |= 4;
	return(8);
}

int z80::op_sb3e(void)		/* SET 3,E */
{
	E |= 8;
	return(8);
}

int z80::op_sb4e(void)		/* SET 4,E */
{
	E |= 16;
	return(8);
}

int z80::op_sb5e(void)		/* SET 5,E */
{
	E |= 32;
	return(8);
}

int z80::op_sb6e(void)		/* SET 6,E */
{
	E |= 64;
	return(8);
}

int z80::op_sb7e(void)		/* SET 7,E */
{
	E |= 128;
	return(8);
}

int z80::op_sb0h(void)		/* SET 0,H */
{
	H |= 1;
	return(8);
}

int z80::op_sb1h(void)		/* SET 1,H */
{
	H |= 2;
	return(8);
}

int z80::op_sb2h(void)		/* SET 2,H */
{
	H |= 4;
	return(8);
}

int z80::op_sb3h(void)		/* SET 3,H */
{
	H |= 8;
	return(8);
}

int z80::op_sb4h(void)		/* SET 4,H */
{
	H |= 16;
	return(8);
}

int z80::op_sb5h(void)		/* SET 5,H */
{
	H |= 32;
	return(8);
}

int z80::op_sb6h(void)		/* SET 6,H */
{
	H |= 64;
	return(8);
}

int z80::op_sb7h(void)		/* SET 7,H */
{
	H |= 128;
	return(8);
}

int z80::op_sb0l(void)		/* SET 0,L */
{
	L |= 1;
	return(8);
}

int z80::op_sb1l(void)		/* SET 1,L */
{
	L |= 2;
	return(8);
}

int z80::op_sb2l(void)		/* SET 2,L */
{
	L |= 4;
	return(8);
}

int z80::op_sb3l(void)		/* SET 3,L */
{
	L |= 8;
	return(8);
}

int z80::op_sb4l(void)		/* SET 4,L */
{
	L |= 16;
	return(8);
}

int z80::op_sb5l(void)		/* SET 5,L */
{
	L |= 32;
	return(8);
}

int z80::op_sb6l(void)		/* SET 6,L */
{
	L |= 64;
	return(8);
}

int z80::op_sb7l(void)		/* SET 7,L */
{
	L |= 128;
	return(8);
}

int z80::op_sb0hl(void)		/* SET 0,(HL) */
{
	(*this)[(H << 8) + L] |= 1;
	return(15);
}

int z80::op_sb1hl(void)		/* SET 1,(HL) */
{
	(*this)[(H << 8) + L] |= 2;
	return(15);
}

int z80::op_sb2hl(void)		/* SET 2,(HL) */
{
	(*this)[(H << 8) + L] |= 4;
	return(15);
}

int z80::op_sb3hl(void)		/* SET 3,(HL) */
{
	(*this)[(H << 8) + L] |= 8;
	return(15);
}

int z80::op_sb4hl(void)		/* SET 4,(HL) */
{
	(*this)[(H << 8) + L] |= 16;
	return(15);
}

int z80::op_sb5hl(void)		/* SET 5,(HL) */
{
	(*this)[(H << 8) + L] |= 32;
	return(15);
}

int z80::op_sb6hl(void)		/* SET 6,(HL) */
{
	(*this)[(H << 8) + L] |= 64;
	return(15);
}

int z80::op_sb7hl(void)		/* SET 7,(HL) */
{
	(*this)[(H << 8) + L] |= 128;
	return(15);
}

int z80::op_rb0a(void)		/* RES 0,A */
{
	A &= ~1;
	return(8);
}

int z80::op_rb1a(void)		/* RES 1,A */
{
	A &= ~2;
	return(8);
}

int z80::op_rb2a(void)		/* RES 2,A */
{
	A &= ~4;
	return(8);
}

int z80::op_rb3a(void)		/* RES 3,A */
{
	A &= ~8;
	return(8);
}

int z80::op_rb4a(void)		/* RES 4,A */
{
	A &= ~16;
	return(8);
}

int z80::op_rb5a(void)		/* RES 5,A */
{
	A &= ~32;
	return(8);
}

int z80::op_rb6a(void)		/* RES 6,A */
{
	A &= ~64;
	return(8);
}

int z80::op_rb7a(void)		/* RES 7,A */
{
	A &= ~128;
	return(8);
}

int z80::op_rb0b(void)		/* RES 0,B */
{
	B &= ~1;
	return(8);
}

int z80::op_rb1b(void)		/* RES 1,B */
{
	B &= ~2;
	return(8);
}

int z80::op_rb2b(void)		/* RES 2,B */
{
	B &= ~4;
	return(8);
}

int z80::op_rb3b(void)		/* RES 3,B */
{
	B &= ~8;
	return(8);
}

int z80::op_rb4b(void)		/* RES 4,B */
{
	B &= ~16;
	return(8);
}

int z80::op_rb5b(void)		/* RES 5,B */
{
	B &= ~32;
	return(8);
}

int z80::op_rb6b(void)		/* RES 6,B */
{
	B &= ~64;
	return(8);
}

int z80::op_rb7b(void)		/* RES 7,B */
{
	B &= ~128;
	return(8);
}

int z80::op_rb0c(void)		/* RES 0,C */
{
	C &= ~1;
	return(8);
}

int z80::op_rb1c(void)		/* RES 1,C */
{
	C &= ~2;
	return(8);
}

int z80::op_rb2c(void)		/* RES 2,C */
{
	C &= ~4;
	return(8);
}

int z80::op_rb3c(void)		/* RES 3,C */
{
	C &= ~8;
	return(8);
}

int z80::op_rb4c(void)		/* RES 4,C */
{
	C &= ~16;
	return(8);
}

int z80::op_rb5c(void)		/* RES 5,C */
{
	C &= ~32;
	return(8);
}

int z80::op_rb6c(void)		/* RES 6,C */
{
	C &= ~64;
	return(8);
}

int z80::op_rb7c(void)		/* RES 7,C */
{
	C &= ~128;
	return(8);
}

int z80::op_rb0d(void)		/* RES 0,D */
{
	D &= ~1;
	return(8);
}

int z80::op_rb1d(void)		/* RES 1,D */
{
	D &= ~2;
	return(8);
}

int z80::op_rb2d(void)		/* RES 2,D */
{
	D &= ~4;
	return(8);
}

int z80::op_rb3d(void)		/* RES 3,D */
{
	D &= ~8;
	return(8);
}

int z80::op_rb4d(void)		/* RES 4,D */
{
	D &= ~16;
	return(8);
}

int z80::op_rb5d(void)		/* RES 5,D */
{
	D &= ~32;
	return(8);
}

int z80::op_rb6d(void)		/* RES 6,D */
{
	D &= ~64;
	return(8);
}

int z80::op_rb7d(void)		/* RES 7,D */
{
	D &= ~128;
	return(8);
}

int z80::op_rb0e(void)		/* RES 0,E */
{
	E &= ~1;
	return(8);
}

int z80::op_rb1e(void)		/* RES 1,E */
{
	E &= ~2;
	return(8);
}

int z80::op_rb2e(void)		/* RES 2,E */
{
	E &= ~4;
	return(8);
}

int z80::op_rb3e(void)		/* RES 3,E */
{
	E &= ~8;
	return(8);
}

int z80::op_rb4e(void)		/* RES 4,E */
{
	E &= ~16;
	return(8);
}

int z80::op_rb5e(void)		/* RES 5,E */
{
	E &= ~32;
	return(8);
}

int z80::op_rb6e(void)		/* RES 6,E */
{
	E &= ~64;
	return(8);
}

int z80::op_rb7e(void)		/* RES 7,E */
{
	E &= ~128;
	return(8);
}

int z80::op_rb0h(void)		/* RES 0,H */
{
	H &= ~1;
	return(8);
}

int z80::op_rb1h(void)		/* RES 1,H */
{
	H &= ~2;
	return(8);
}

int z80::op_rb2h(void)		/* RES 2,H */
{
	H &= ~4;
	return(8);
}

int z80::op_rb3h(void)		/* RES 3,H */
{
	H &= ~8;
	return(8);
}

int z80::op_rb4h(void)		/* RES 4,H */
{
	H &= ~16;
	return(8);
}

int z80::op_rb5h(void)		/* RES 5,H */
{
	H &= ~32;
	return(8);
}

int z80::op_rb6h(void)		/* RES 6,H */
{
	H &= ~64;
	return(8);
}

int z80::op_rb7h(void)		/* RES 7,H */
{
	H &= ~128;
	return(8);
}

int z80::op_rb0l(void)		/* RES 0,L */
{
	L &= ~1;
	return(8);
}

int z80::op_rb1l(void)		/* RES 1,L */
{
	L &= ~2;
	return(8);
}

int z80::op_rb2l(void)		/* RES 2,L */
{
	L &= ~4;
	return(8);
}

int z80::op_rb3l(void)		/* RES 3,L */
{
	L &= ~8;
	return(8);
}

int z80::op_rb4l(void)		/* RES 4,L */
{
	L &= ~16;
	return(8);
}

int z80::op_rb5l(void)		/* RES 5,L */
{
	L &= ~32;
	return(8);
}

int z80::op_rb6l(void)		/* RES 6,L */
{
	L &= ~64;
	return(8);
}

int z80::op_rb7l(void)		/* RES 7,L */
{
	L &= ~128;
	return(8);
}

int z80::op_rb0hl(void)		/* RES 0,(HL) */
{
	(*this)[(H << 8) + L] &= ~1;
	return(15);
}

int z80::op_rb1hl(void)		/* RES 1,(HL) */
{
	(*this)[(H << 8) + L] &= ~2;
	return(15);
}

int z80::op_rb2hl(void)		/* RES 2,(HL) */
{
	(*this)[(H << 8) + L] &= ~4;
	return(15);
}

int z80::op_rb3hl(void)		/* RES 3,(HL) */
{
	(*this)[(H << 8) + L] &= ~8;
	return(15);
}

int z80::op_rb4hl(void)		/* RES 4,(HL) */
{
	(*this)[(H << 8) + L] &= ~16;
	return(15);
}

int z80::op_rb5hl(void)		/* RES 5,(HL) */
{
	(*this)[(H << 8) + L] &= ~32;
	return(15);
}

int z80::op_rb6hl(void)		/* RES 6,(HL) */
{
	(*this)[(H << 8) + L] &= ~64;
	return(15);
}

int z80::op_rb7hl(void)		/* RES 7,(HL) */
{
	(*this)[(H << 8) + L] &= ~128;
	return(15);
}

int z80::op_tb0a(void)		/* BIT 0,A */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(A & 1)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb1a(void)		/* BIT 1,A */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(A & 2)	? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb2a(void)		/* BIT 2,A */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(A & 4)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb3a(void)		/* BIT 3,A */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(A & 8)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb4a(void)		/* BIT 4,A */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(A & 16) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb5a(void)		/* BIT 5,A */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(A & 32) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb6a(void)		/* BIT 6,A */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(A & 64) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb7a(void)		/* BIT 7,A */
{
	F &= ~N_FLAG;
	F |= H_FLAG;
	if (A & 128) {
		F &= ~(Z_FLAG | P_FLAG);
		F |= S_FLAG;
	} else {
		F |= (Z_FLAG | P_FLAG);
		F &= ~S_FLAG;
	}
	return(8);
}

int z80::op_tb0b(void)		/* BIT 0,B */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(B & 1)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb1b(void)		/* BIT 1,B */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(B & 2)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb2b(void)		/* BIT 2,B */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(B & 4)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb3b(void)		/* BIT 3,B */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(B & 8)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb4b(void)		/* BIT 4,B */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(B & 16) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb5b(void)		/* BIT 5,B */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(B & 32) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb6b(void)		/* BIT 6,B */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(B & 64) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb7b(void)		/* BIT 7,B */
{
	F &= ~N_FLAG;
	F |= H_FLAG;
	if (B & 128) {
		F &= ~(Z_FLAG | P_FLAG);
		F |= S_FLAG;
	} else {
		F |= (Z_FLAG | P_FLAG);
		F &= ~S_FLAG;
	}
	return(8);
}

int z80::op_tb0c(void)		/* BIT 0,C */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(C & 1)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb1c(void)		/* BIT 1,C */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(C & 2)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb2c(void)		/* BIT 2,C */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(C & 4)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb3c(void)		/* BIT 3,C */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(C & 8)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb4c(void)		/* BIT 4,C */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(C & 16) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb5c(void)		/* BIT 5,C */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(C & 32) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb6c(void)		/* BIT 6,C */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(C & 64) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb7c(void)		/* BIT 7,C */
{
	F &= ~N_FLAG;
	F |= H_FLAG;
	if (C & 128) {
		F &= ~(Z_FLAG | P_FLAG);
		F |= S_FLAG;
	} else {
		F |= (Z_FLAG | P_FLAG);
		F &= ~S_FLAG;
	}
	return(8);
}

int z80::op_tb0d(void)		/* BIT 0,D */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(D & 1)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb1d(void)		/* BIT 1,D */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(D & 2)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb2d(void)		/* BIT 2,D */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(D & 4)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb3d(void)		/* BIT 3,D */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(D & 8)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb4d(void)		/* BIT 4,D */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(D & 16) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb5d(void)		/* BIT 5,D */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(D & 32) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb6d(void)		/* BIT 6,D */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(D & 64) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb7d(void)		/* BIT 7,D */
{
	F &= ~N_FLAG;
	F |= H_FLAG;
	if (D & 128) {
		F &= ~(Z_FLAG | P_FLAG);
		F |= S_FLAG;
	} else {
		F |= (Z_FLAG | P_FLAG);
		F &= ~S_FLAG;
	}
	return(8);
}

int z80::op_tb0e(void)		/* BIT 0,E */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(E & 1)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb1e(void)		/* BIT 1,E */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(E & 2)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb2e(void)		/* BIT 2,E */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(E & 4)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb3e(void)		/* BIT 3,E */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(E & 8)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb4e(void)		/* BIT 4,E */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(E & 16) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb5e(void)		/* BIT 5,E */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(E & 32) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb6e(void)		/* BIT 6,E */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(E & 64) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb7e(void)		/* BIT 7,E */
{
	F &= ~N_FLAG;
	F |= H_FLAG;
	if (E & 128) {
		F &= ~(Z_FLAG | P_FLAG);
		F |= S_FLAG;
	} else {
		F |= (Z_FLAG | P_FLAG);
		F &= ~S_FLAG;
	}
	return(8);
}

int z80::op_tb0h(void)		/* BIT 0,H */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(H & 1)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb1h(void)		/* BIT 1,H */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(H & 2)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb2h(void)		/* BIT 2,H */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(H & 4)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb3h(void)		/* BIT 3,H */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(H & 8)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb4h(void)		/* BIT 4,H */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(H & 16) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb5h(void)		/* BIT 5,H */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(H & 32) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb6h(void)		/* BIT 6,H */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(H & 64) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb7h(void)		/* BIT 7,H */
{
	F &= ~N_FLAG;
	F |= H_FLAG;
	if (H & 128) {
		F &= ~(Z_FLAG | P_FLAG);
		F |= S_FLAG;
	} else {
		F |= (Z_FLAG | P_FLAG);
		F &= ~S_FLAG;
	}
	return(8);
}

int z80::op_tb0l(void)		/* BIT 0,L */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(L & 1)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb1l(void)		/* BIT 1,L */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(L & 2)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb2l(void)		/* BIT 2,L */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(L & 4)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb3l(void)		/* BIT 3,L */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(L & 8)	? (F &=	~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb4l(void)		/* BIT 4,L */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(L & 16) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb5l(void)		/* BIT 5,L */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(L & 32) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb6l(void)		/* BIT 6,L */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(L & 64) ? (F &= ~(Z_FLAG | P_FLAG)) : (F |= (Z_FLAG | P_FLAG));
	return(8);
}

int z80::op_tb7l(void)		/* BIT 7,L */
{
	F &= ~N_FLAG;
	F |= H_FLAG;
	if (L & 128) {
		F &= ~(Z_FLAG | P_FLAG);
		F |= S_FLAG;
	} else {
		F |= (Z_FLAG | P_FLAG);
		F &= ~S_FLAG;
	}
	return(8);
}

int z80::op_tb0hl(void)		/* BIT 0,(HL) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(*this)[(H << 8) + L] & 1 ? (F &= ~(Z_FLAG | P_FLAG))
				    : (F |= (Z_FLAG | P_FLAG));
	return(12);
}

int z80::op_tb1hl(void)		/* BIT 1,(HL) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(*this)[(H << 8) + L] & 2 ? (F &= ~(Z_FLAG | P_FLAG))
				    : (F |= (Z_FLAG | P_FLAG));
	return(12);
}

int z80::op_tb2hl(void)		/* BIT 2,(HL) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(*this)[(H << 8) + L] & 4 ? (F &= ~(Z_FLAG | P_FLAG))
				    : (F |= (Z_FLAG | P_FLAG));
	return(12);
}

int z80::op_tb3hl(void)		/* BIT 3,(HL) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(*this)[(H << 8) + L] & 8 ? (F &= ~(Z_FLAG | P_FLAG))
				    : (F |= (Z_FLAG | P_FLAG));
	return(12);
}

int z80::op_tb4hl(void)		/* BIT 4,(HL) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(*this)[(H << 8) + L] & 16 ? (F &= ~(Z_FLAG | P_FLAG))
				     : (F |= (Z_FLAG | P_FLAG));
	return(12);
}

int z80::op_tb5hl(void)		/* BIT 5,(HL) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(*this)[(H << 8) + L] & 32 ? (F &= ~(Z_FLAG | P_FLAG))
				     : (F |= (Z_FLAG | P_FLAG));
	return(12);
}

int z80::op_tb6hl(void)		/* BIT 6,(HL) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	(*this)[(H << 8) + L] & 64 ? (F &= ~(Z_FLAG | P_FLAG))
				     : (F |= (Z_FLAG | P_FLAG));
	return(12);
}

int z80::op_tb7hl(void)		/* BIT 7,(HL) */
{
	F &= ~N_FLAG;
	F |= H_FLAG;
	if ((*this)[(H << 8) + L] & 128) {
		F &= ~(Z_FLAG | P_FLAG);
		F |= S_FLAG;
	} else {
		F |= (Z_FLAG | P_FLAG);
		F &= ~S_FLAG;
	}
	return(12);
}
