/*
 * Z80SIM  -  a	Z80-CPU	simulator
 *
 * Copyright (C) 1987-2006 by Udo Munk
 *
 * History:
 * 28-SEP-87 Development on TARGON/35 with AT&T Unix System V.3
 * 11-JAN-89 Release 1.1
 * 08-FEB-89 Release 1.2
 * 13-MAR-89 Release 1.3
 * 09-FEB-90 Release 1.4 Ported to TARGON/31 M10/30
 * 20-DEC-90 Release 1.5 Ported to COHERENT 3.0
 * 10-JUN-92 Release 1.6 long casting problem solved with COHERENT 3.2
 *			 and some optimization
 * 25-JUN-92 Release 1.7 comments in english
 * 07-OCT-06 Release 1.8 modified to compile on modern POSIX OS's
 * 27-MAR-07 Release 2.0 Ported to CPP by J.Sievers <JSievers@nadhh.Hanse.de> 
 */

/*
 *	Like the function "cpu()" this one emulates multi byte opcodes
 *	starting with 0xdd
 */
#include "stdafx.h"
#include "zilog.h"


int z80::op_dd_handel(void)
{
	register int t;

	t = (this->*op_dd[(*this)[PC++]]) ();		/* execute next opcode */

	return(t);
}


int z80::op_popix(void)		/* POP IX */
{
	IX = (*this)[STACK++];
	IX += (*this)[STACK++] << 8;
	return(14);
}

int z80::op_pusix(void)		/* PUSH IX */
{
	(*this)[--STACK] = IX >> 8;
	(*this)[--STACK] = (BYTE)IX;
	return(15);
}

int z80::op_jpix(void)		/* JP (IX)	*/
{
	PC = IX;
	return(8);
}

int z80::op_exspx(void)		/* EX (SP),IX */
{
	register int i;

	i = (*this)[STACK] + ((*this)[STACK +1] << 8);
	(*this)[STACK] = (BYTE)IX;
	(*this)[STACK+ 1] = IX >> 8;
	IX = i;
	return(23);
}

int z80::op_ldspx(void)		/* LD SP,IX */
{
	STACK =	IX;
	return(10);
}

int z80::op_ldixnn(void)		/* LD IX,nn */
{
	IX = (*this)[PC++];
	IX += (*this)[PC++] << 8;
	return(14);
}

int z80::op_ldixinn(void)		/* LD IX,(nn) */
{
	register WORD p;

	p = (*this)[PC++];
	p += (*this)[PC++] << 8;
	IX = (*this)[p++];
	IX += (*this)[p++] << 8;
	return(20);
}

int z80::op_ldinx(void)		/* LD (nn),IX */
{
	register WORD p;

	p = (*this)[PC++];
	p += (*this)[PC++] << 8;
	(*this)[p++] = (BYTE)IX;
	(*this)[p++] = IX	>> 8;
	return(20);
}

int z80::op_adaxd(void)		/* ADD A,(IX+d) */
{
	register int i;
	register BYTE P;

	P = (*this)[IX + (char)(*this)[PC++]];
	((A & 0xf) + (P	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + P > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(19);
}

int z80::op_acaxd(void)		/* ADC A,(IX+d) */
{
	register int i,	carry;
	register BYTE P;

	carry =	(F & C_FLAG) ? 1 : 0;
	P = (*this)[IX + (char)	(*this)[PC++]];
	((A & 0xf) + (P	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + P + carry > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) P + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(19);
}

int z80::op_suaxd(void)		/* SUB A,(IX+d) */
{
	register int i;
	register BYTE P;

	P = (*this)[IX + (char)	(*this)[PC++]];
	((P & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(19);
}

int z80::op_scaxd(void)		/* SBC A,(IX+d) */
{
	register int i,	carry;
	register BYTE P;

	carry =	(F & C_FLAG) ? 1 : 0;
	P = (*this)[IX + (char)	(*this)[PC++]];
	((P & 0xf) + carry > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P + carry > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) P - carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(19);
}

int z80::op_andxd(void)		/* AND (IX+d) */
{
	A &= (*this)[IX	+ (char) (*this)[PC++]];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(19);
}

int z80::op_xorxd(void)		/* XOR (IX+d) */
{
	A ^= (*this)[IX	+ (char) (*this)[PC++]];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(19);
}

int z80::op_orxd(void)		/* OR (IX+d) */
{
	A |= (*this)[IX	+ (char) (*this)[PC++]];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(19);
}

int z80::op_cpxd(void)		/* CP (IX+d) */
{
	register int i;
	register BYTE P;

	P = (*this)[IX + (char)	(*this)[PC++]];
	((P & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	i = (char) A - (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(19);
}

int z80::op_incxd(void)		/* INC (IX+d) */
{
	register WORD p;

	p = IX + (char) (*this)[PC++];
	(((*this)[p++] & 0xf) + 1	> 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	((*this)[p++])++;
	((*this)[p++] == 128) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	((*this)[p++] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	((*this)[p++]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	F &= ~N_FLAG;
	return(23);
}

int z80::op_decxd(void)		/* DEC (IX+d) */
{
	register WORD p;

	p = IX + (char) (*this)[PC++];
	((((*this)[p++] - 1) & 0xf)	== 0xf)	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	((*this)[p++])--;
	((*this)[p++] == 127) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	((*this)[p++] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	((*this)[p++]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	F |= N_FLAG;
	return(23);
}

int z80::op_addxb(void)		/* ADD IX,BC */
{
	register int carry;
	BYTE ixl = IX & 0xff;
	BYTE ixh = IX >> 8;
	
	carry = (ixl + C > 255) ? 1 : 0;
	ixl += C;
	((ixh & 0xf) + (B & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(ixh + B + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	ixh += B + carry;
	IX = (ixh << 8) + ixl;
	F &= ~N_FLAG;
	return(15);
}

int z80::op_addxd(void)		/* ADD IX,DE */
{
	register int carry;
	BYTE ixl = IX & 0xff;
	BYTE ixh = IX >> 8;
	
	carry = (ixl + E > 255) ? 1 : 0;
	ixl += E;
	((ixh & 0xf) + (D & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(ixh + D + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	ixh += D + carry;
	IX = (ixh << 8) + ixl;
	F &= ~N_FLAG;
	return(15);
}

int z80::op_addxs(void)		/* ADD IX,SP */
{
	register int carry;
	BYTE ixl = IX & 0xff;
	BYTE ixh = IX >> 8;
	BYTE spl = STACK & 0xff;
	BYTE sph = STACK >> 8;
	
	carry = (ixl + spl > 255) ? 1 : 0;
	ixl += spl;
	((ixh & 0xf) + (sph & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(ixh + sph + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	ixh += sph + carry;
	IX = (ixh << 8) + ixl;
	F &= ~N_FLAG;
	return(15);
}

int z80::op_addxx(void)		/* ADD IX,IX */
{
	register int carry;
	BYTE ixl = IX & 0xff;
	BYTE ixh = IX >> 8;
	
	carry = (ixl << 1 > 255) ? 1 : 0;
	ixl <<= 1;
	((ixh & 0xf) + (ixh & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(ixh + ixh + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	ixh += ixh + carry;
	IX = (ixh << 8) + ixl;
	F &= ~N_FLAG;
	return(15);
}

int z80::op_incix(void)		/* INC IX */
{
	IX++;
	return(10);
}

int z80::op_decix(void)		/* DEC IX */
{
	IX--;
	return(10);
}

int z80::op_ldaxd(void)		/* LD A,(IX+d) */
{
	A = (*this)[IX + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldbxd(void)		/* LD B,(IX+d) */
{
	B = (*this)[IX + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldcxd(void)		/* LD C,(IX+d) */
{
	C = (*this)[IX + (char) (*this)[PC++]];
	return(19);
}

int z80::op_lddxd(void)		/* LD D,(IX+d) */
{
	D = (*this)[IX + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldexd(void)		/* LD E,(IX+d) */
{
	E = (*this)[IX + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldhxd(void)		/* LD H,(IX+d) */
{
	H = (*this)[IX + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldlxd(void)		/* LD L,(IX+d) */
{
	L = (*this)[IX + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldxda(void)		/* LD (IX+d),A */
{
	(*this)[IX + (char) (*this)[PC++]] = A;
	return(19);
}

int z80::op_ldxdb(void)		/* LD (IX+d),B */
{
	(*this)[IX + (char) (*this)[PC++]] = B;
	return(19);
}

int z80::op_ldxdc(void)		/* LD (IX+d),C */
{
	(*this)[IX + (char) (*this)[PC++]] = C;
	return(19);
}

int z80::op_ldxdd(void)		/* LD (IX+d),D */
{
	(*this)[IX + (char) (*this)[PC++]] = D;
	return(19);
}

int z80::op_ldxde(void)		/* LD (IX+d),E */
{
	(*this)[IX + (char) (*this)[PC++]] = E;
	return(19);
}

int z80::op_ldxdh(void)		/* LD (IX+d),H */
{
	(*this)[IX + (char) (*this)[PC++]] = H;
	return(19);
}

int z80::op_ldxdl(void)		/* LD (IX+d),L */
{
	(*this)[IX + (char) (*this)[PC++]] = L;
	return(19);
}

int z80::op_ldxdn(void)		/* LD (IX+d),n */
{
	register int d;

	d = (char) (*this)[PC++];
	(*this)[IX + d ] = (*this)[PC++];
	return(19);
}
