/*
 * Z80SIM  -  a	Z80-CPU	simulator
 *
 * Copyright (C) 1987-2006 by Udo Munk
 *
 * History:
 * 28-SEP-87 Development on TARGON/35 with AT&T Unix System V.3
 * 11-JAN-89 Release 1.1
 * 08-FEB-89 Release 1.2
 * 13-MAR-89 Release 1.3
 * 09-FEB-90 Release 1.4 Ported to TARGON/31 M10/30
 * 20-DEC-90 Release 1.5 Ported to COHERENT 3.0
 * 10-JUN-92 Release 1.6 long casting problem solved with COHERENT 3.2
 *			 and some optimization
 * 25-JUN-92 Release 1.7 comments in english
 * 07-OCT-06 Release 1.8 modified to compile on modern POSIX OS's
 * 27-MAR-07 Release 2.0 Ported to CPP by J.Sievers <JSievers@nadhh.Hanse.de> 
 */

/*
 *	Like the function "cpu()" this one emulates multi byte opcodes
 *	starting with 0xed
 */
#include "stdafx.h"
#include "zilog.h"


int z80::op_ed_handel(void)
{
	register int t;

	t = (this->*op_ed[(*this)[PC++]]) ();		/* execute next opcode */
	return(t);
}


int z80::op_im0(void)			/* IM 0 */
{
	int_mode = IM0;
	return(8);
}

int z80::op_im1(void)			/* IM 1 */
{
	int_mode = IM1;
	return(8);
}

int z80::op_im2(void)			/* IM 2 */
{
	int_mode = IM2;
	return(8);
}

int z80::op_reti(void)		/* RETI */
{
	register unsigned i;

	i = (*this)[STACK++];
	i += (*this)[STACK++] << 8;
	PC = i;
	return(14);
}

int z80::op_retn(void)		/* RETN */
{
	register unsigned i;

	i = (*this)[STACK++];
	i += (*this)[STACK++] << 8;
	PC = i;
	IFF >>=1;
	return(14);
}

int z80::op_neg(void)			/* NEG */
{
	(A) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	(A == 0x80) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(0 - ((char) A & 0xf) <	0) ? (F	|= H_FLAG) : (F	&= ~H_FLAG);
	A = 0 -	A;
	F |= N_FLAG;
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(8);
}

int z80::op_inaic(void)		/* IN A,(C) */
{
	A = io_in(C);
	F &= ~(N_FLAG |	H_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(12);
}

int z80::op_inbic(void)		/* IN B,(C) */
{
	B = io_in(C);
	F &= ~(N_FLAG |	H_FLAG);
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(B & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[B]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(12);
}

int z80::op_incic(void)		/* IN C,(C) */
{
	C = io_in(C);
	F &= ~(N_FLAG |	H_FLAG);
	(C) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(C & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[C]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(12);
}

int z80::op_indic(void)		/* IN D,(C) */
{
	D = io_in(C);
	F &= ~(N_FLAG |	H_FLAG);
	(D) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(D & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[D]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(12);
}

int z80::op_ineic(void)		/* IN E,(C) */
{
	E = io_in(C);
	F &= ~(N_FLAG |	H_FLAG);
	(E) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(E & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[E]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(12);
}

int z80::op_inhic(void)		/* IN H,(C) */
{
	H = io_in(C);
	F &= ~(N_FLAG |	H_FLAG);
	(H) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[H]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(12);
}

int z80::op_inlic(void)		/* IN L,(C) */
{
	L = io_in(C);
	F &= ~(N_FLAG |	H_FLAG);
	(L) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(L & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[L]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(12);
}

int z80::op_outca(void)		/* OUT (C),A */
{

	io_out(C, A);
	return(12);
}

int z80::op_outcb(void)		/* OUT (C),B */
{


	io_out(C, B);
	return(12);
}

int z80::op_outcc(void)		/* OUT (C),C */
{


	io_out(C, C);
	return(12);
}

int z80::op_outcd(void)		/* OUT (C),D */
{


	io_out(C, D);
	return(12);
}

int z80::op_outce(void)		/* OUT (C),E */
{


	io_out(C, E);
	return(12);
}

int z80::op_outch(void)		/* OUT (C),H */
{


	io_out(C, H);
	return(12);
}

int z80::op_outcl(void)		/* OUT (C),L */
{


	io_out(C, L);
	return(12);
}

int z80::op_ini(void)			/* INI */
{


	(*this)[(H << 8) + L] =	io_in(C);
	L++;
	if (!L)
		H++;
	B--;
	F |= N_FLAG;
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	return(16);
}

int z80::op_inir(void)		/* INIR */
{
	register int t	= -21;
	register WORD d;


	d = (H << 8) + L;
	do {
		(*this)[d++] = io_in(C);
		B--;
		t += 21;
	} while	(B);
	H = d >> 8;
	L = (BYTE)d ;
	F |= N_FLAG | Z_FLAG;
	return(t + 16);
}

int z80::op_ind(void)			/* IND */
{


	(*this)[(H << 8) + L] =	io_in(C);
	L--;
	if (L == 0xff)
		H--;
	B--;
	F |= N_FLAG;
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	return(16);
}

int z80::op_indr(void)		/* INDR */
{
	register int t	= -21;
	register WORD d;


	d = (H <<	8) + L;
	do {
		(*this)[d--] = io_in(C);
		B--;
		t += 21;
	} while	(B);
	H = d >> 8;
	L = (BYTE)d;
	F |= N_FLAG | Z_FLAG;
	return(t + 16);
}

int z80::op_outi(void)		/* OUTI */
{


	io_out(C, (*this)[(H <<	8) * L]);
	L++;
	if (!L)
		H++;
	B--;
	F |= N_FLAG;
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	return(16);
}

int z80::op_otir(void)		/* OTIR */
{
	register int t	= -21;
	register WORD d;


	d = (H <<	8) + L;
	do {
		io_out(C, (*this)[d++]);
		B--;
		t += 21;
	} while	(B);
	H = d >> 8;
	L = (BYTE)d ;
	F |= N_FLAG | Z_FLAG;
	return(t + 16);
}

int z80::op_outd(void)		/* OUTD */
{


	io_out(C, (*this)[(H <<	8) * L]);
	L--;
	if (L == 0xff)
		H--;
	B--;
	F |= N_FLAG;
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	return(16);
}

int z80::op_otdr(void)		/* OTDR */
{
	register int t	= -21;
	register WORD d;


	d = (H <<	8) + L;
	do {
		io_out(C, (*this)[d--]);
		B--;
		t += 21;
	} while	(B);
	H = d >> 8;
	L = (BYTE)d;
	F |= N_FLAG | Z_FLAG;
	return(t + 16);
}

int z80::op_ldai(void)		/* LD A,I */
{
	A = I;
	F &= ~(N_FLAG |	H_FLAG);
	(IFF & 2) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(9);
}

int z80::op_ldar(void)		/* LD A,R */
{
	A = (BYTE) R;
	F &= ~(N_FLAG |	H_FLAG);
	(IFF & 2) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(9);
}

int z80::op_ldia(void)		/* LD I,A */
{
	I = A;
	return(9);
}

int z80::op_ldra(void)		/* LD R,A */
{
	R = A;
	return(9);
}

int z80::op_ldbcinn(void)		/* LD BC,(nn) */
{
	register WORD p;

	p = (*this)[PC++];
	p += (*this)[PC++] << 8;
	C = (*this)[p++];
	B = (*this)[p];
	return(20);
}

int z80::op_lddeinn(void)		/* LD DE,(nn) */
{
	register WORD p;

	p = (*this)[PC++];
	p += (*this)[PC++] << 8;
	E = (*this)[p++];
	D = (*this)[p];
	return(20);
}

int z80::op_ldspinn(void)		/* LD SP,(nn) */
{
	register WORD p;

	p = (*this)[PC++];
	p += (*this)[PC++] << 8;
	STACK =	(*this)[p++];
	STACK += (*this)[p] << 8;
	return(20);
}

int z80::op_ldinbc(void)		/* LD (nn),BC */
{
	register WORD p;

	p = (*this)[PC++];
	p += (*this)[PC++] << 8;
	(*this)[p++] = C;
	(*this)[p] = B;
	return(20);
}

int z80::op_ldinde(void)		/* LD (nn),DE */
{
	register WORD p;

	p = (*this)[PC++];
	p += (*this)[PC++] << 8;
	(*this)[p++] = E;
	(*this)[p] = D;
	return(20);
}

int z80::op_ldinsp(void)		/* LD (nn),SP */
{
	register WORD p;
	register int i;

	p = (*this)[PC++];
	p += (*this)[PC++] << 8;
	i = STACK ;
	(*this)[p++] = i;
	(*this)[p] = i >> 8;
	return(20);
}

int z80::op_adchb(void)		/* ADC HL,BC */
{
	register int carry;
	int lcarry;
	register WORD hl, bc;
	register long i;

	carry =	(F & C_FLAG) ? 1 : 0;
	lcarry = (L + C > 255) ? 1 : 0;
	((H & 0xf) + (B & 0xf) + carry + lcarry > 0xf) ? (F |= H_FLAG)
						       : (F &= ~H_FLAG);
	hl = (H	<< 8) +	L;
	bc = (B	<< 8) +	C;
	i = ((long)hl) + ((long)bc) + carry;
	((hl < 0x8000) && (i > 0x7fffL)) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(i > 0xffffL) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	H = (BYTE)(i >> 8);
	L = (BYTE)i;
	F &= ~N_FLAG;
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(15);
}

int z80::op_adchd(void)		/* ADC HL,DE */
{
	register int carry;
	int lcarry;
	register WORD hl, de;
	register long i;

	carry =	(F & C_FLAG) ? 1 : 0;
	lcarry = (L + E > 255) ? 1 : 0;
	((H & 0xf) + (D & 0xf) + carry + lcarry > 0xf) ? (F |= H_FLAG)
						       : (F &= ~H_FLAG);
	hl = (H	<< 8) +	L;
	de = (D	<< 8) +	E;
	i = ((long)hl) + ((long)de) + carry;
	((hl < 0x8000) && (i > 0x7fffL)) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(i > 0xffffL) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	H = (BYTE)(i >> 8);
	L = (BYTE)i;
	F &= ~N_FLAG;
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(15);
}

int z80::op_adchh(void)		/* ADC HL,HL */
{
	register int carry;
	int lcarry;
	register WORD hl;
	register long i;

	carry =	(F & C_FLAG) ? 1 : 0;
	lcarry = (L + L > 255) ? 1 : 0;
	((H & 0xf) + (H & 0xf) + carry + lcarry > 0xf) ? (F |= H_FLAG)
						       : (F &= ~H_FLAG);
	hl = (H	<< 8) +	L;
	i = ((((long)hl) << 1) + carry);
	((hl < 0x8000) && (i > 0x7fffL)) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(i > 0xffffL) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	H = (BYTE)(i >> 8);
	L = (BYTE)i;
	F &= ~N_FLAG;
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(15);
}

int z80::op_adchs(void)		/* ADC HL,SP */
{
	register int carry;
	int lcarry;
	register WORD hl, sp;
	register long i;

	carry =	(F & C_FLAG) ? 1 : 0;
	hl = (H	<< 8) +	L;
	sp = STACK ;
	lcarry = (L + (sp & 0xff) > 255) ? 1 : 0;
	((H & 0xf) + ((sp >> 8) & 0xf) + carry + lcarry > 0xf) ? (F |= H_FLAG)
							       : (F &= ~H_FLAG);
	i = ((long)hl) + ((long)sp) + carry;
	((hl < 0x8000) && (i > 0x7fffL)) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(i > 0xffffL) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	H = (BYTE)(i >> 8);
	L = (BYTE)i;
	F &= ~N_FLAG;
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(15);
}

int z80::op_sbchb(void)		/* SBC HL,BC */
{
	register int carry;
	int lcarry;
	register WORD hl, bc;
	register long i;

	carry =	(F & C_FLAG) ? 1 : 0;
	lcarry = (C > L) ? 1 : 0;
	((B & 0xf) + carry + lcarry > (H & 0xf)) ? (F |= H_FLAG)
						 : (F &= ~H_FLAG);
	hl = (H	<< 8) +	L;
	bc = (B	<< 8) +	C;
	i = ((long)hl) - ((long)bc) - carry;
	((hl > 0x7fff) && (i < 0x8000L)) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(i < 0L) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	H = (BYTE)(i >> 8);
	L = (BYTE)i;
	F |= N_FLAG;
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(15);
}

int z80::op_sbchd(void)		/* SBC HL,DE */
{
	register int carry;
	int lcarry;
	register WORD hl, de;
	register long i;

	carry =	(F & C_FLAG) ? 1 : 0;
	lcarry = (E > L) ? 1 : 0;
	((D & 0xf) + carry + lcarry > (H & 0xf)) ? (F |= H_FLAG)
						 : (F &= ~H_FLAG);
	hl = (H	<< 8) +	L;
	de = (D	<< 8) +	E;
	i = ((long)hl) - ((long)de) - carry;
	((hl > 0x7fff) && (i < 0x8000L)) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(i < 0L) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	H = (BYTE)(i >> 8);
	L = (BYTE)i;
	F |= N_FLAG;
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(15);
}

int z80::op_sbchh(void)		/* SBC HL,HL */
{
	if (F &	C_FLAG)	{
		F |= S_FLAG | P_FLAG | N_FLAG |	C_FLAG | H_FLAG;
		F &= ~Z_FLAG;
		H = L =	255;
	} else {
		F |= Z_FLAG | N_FLAG;
		F &= ~(S_FLAG |	P_FLAG | C_FLAG | H_FLAG);
		H = L =	0;
	}
	return(15);
}

int z80::op_sbchs(void)		/* SBC HL,SP */
{
	register int carry;
	int lcarry;
	register WORD hl, sp;
	register long i;

	carry =	(F & C_FLAG) ? 1 : 0;
	hl = (H	<< 8) +	L;
	sp = STACK;
	lcarry = ((sp & 0xff) > L) ? 1 : 0;
	(((sp >> 8) & 0xf) + carry + lcarry > (H & 0xf)) ? (F |= H_FLAG)
							 : (F &= ~H_FLAG);
	i = ((long)hl) - ((long)sp) - carry;
	((hl > 0x7fff) && (i < 0x8000L)) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(i < 0L) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	H = (BYTE)(i >> 8);
	L = (BYTE)i;
	F |= N_FLAG;
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(15);
}

int z80::op_ldi(void)			/* LDI */
{
	(*this)[(D << 8) + E] =	(*this)[(H << 8) + L];
	E++;
	if (!E)
		D++;
	L++;
	if (!L)
		H++;
	C--;
	if (C == 0xff)
		B--;
	(B | C)	? (F |=	P_FLAG)	: (F &=	~P_FLAG);
	F &= ~(N_FLAG |	H_FLAG);
	return(16);
}

int z80::op_ldir(void)		/* LDIR */
{
	register int t	= -21;
	register WORD i;
	register WORD s, d;

	i = (B << 8) + C;
	d = (D << 8) + E;
	s = (H << 8) + L;
	do {
		(*this)[d++] = (*this)[s++];
		t += 21;
	} while	(--i);
	B = C =	0;
	D = d >> 8;
	E = (BYTE)d;
	H = s >> 8;
	L = (BYTE)s;
	F &= ~(N_FLAG |	P_FLAG | H_FLAG);
	return(t + 16);
}

int z80::op_ldd(void)			/* LDD */
{
	(*this)[(D << 8) + E] =	(*this)[(H << 8) + L];
	E--;
	if (E == 0xff)
		D--;
	L--;
	if (L == 0xff)
		H--;
	C--;
	if (C == 0xff)
		B--;
	(B | C)	? (F |=	P_FLAG)	: (F &=	~P_FLAG);
	F &= ~(N_FLAG |	H_FLAG);
	return(16);
}

int z80::op_lddr(void)		/* LDDR */
{
	register int t	= -21;
	register WORD i;
	register WORD s, d;

	i = (B << 8) + C;
	d = (D <<	8) + E;
	s = (H <<	8) + L;
	do {
		(*this)[d--] = (*this)[s--];
		t += 21;
	} while	(--i);
	B = C =	0;
	D = d >> 8;
	E = (BYTE)d;
	H = s >> 8;
	L = (BYTE)s;
	F &= ~(N_FLAG |	P_FLAG | H_FLAG);
	return(t + 16);
}

int z80::op_cpi(void)		/* CPI */
{
	register BYTE i;

	i = (*this)[(H << 8) + L];
	((i & 0xf) > (A & 0xF)) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	i = A -	i;
	L++;
	if (!L)
		H++;
	C--;
	if (C == 0xff)
		B--;
	F |= N_FLAG;
	(B | C)	? (F |=	P_FLAG)	: (F &=	~P_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(16);
}

int z80::op_cpir(void)	/* CPIR */
				/* H Flag not set!!! */
{
	register int t	= -21;
	register WORD s;
	register BYTE d;
	register WORD i;

	i = (B << 8) + C;
	s = (H <<	8) + L;
	do {
		d = A -	(*this)[s++];
		t += 21;
	} while	(--i &&	d);
	F |= N_FLAG;
	B = i >> 8;
	C = (BYTE)i;
	H = s >> 8;
	L = (BYTE)s ;
	(i) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(d) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(d & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(t + 16);
}

int z80::op_cpdop(void)	/* CPD */
{
	register BYTE i;

	i = (*this)[(H << 8) + L];
	((i & 0xf) > (A & 0xF)) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	i = A -	i;
	L--;
	if (L == 0xff)
		H--;
	C--;
	if (C == 0xff)
		B--;
	F |= N_FLAG;
	(B | C)	? (F |=	P_FLAG)	: (F &=	~P_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(16);
}

int z80::op_cpdr(void)	/* CPDR */
				/* H Flag not set!!! */
{
	register int t	= -21;
	register WORD s;
	register BYTE d;
	register WORD i;

	i = (B << 8) + C;
	s = (H <<	8) + L;
	do {
		d = A -	(*this)[s--];
		t += 21;
	} while	(--i &&	d);
	F |= N_FLAG;
	B = i >> 8;
	C = (BYTE)i;
	H = s >> 8;
	L = (BYTE)s;
	(i) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	(d) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(d & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	return(t + 16);
}

int z80::op_oprld(void)	/* RLD (HL) */
{
	register int i,	j;

	i = (*this)[(H << 8) + L];
	j = A &	0x0f;
	A = (A & 0xf0) | (i >> 4);
	i = (i << 4) | j;
	(*this)[(H << 8) + L] =	i;
	F &= ~(H_FLAG |	N_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(18);
}

int z80::op_oprrd(void)	/* RRD (HL) */
{
	register int i,	j;

	i = (*this)[(H << 8) + L];
	j = A &	0x0f;
	A = (A & 0xf0) | (i & 0x0f);
	i = (i >> 4) | (j << 4);
	(*this)[(H << 8) + L] =	i;
	F &= ~(H_FLAG |	N_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(18);
}
