/*
 * Z80SIM  -  a	Z80-CPU	simulator
 *
 * Copyright (C) 1987-2006 by Udo Munk
 *
 * History:
 * 28-SEP-87 Development on TARGON/35 with AT&T Unix System V.3
 * 11-JAN-89 Release 1.1
 * 08-FEB-89 Release 1.2
 * 13-MAR-89 Release 1.3
 * 09-FEB-90 Release 1.4 Ported to TARGON/31 M10/30
 * 20-DEC-90 Release 1.5 Ported to COHERENT 3.0
 * 10-JUN-92 Release 1.6 long casting problem solved with COHERENT 3.2
 *			 and some optimization
 * 25-JUN-92 Release 1.7 comments in english
 * 07-OCT-06 Release 1.8 modified to compile on modern POSIX OS's
 * 27-MAR-07 Release 2.0 Ported to CPP by J.Sievers <JSievers@nadhh.Hanse.de> 
 */

/*
 *	Like the function "cpu()" this one emulates multi byte opcodes
 *	starting with 0xfd
 */
#include "stdafx.h"
#include "zilog.h"


int z80::op_fd_handel(void)
{
	register int t;

	t = (this->*op_fd[(*this)[PC++]]) ();		/* execute next opcode */
	return(t);
}


int z80::op_popiy(void)		/* POP IY */
{
	IY = (*this)[STACK++];
	IY += (*this)[STACK++] << 8;
	return(14);
}

int z80::op_pusiy(void)		/* PUSH IY */
{
	(*this)[--STACK] = IY >> 8;
	(*this)[--STACK] = (BYTE)IY;
	return(15);
}

int z80::op_jpiy(void)		/* JP (IY) */
{
	PC = IY;
	return(8);
}

int z80::op_exspy(void)		/* EX (SP),IY */
{
	register int i;

	i = (*this)[STACK] + ((*this)[STACK +1] << 8);
	(*this)[STACK] = (BYTE)IY;
	(*this)[STACK + 1] = IY >> 8;
	IY = i;
	return(23);
}

int z80::op_ldspy(void)		/* LD SP,IY */
{
	STACK =	IY;
	return(10);
}

int z80::op_ldiynn(void)		/* LD IY,nn */
{
	IY = (*this)[PC++];
	IY += (*this)[PC++] << 8;
	return(14);
}

int z80::op_ldiyinn(void)		/* LD IY,(nn) */
{
	register WORD p;

	p = (*this)[PC++];
	p += (*this)[PC++] << 8;
	IY = (*this)[p++];
	IY += (*this)[p] << 8;
	return(20);
}

int z80::op_ldiny(void)		/* LD (nn),IY */
{
	register WORD p;

	p = (*this)[PC++];
	p += (*this)[PC++] << 8;
	(*this)[p++] = (BYTE)IY;
	(*this)[p] = IY	>> 8;
	return(20);
}

int z80::op_adayd(void)		/* ADD A,(IY+d) */
{
	register int i;
	register BYTE P;

	P = (*this)[IY + (char)	(*this)[PC++]];
	((A & 0xf) + (P	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + P > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(19);
}

int z80::op_acayd(void)		/* ADC A,(IY+d) */
{
	register int i,	carry;
	register BYTE P;

	carry =	(F & C_FLAG) ? 1 : 0;
	P = (*this)[IY + (char)	(*this)[PC++]];
	((A & 0xf) + (P	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + P + carry > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) P + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(19);
}

int z80::op_suayd(void)		/* SUB A,(IY+d) */
{
	register int i;
	register BYTE P;

	P = (*this)[IY + (char)	(*this)[PC++]];
	((P & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(19);
}

int z80::op_scayd(void)		/* SBC A,(IY+d) */
{
	register int i,	carry;
	register BYTE P;

	carry =	(F & C_FLAG) ? 1 : 0;
	P = (*this)[IY + (char)	(*this)[PC++]];
	((P & 0xf) + carry > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P + carry > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) P - carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(19);
}

int z80::op_andyd(void)		/* AND (IY+d) */
{
	A &= (*this)[IY	+ (char) (*this)[PC++]];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(19);
}

int z80::op_xoryd(void)		/* XOR (IY+d) */
{
	A ^= (*this)[IY	+ (char) (*this)[PC++]];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(19);
}

int z80::op_oryd(void)		/* OR (IY+d) */
{
	A |= (*this)[IY	+ (char) (*this)[PC++]];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(19);
}

int z80::op_cpyd(void)		/* CP (IY+d) */
{
	register int i;
	register BYTE P;

	P = (*this)[IY + (char)	(*this)[PC++]];
	((P & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	i = (char) A - (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(19);
}

int z80::op_incyd(void)		/* INC (IY+d) */
{
	register WORD p;

	p = IY + (char) (*this)[PC++];
	(((*this)[p] & 0xf) + 1	> 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	((*this)[p])++;
	((*this)[p] == 128) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	F &= ~N_FLAG;
	return(23);
}

int z80::op_decyd(void)		/* DEC (IY+d) */
{
	register WORD p;

	p = IY + (char) (*this)[PC++];
	((((*this)[p] - 1) & 0xf) == 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	((*this)[p])--;
	((*this)[p] == 127) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	F |= N_FLAG;
	return(23);
}

int z80::op_addyb(void)		/* ADD IY,BC */
{
	register int carry;
	BYTE iyl = IY & 0xff;
	BYTE iyh = IY >> 8;
	
	carry = (iyl + C > 255) ? 1 : 0;
	iyl += C;
	((iyh & 0xf) + (B & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(iyh + B + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	iyh += B + carry;
	IY = (iyh << 8) + iyl;
	F &= ~N_FLAG;
	return(15);
}

int z80::op_addyd(void)		/* ADD IY,DE */
{
	register int carry;
	BYTE iyl = IY & 0xff;
	BYTE iyh = IY >> 8;
	
	carry = (iyl + E > 255) ? 1 : 0;
	iyl += E;
	((iyh & 0xf) + (D & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(iyh + D + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	iyh += D + carry;
	IY = (iyh << 8) + iyl;
	F &= ~N_FLAG;
	return(15);
}

int z80::op_addys(void)		/* ADD IY,SP */
{
	register int carry;
	BYTE iyl = IY & 0xff;
	BYTE iyh = IY >> 8;
	BYTE spl = STACK & 0xff;
	BYTE sph = STACK >> 8;
	
	carry = (iyl + spl > 255) ? 1 : 0;
	iyl += spl;
	((iyh & 0xf) + (sph & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(iyh + sph + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	iyh += sph + carry;
	IY = (iyh << 8) + iyl;
	F &= ~N_FLAG;
	return(15);
}

int z80::op_addyy(void)		/* ADD IY,IY */
{
	register int carry;
	BYTE iyl = IY & 0xff;
	BYTE iyh = IY >> 8;
	
	carry = (iyl << 1 > 255) ? 1 : 0;
	iyl <<= 1;
	((iyh & 0xf) + (iyh & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(iyh + iyh + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	iyh += iyh + carry;
	IY = (iyh << 8) + iyl;
	F &= ~N_FLAG;
	return(15);
}

int z80::op_inciy(void)		/* INC IY */
{
	IY++;
	return(10);
}

int z80::op_deciy(void)		/* DEC IY */
{
	IY--;
	return(10);
}

int z80::op_ldayd(void)		/* LD A,(IY+d) */
{
	A = (*this)[IY + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldbyd(void)		/* LD B,(IY+d) */
{
	B = (*this)[IY + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldcyd(void)		/* LD C,(IY+d) */
{
	C = (*this)[IY + (char) (*this)[PC++]];
	return(19);
}

int z80::op_lddyd(void)		/* LD D,(IY+d) */
{
	D = (*this)[IY + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldeyd(void)		/* LD E,(IY+d) */
{
	E = (*this)[IY + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldhyd(void)		/* LD H,(IY+d) */
{
	H = (*this)[IY + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldlyd(void)		/* LD L,(IY+d) */
{
	L = (*this)[IY + (char) (*this)[PC++]];
	return(19);
}

int z80::op_ldyda(void)		/* LD (IY+d),A */
{
	(*this)[IY + (char) (*this)[PC++]] = A;
	return(19);
}

int z80::op_ldydb(void)		/* LD (IY+d),B */
{
	(*this)[IY + (char) (*this)[PC++]] = B;
	return(19);
}

int z80::op_ldydc(void)		/* LD (IY+d),C */
{
	(*this)[IY + (char) (*this)[PC++]] = C;
	return(19);
}

int z80::op_ldydd(void)		/* LD (IY+d),D */
{
	(*this)[IY + (char) (*this)[PC++]] = D;
	return(19);
}

int z80::op_ldyde(void)		/* LD (IY+d),E */
{
	(*this)[IY + (char) (*this)[PC++]] = E;
	return(19);
}

int z80::op_ldydh(void)		/* LD (IY+d),H */
{
	(*this)[IY + (char) (*this)[PC++]] = H;
	return(19);
}

int z80::op_ldydl(void)		/* LD (IY+d),L */
{
	(*this)[IY + (char) (*this)[PC++]] = L;
	return(19);
}

int z80::op_ldydn(void)		/* LD (IY+d),n */
{
	register int d;

	d = (char) (*this)[PC++];
	(*this)[IY + d]	= (*this)[PC++];
	return(19);
}
