/*
 * Z80SIM  -  a	Z80-CPU	simulator
 *
 * Copyright (C) 1987-2006 by Udo Munk
 *
 * History:
 * 28-SEP-87 Development on TARGON/35 with AT&T Unix System V.3
 * 11-JAN-89 Release 1.1
 * 08-FEB-89 Release 1.2
 * 13-MAR-89 Release 1.3
 * 09-FEB-90 Release 1.4 Ported to TARGON/31 M10/30
 * 20-DEC-90 Release 1.5 Ported to COHERENT 3.0
 * 10-JUN-92 Release 1.6 long casting problem solved with COHERENT 3.2
 *			 and some optimization
 * 25-JUN-92 Release 1.7 comments in english
 * 07-OCT-06 Release 1.8 modified to compile on modern POSIX OS's
 * 27-MAR-07 Release 2.0 Ported to CPP by J.Sievers <JSievers@nadhh.Hanse.de> 
 */

/*
 *	Like the function "cpu()" this one emulates 4 byte opcodes
 *	starting with 0xdd 0xcb
 */
#include "stdafx.h"
#include "zilog.h"



int z80::op_ddcb_handel(void)
{
	register int d;
	register int t;

	d = (char) (*this)[PC++];

	t = (this->*op_ddcb[(*this)[PC++]]) (d);	/* execute next opcode */

	return(t);
}


int z80::op_tb0ixd(int data)		/* BIT 0,(IX+d) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	((*this)[IX + data] & 1) ? (F &= ~(Z_FLAG | P_FLAG))
				 : (F |= (Z_FLAG | P_FLAG));
	return(20);
}

int z80::op_tb1ixd(int data)		/* BIT 1,(IX+d) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	((*this)[IX + data] & 2) ? (F &= ~(Z_FLAG | P_FLAG))
				 : (F |= (Z_FLAG | P_FLAG));
	return(20);
}

int z80::op_tb2ixd(int data)		/* BIT 2,(IX+d) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	((*this)[IX + data] & 4) ? (F &= ~(Z_FLAG | P_FLAG))
				 : (F |= (Z_FLAG | P_FLAG));
	return(20);
}

int z80::op_tb3ixd(int data)		/* BIT 3,(IX+d) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	((*this)[IX + data] & 8) ? (F &= ~(Z_FLAG | P_FLAG))
				 : (F |= (Z_FLAG | P_FLAG));
	return(20);
}

int z80::op_tb4ixd(int data)		/* BIT 4,(IX+d) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	((*this)[IX + data] & 16) ? (F &= ~(Z_FLAG | P_FLAG))
				  : (F |= (Z_FLAG | P_FLAG));
	return(20);
}

int z80::op_tb5ixd(int data)		/* BIT 5,(IX+d) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	((*this)[IX + data] & 32) ? (F &= ~(Z_FLAG | P_FLAG))
				  : (F |= (Z_FLAG | P_FLAG));
	return(20);
}

int z80::op_tb6ixd(int data)		/* BIT 6,(IX+d) */
{
	F &= ~(N_FLAG | S_FLAG);
	F |= H_FLAG;
	((*this)[IX + data] & 64) ? (F &= ~(Z_FLAG | P_FLAG))
				  : (F |= (Z_FLAG | P_FLAG));
	return(20);
}

int z80::op_tb7ixd(int data)		/* BIT 7,(IX+d) */
{
	F &= ~N_FLAG;
	F |= H_FLAG;
	if ((*this)[IX + data] & 128) {
		F &= ~(Z_FLAG | P_FLAG);
		F |= S_FLAG;
	} else {
		F |= (Z_FLAG | P_FLAG);
		F &= ~S_FLAG;
	}
	return(20);
}

int z80::op_rb0ixd(int data)		/* RES 0,(IX+d) */
{
	(*this)[IX + data] &= ~1;
	return(23);
}

int z80::op_rb1ixd(int data)		/* RES 1,(IX+d) */
{
	(*this)[IX + data] &= ~2;
	return(23);
}

int z80::op_rb2ixd(int data)		/* RES 2,(IX+d) */
{
	(*this)[IX + data] &= ~4;
	return(23);
}

int z80::op_rb3ixd(int data)		/* RES 3,(IX+d) */
{
	(*this)[IX + data] &= ~8;
	return(23);
}

int z80::op_rb4ixd(int data)		/* RES 4,(IX+d) */
{
	(*this)[IX + data] &= ~16;
	return(23);
}

int z80::op_rb5ixd(int data)		/* RES 5,(IX+d) */
{
	(*this)[IX + data] &= ~32;
	return(23);
}

int z80::op_rb6ixd(int data)		/* RES 6,(IX+d) */
{
	(*this)[IX + data] &= ~64;
	return(23);
}

int z80::op_rb7ixd(int data)		/* RES 7,(IX+d) */
{
	(*this)[IX + data] &= ~128;
	return(23);
}

int z80::op_sb0ixd(int data)		/* SET 0,(IX+d) */
{
	(*this)[IX + data] |= 1;
	return(23);
}

int z80::op_sb1ixd(int data)		/* SET 1,(IX+d) */
{
	(*this)[IX + data] |= 2;
	return(23);
}

int z80::op_sb2ixd(int data)		/* SET 2,(IX+d) */
{
	(*this)[IX + data] |= 4;
	return(23);
}

int z80::op_sb3ixd(int data)		/* SET 3,(IX+d) */
{
	(*this)[IX + data] |= 8;
	return(23);
}

int z80::op_sb4ixd(int data)		/* SET 4,(IX+d) */
{
	(*this)[IX + data] |= 16;
	return(23);
}

int z80::op_sb5ixd(int data)		/* SET 5,(IX+d) */
{
	(*this)[IX + data] |= 32;
	return(23);
}

int z80::op_sb6ixd(int data)		/* SET 6,(IX+d) */
{
	(*this)[IX + data] |= 64;
	return(23);
}

int z80::op_sb7ixd(int data)		/* SET 7,(IX+d) */
{
	(*this)[IX + data] |= 128;
	return(23);
}

int z80::op_rlcixd(int data)		/* RLC (IX+d) */
{
	register int i;
	register WORD p;

	p = IX + data;
	i = (*this)[p] & 128;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	(*this)[p] <<= 1;
	if (i) (*this)[p] |= 1;
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(23);
}

int z80::op_rrcixd(int data)		/* RRC (IX+d) */
{
	register int i;
	register WORD p;

	p = IX + data;
	i = (*this)[p] & 1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	(*this)[p] >>= 1;
	if (i) (*this)[p] |= 128;
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(23);
}

int z80::op_rlixd(int data)		/* RL (IX+d) */
{
	register int old_c_flag;
	register WORD p;

	p = IX + data;
	old_c_flag = F & C_FLAG;
	((*this)[p] & 128) ? (F	|= C_FLAG) : (F	&= ~C_FLAG);
	(*this)[p] <<= 1;
	if (old_c_flag)	(*this)[p] |= 1;
	F &= ~(H_FLAG |	N_FLAG);
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(23);
}

int z80::op_rrixd(int data)		/* RR (IX+d) */
{
	register int old_c_flag;
	register WORD p;

	old_c_flag = F & C_FLAG;
	p = IX + data;
	((*this)[p] & 1) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	(*this)[p] >>= 1;
	if (old_c_flag)	(*this)[p] |= 128;
	F &= ~(H_FLAG |	N_FLAG);
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(23);
}

int z80::op_slaixd(int data)		/* SLA (IX+d) */
{
	register WORD p;

	p = IX + data;
	((*this)[p] & 128) ? (F	|= C_FLAG) : (F	&= ~C_FLAG);
	(*this)[p] <<= 1;
	F &= ~(H_FLAG |	N_FLAG);
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(23);
}

int z80::op_sraixd(int data)		/* SRA (IX+d) */
{
	register int i;
	register WORD p;

	p = IX + data;
	i = (*this)[p] & 128;
	((*this)[p] & 1) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	(*this)[p] >>= 1;
	(*this)[p] |= i;
	F &= ~(H_FLAG |	N_FLAG);
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(23);
}

int z80::op_srlixd(int data)		/* SRL (IX+d) */
{
	register WORD p;

	p = IX + data;
	((*this)[p] & 1) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	(*this)[p] >>= 1;
	F &= ~(H_FLAG |	N_FLAG);
	((*this)[p]) ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(parrity[(*this)[p]]) ?	(F &= ~P_FLAG) : (F |= P_FLAG);
	return(23);
}
