#ifndef __ZILOG_H__
#define __ZILOG_H__

/*
 * ZILOG.H  -  a Z80-CPU simulator
 *
 * Copyright (C) 1987-2006 by Udo Munk
 *
 * This modul contains a complex I/O-simulation for running
 * CP/M 2, CP/M 3, MP/M...
 * Please note this this doesn't emulate any hardware which
 * ever existed, we've got all virtual circuits in here!
 *
 * History:
 * 
 */

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the ZILOG_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// ZILOG_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef ZILOG_EXPORTS
#define ZILOG_API __declspec(dllexport)
#else
#define ZILOG_API __declspec(dllimport)
#endif
#include <stdio.h>
#pragma warning (disable: 4996)
#define TELNETCONSOLE

extern BYTE ram[];
extern void write_term(char c);
extern char read_term(void);
extern int kbhit_term();

#ifdef TELNETCONSOLE
#define CONSCHK kbhit_term
#define CONSINP read_term
#define CONSOUT write_term
#else
__inline char term_get()
{
	char c;
	fread(&c,1,1,stdin);
	return c;
}
__inline void term_put(char c)
{
	fwrite(&c,1,1,stdout);
	return;
}
#define CONSCHK _kbhit
#define CONSINP term_get
#define CONSOUT term_put
#endif

/*
 *	The following lines of this file should not be modified by user
 */
#define	COPYR	"Copyright (C) 1987-2006 by Udo Munk"
#define	RELEASE	"1.9"
#define CPPPORT "\tShared z80 CPU port 2007 V 1.9-cpp-1.0 by JSievers@nadhh.Hnase.DE"


// This class is exported from the zilog.dll
class ZILOG_API z80
{
public:
	
	static const char version[];

	z80();
	virtual ~z80(){};

	virtual void run(void);
	virtual void reset(void);
	virtual void nmi(void);
	virtual void intr(void);
	virtual bool busreq(void){ return bus = FALSE;};

#ifdef WANT_DISASS
	WORD disass(WORD pc);
	WORD disass(void) { return disass(PC);}
	FILE*   fdis;
#endif
	typedef enum {			/* type of CPU interrupt */
		INT_NONE    ,
		INT_NMI		,		/* non maskable interrupt */
		INT_INT				/* maskable interrupt */
	} int_type_t;

	typedef enum {			/* type of CPU interrupt */
		IM0 ,
		IM1	,		/* non maskable interrupt */
		IM2			/* maskable interrupt */
	} int_mode_t;

	enum {
 	S_FLAG	=	128,	/* bit definitions of CPU flags */
 	Z_FLAG	=	64,
 	N2_FLAG	=	32,
 	H_FLAG	=	16,
 	N1_FLAG	=	8,
 	P_FLAG	=	4,
 	N_FLAG	=	2,
 	C_FLAG	=	1,
};

/*
 *	CPU-Register
 */
	BYTE A,B,C,D,E,H,L;		/* Z80 primary registers */
	int  F;					/* normaly 8-Bit, but int is faster */
	WORD IX, IY;
	BYTE A_,B_,C_,D_,E_,H_,L_;	/* Z80 secoundary registers */
	int  F_;
	WORD PC;				/* Z80 programm counter */
	WORD STACK;				/* Z80 stackpointer */
	BYTE I;					/* Z80 interrupt register */
	BYTE IFF;				/* Z80 interrupt flags */
	long R;					/* Z80 refresh register */
							/* is normaly a 8 bit register	*/
							/* the 32 bits are used to measure the */
							/* clock frequency */
	void save_core(void);
	void load_core(void);

protected:
	long	t_states;
	bool    hlt;
	bool	bus;

	volatile int_mode_t	int_mode;
	volatile int		int_type;	

	virtual BYTE& operator [] (WORD addr) = 0;	// Interface to Memory
	virtual BYTE io_in(BYTE adr) = 0;			// Interface for port input
	virtual void io_out(BYTE adr, BYTE data) = 0;// Interface for Protn output

	virtual void m1(WORD addr){};
	virtual void halt(WORD pc) {};
	virtual BYTE nmiack(void)=0;
	virtual BYTE intrack(int_mode_t mode)=0;
	virtual BOOL busack(void){return bus = FALSE;};

	// Called after each single step. Must return TRUE to continue running
	virtual BOOL nextstep(void) { return TRUE;}

	typedef int (z80::*optf_t)(void);
	static const optf_t op_sim[256];
	static const optf_t op_cb[256];
	static const optf_t op_dd[256];
	static const optf_t op_ed[256];
	static const optf_t op_fd[256];

	typedef int (z80::*optf1_t)(int);
	static const optf1_t op_ddcb[256];
	static const optf1_t op_fdcb[256];

	int op_cb_handel(void);
	int op_dd_handel(void);
	int op_ed_handel(void);
	int op_fd_handel(void);
	int op_fdcb_handel(void);

	static const int parrity[256];

	// All z80 functions prototypes.
	virtual int op_notimpl(void){ return 0;};
	int op_nop(void);
	int op_halt(void);
	int op_scf(void);
	int op_ccf(void);
	int op_cpl(void);
	int op_daa(void);
	int op_ei(void);
	int op_di(void);
	int op_in(void);
	int op_out(void);
	int op_ldan(void);
	int op_ldbn(void);
	int op_ldcn(void);
	int op_lddn(void);
	int op_lden(void);
	int op_ldhn(void);
	int op_ldln(void);
	int op_ldabc(void);
	int op_ldade(void);
	int op_ldann(void);
	int op_ldbca(void);
	int op_lddea(void);
	int op_ldnna(void);
	int op_ldhla(void);
	int op_ldhlb(void);
	int op_ldhlc(void);
	int op_ldhld(void);
	int op_ldhle(void);
	int op_ldhlh(void);
	int op_ldhll(void);
	int op_ldhl1(void);
	int op_ldaa(void);
	int op_ldab(void);
	int op_ldac(void);
	int op_ldad(void);
	int op_ldae(void);
	int op_ldah(void);
	int op_ldal(void);
	int op_ldahl(void);
	int op_ldba(void);
	int op_ldbb(void);
	int op_ldbc(void);
	int op_ldbd(void);
	int op_ldbe(void);
	int op_ldbh(void);
	int op_ldbl(void);
	int op_ldbhl(void);
	int op_ldca(void);
	int op_ldcb(void);
	int op_ldcc(void);
	int op_ldcd(void);
	int op_ldce(void);
	int op_ldch(void);
	int op_ldcl(void);
	int op_ldchl(void);
	int op_ldda(void);
	int op_lddb(void);
	int op_lddc(void);
	int op_lddd(void);
	int op_ldde(void);
	int op_lddh(void);
	int op_lddl(void);
	int op_lddhl(void);
	int op_ldea(void);
	int op_ldeb(void);
	int op_ldec(void);
	int op_lded(void);
	int op_ldee(void);
	int op_ldeh(void);
	int op_ldel(void);
	int op_ldehl(void);
	int op_ldha(void);
	int op_ldhb(void);
	int op_ldhc(void);
	int op_ldhd(void);
	int op_ldhe(void);
	int op_ldhh(void);
	int op_ldhl(void);
	int op_ldhhl(void);
	int op_ldla(void);
	int op_ldlb(void);
	int op_ldlc(void);
	int op_ldld(void);
	int op_ldle(void);
	int op_ldlh(void);
	int op_ldll(void);
	int op_ldlhl(void);
	int op_ldbcnn(void);
	int op_lddenn(void);
	int op_ldhlnn(void);
	int op_ldspnn(void);
	int op_ldsphl(void);
	int op_ldhlin(void);
	int op_ldinhl(void);
	int op_incbc(void);
	int op_incde(void);
	int op_inchl(void);
	int op_incsp(void);
	int op_decbc(void);
	int op_decde(void);
	int op_dechl(void);
	int op_decsp(void);
	int op_adhlbc(void);
	int op_adhlde(void);
	int op_adhlhl(void);
	int op_adhlsp(void);
	int op_anda(void);
	int op_andb(void);
	int op_andc(void);
	int op_andd(void);
	int op_ande(void);
	int op_andh(void);
	int op_andl(void);
	int op_andhl(void);
	int op_andn(void);
	int op_ora(void);
	int op_orb(void);
	int op_orc(void);
	int op_ord(void);
	int op_ore(void);
	int op_orh(void);
	int op_orl(void);
	int op_orhl(void);
	int op_orn(void);
	int op_xora(void);
	int op_xorb(void);
	int op_xorc(void);
	int op_xord(void);
	int op_xore(void);
	int op_xorh(void);
	int op_xorl(void);
	int op_xorhl(void);
	int op_xorn(void);
	int op_adda(void);
	int op_addb(void);
	int op_addc(void);
	int op_addd(void);
	int op_adde(void);
	int op_addh(void);
	int op_addl(void);
	int op_addhl(void);
	int op_addn(void);
	int op_adca(void);
	int op_adcb(void);
	int op_adcc(void);
	int op_adcd(void);
	int op_adce(void);
	int op_adch(void);
	int op_adcl(void);
	int op_adchl(void);
	int op_adcn(void);
	int op_suba(void);
	int op_subb(void);
	int op_subc(void);
	int op_subd(void);
	int op_sube(void);
	int op_subh(void);
	int op_subl(void);
	int op_subhl(void);
	int op_subn(void);
	int op_sbca(void);
	int op_sbcb(void);
	int op_sbcc(void);
	int op_sbcd(void);
	int op_sbce(void);
	int op_sbch(void);
	int op_sbcl(void);
	int op_sbchl(void);
	int op_sbcn(void);
	int op_cpa(void);
	int op_cpb(void);
	int op_cpc(void);
	int op_cpd(void);
	int op_cpe(void);
	int op_cph(void);
	int op_cplr(void);
	int op_cphl(void);
	int op_cpn(void);
	int op_inca(void);
	int op_incb(void);
	int op_incc(void);
	int op_incd(void);
	int op_ince(void);
	int op_inch(void);
	int op_incl(void);
	int op_incihl(void);
	int op_deca(void);
	int op_decb(void);
	int op_decc(void);
	int op_decd(void);
	int op_dece(void);
	int op_dech(void);
	int op_decl(void);
	int op_decihl(void);
	int op_rlca(void);
	int op_rrca(void);
	int op_rla(void);
	int op_rra(void);
	int op_exdehl(void);
	int op_exafaf(void);
	int op_exx(void);
	int op_exsphl(void);
	int op_pushaf(void);
	int op_pushbc(void);
	int op_pushde(void);
	int op_pushhl(void);
	int op_popaf(void);
	int op_popbc(void);
	int op_popde(void);
	int op_pophl(void);
	int op_jp(void);
	int op_jphl(void);
	int op_jr(void);
	int op_djnz(void);
	int op_call(void);
	int op_ret(void);
	int op_jpz(void);
	int op_jpnz(void);
	int op_jpc(void);
	int op_jpnc(void);
	int op_jppe(void);
	int op_jppo(void);
	int op_jpm(void);
	int op_jpp(void);
	int op_calz(void);
	int op_calnz(void);
	int op_calc(void);
	int op_calnc(void);
	int op_calpe(void);
	int op_calpo(void);
	int op_calm(void);
	int op_calp(void);
	int op_retz(void);
	int op_retnz(void);
	int op_retc(void);
	int op_retnc(void);
	int op_retpe(void);
	int op_retpo(void);
	int op_retm(void);
	int op_retp(void);
	int op_jrz(void);
	int op_jrnz(void);
	int op_jrc(void);
	int op_jrnc(void);
	int op_rst00(void);
	int op_rst08(void);
	int op_rst10(void);
	int op_rst18(void);
	int op_rst20(void);
	int op_rst28(void);
	int op_rst30(void);
	int op_rst38(void);


	virtual int trap_cb(void){ return 0;};
	int op_srla(void);
	int op_srlb(void);
	int op_srlc(void);
	int op_srld(void);
	int op_srle(void);
	int op_srlh(void);
	int op_srll(void);
	int op_srlhl(void);
	int op_slaa(void);
	int op_slab(void);
	int op_slac(void);
	int op_slad(void);
	int op_slae(void);
	int op_slah(void);
	int op_slal(void);
	int op_slahl(void);
	int op_rlra(void);
	int op_rlb(void);
	int op_rlc(void);
	int op_rld(void);
	int op_rle(void);
	int op_rlh(void);
	int op_rll(void);
	int op_rlhl(void);
	int op_rrra(void);
	int op_rrb(void);
	int op_rrc(void);
	int op_rrd(void);
	int op_rre(void);
	int op_rrh(void);
	int op_rrl(void);
	int op_rrhl(void);
	int op_rrcra(void);
	int op_rrcb(void);
	int op_rrcc(void);
	int op_rrcd(void);
	int op_rrce(void);
	int op_rrch(void);
	int op_rrcl(void);
	int op_rrchl(void);
	int op_rlcra(void);
	int op_rlcb(void);
	int op_rlcc(void);
	int op_rlcd(void);
	int op_rlce(void);
	int op_rlch(void);
	int op_rlcl(void);
	int op_rlchl(void);
	int op_sraa(void);
	int op_srab(void);
	int op_srac(void);
	int op_srad(void);
	int op_srae(void);
	int op_srah(void);
	int op_sral(void);
	int op_srahl(void);
	int op_sb0a(void);
	int op_sb1a(void);
	int op_sb2a(void);
	int op_sb3a(void);
	int op_sb4a(void);
	int op_sb5a(void);
	int op_sb6a(void);
	int op_sb7a(void);
	int op_sb0b(void);
	int op_sb1b(void);
	int op_sb2b(void);
	int op_sb3b(void);
	int op_sb4b(void);
	int op_sb5b(void);
	int op_sb6b(void);
	int op_sb7b(void);
	int op_sb0c(void);
	int op_sb1c(void);
	int op_sb2c(void);
	int op_sb3c(void);
	int op_sb4c(void);
	int op_sb5c(void);
	int op_sb6c(void);
	int op_sb7c(void);
	int op_sb0d(void);
	int op_sb1d(void);
	int op_sb2d(void);
	int op_sb3d(void);
	int op_sb4d(void);
	int op_sb5d(void);
	int op_sb6d(void);
	int op_sb7d(void);
	int op_sb0e(void);
	int op_sb1e(void);
	int op_sb2e(void);
	int op_sb3e(void);
	int op_sb4e(void);
	int op_sb5e(void);
	int op_sb6e(void);
	int op_sb7e(void);
	int op_sb0h(void);
	int op_sb1h(void);
	int op_sb2h(void);
	int op_sb3h(void);
	int op_sb4h(void);
	int op_sb5h(void);
	int op_sb6h(void);
	int op_sb7h(void);
	int op_sb0l(void);
	int op_sb1l(void);
	int op_sb2l(void);
	int op_sb3l(void);
	int op_sb4l(void);
	int op_sb5l(void);
	int op_sb6l(void);
	int op_sb7l(void);
	int op_sb0hl(void);
	int op_sb1hl(void);
	int op_sb2hl(void);
	int op_sb3hl(void);
	int op_sb4hl(void);
	int op_sb5hl(void);
	int op_sb6hl(void);
	int op_sb7hl(void);
	int op_rb0a(void);
	int op_rb1a(void);
	int op_rb2a(void);
	int op_rb3a(void);
	int op_rb4a(void);
	int op_rb5a(void);
	int op_rb6a(void);
	int op_rb7a(void);
	int op_rb0b(void);
	int op_rb1b(void);
	int op_rb2b(void);
	int op_rb3b(void);
	int op_rb4b(void);
	int op_rb5b(void);
	int op_rb6b(void);
	int op_rb7b(void);
	int op_rb0c(void);
	int op_rb1c(void);
	int op_rb2c(void);
	int op_rb3c(void);
	int op_rb4c(void);
	int op_rb5c(void);
	int op_rb6c(void);
	int op_rb7c(void);
	int op_rb0d(void);
	int op_rb1d(void);
	int op_rb2d(void);
	int op_rb3d(void);
	int op_rb4d(void);
	int op_rb5d(void);
	int op_rb6d(void);
	int op_rb7d(void);
	int op_rb0e(void);
	int op_rb1e(void);
	int op_rb2e(void);
	int op_rb3e(void);
	int op_rb4e(void);
	int op_rb5e(void);
	int op_rb6e(void);
	int op_rb7e(void);
	int op_rb0h(void);
	int op_rb1h(void);
	int op_rb2h(void);
	int op_rb3h(void);
	int op_rb4h(void);
	int op_rb5h(void);
	int op_rb6h(void);
	int op_rb7h(void);
	int op_rb0l(void);
	int op_rb1l(void);
	int op_rb2l(void);
	int op_rb3l(void);
	int op_rb4l(void);
	int op_rb5l(void);
	int op_rb6l(void);
	int op_rb7l(void);
	int op_rb0hl(void);
	int op_rb1hl(void);
	int op_rb2hl(void);
	int op_rb3hl(void);
	int op_rb4hl(void);
	int op_rb5hl(void);
	int op_rb6hl(void);
	int op_rb7hl(void);
	int op_tb0a(void);
	int op_tb1a(void);
	int op_tb2a(void);
	int op_tb3a(void);
	int op_tb4a(void);
	int op_tb5a(void);
	int op_tb6a(void);
	int op_tb7a(void);
	int op_tb0b(void);
	int op_tb1b(void);
	int op_tb2b(void);
	int op_tb3b(void);
	int op_tb4b(void);
	int op_tb5b(void);
	int op_tb6b(void);
	int op_tb7b(void);
	int op_tb0c(void);
	int op_tb1c(void);
	int op_tb2c(void);
	int op_tb3c(void);
	int op_tb4c(void);
	int op_tb5c(void);
	int op_tb6c(void);
	int op_tb7c(void);
	int op_tb0d(void);
	int op_tb1d(void);
	int op_tb2d(void);
	int op_tb3d(void);
	int op_tb4d(void);
	int op_tb5d(void);
	int op_tb6d(void);
	int op_tb7d(void);
	int op_tb0e(void);
	int op_tb1e(void);
	int op_tb2e(void);
	int op_tb3e(void);
	int op_tb4e(void);
	int op_tb5e(void);
	int op_tb6e(void);
	int op_tb7e(void);
	int op_tb0h(void);
	int op_tb1h(void);
	int op_tb2h(void);
	int op_tb3h(void);
	int op_tb4h(void);
	int op_tb5h(void);
	int op_tb6h(void);
	int op_tb7h(void);
	int op_tb0l(void);
	int op_tb1l(void);
	int op_tb2l(void);
	int op_tb3l(void);
	int op_tb4l(void);
	int op_tb5l(void);
	int op_tb6l(void);
	int op_tb7l(void);
	int op_tb0hl(void);
	int op_tb1hl(void);
	int op_tb2hl(void);
	int op_tb3hl(void);
	int op_tb4hl(void);
	int op_tb5hl(void);
	int op_tb6hl(void);
	int op_tb7hl(void);

	virtual int trap_dd(void){ return 0;};
	int op_popix(void);
	int op_pusix(void);
	int op_jpix(void);
	int op_exspx(void);
	int op_ldspx(void);
	int op_ldixnn(void);
	int op_ldixinn(void);
	int op_ldinx(void);
	int op_adaxd(void);
	int op_acaxd(void);
	int op_suaxd(void);
	int op_scaxd(void);
	int op_andxd(void);
	int op_xorxd(void);
	int op_orxd(void);
	int op_cpxd(void);
	int op_decxd(void);
	int op_incxd(void);
	int op_addxb(void);
	int op_addxd(void);
	int op_addxs(void);
	int op_addxx();
	int op_incix(void);
	int op_decix(void);
	int op_ldaxd(void);
	int op_ldbxd(void);
	int op_ldcxd(void);
	int op_lddxd();
	int op_ldexd(void);
	int op_ldhxd(void);
	int op_ldlxd(void);
	int op_ldxda(void);
	int op_ldxdb(void);
	int op_ldxdc(void);
	int op_ldxdd();
	int op_ldxde(void);
	int op_ldxdh(void);
	int op_ldxdl(void);
	int op_ldxdn(void);
	int op_ddcb_handel(void);

	virtual int trap_ed(void){ return 0;};
	int op_im0(void);
	int op_im1(void);
	int op_im2(void);
	int op_reti(void);	
	int op_retn(void);
	int op_neg(void);
	int op_inaic(void);
	int op_inbic(void);
	int op_incic(void);
	int op_indic(void);
	int op_ineic(void);
	int op_inhic(void);
	int op_inlic(void);
	int op_outca(void);
	int op_outcb(void);
	int op_outcc(void);
	int op_outcd(void);
	int op_outce(void);
	int op_outch(void);
	int op_outcl(void);
	int op_ini(void);
	int op_inir(void);
	int op_ind(void);
	int op_indr(void);
	int op_outi(void);
	int op_otir(void);
	int op_outd(void);
	int op_otdr(void);
	int op_ldai(void);
	int op_ldar(void);
	int op_ldia(void);
	int op_ldra(void);
	int op_ldbcinn(void);
	int op_lddeinn(void);
	int op_ldspinn(void);
	int op_ldinbc(void);
	int op_ldinde(void);
	int op_ldinsp(void);
	int op_adchb(void);
	int op_adchd(void);
	int op_adchh(void);
	int op_adchs(void);
	int op_sbchb(void);
	int op_sbchd(void);
	int op_sbchh(void);
	int op_sbchs(void);
	int op_ldi(void);
	int op_ldir(void);
	int op_ldd(void);
	int op_lddr(void);
	int op_cpi(void);
	int op_cpir(void);
	int op_cpdop(void);
	int op_cpdr(void);
	int op_oprld(void);
	int op_oprrd(void);

	virtual int trap_fd(void){ return 0;};
	int op_popiy(void);
	int op_pusiy(void);
	int op_jpiy(void);
	int op_exspy(void);
	int op_ldspy(void);
	int op_ldiynn(void);
	int op_ldiyinn(void);
	int op_ldiny(void);
	int op_adayd(void);
	int op_acayd(void);
	int op_suayd(void);
	int op_scayd(void);
	int op_addyb(void);
	int op_addyd(void);
	int op_addys(void);
	int op_addyy(void);
	int op_andyd(void);
	int op_xoryd(void);
	int op_oryd(void);
	int op_cpyd(void);
	int op_decyd(void);
	int op_incyd(void);
	int op_inciy(void);
	int op_deciy(void);
	int op_ldayd(void);
	int op_ldbyd(void);
	int op_ldcyd(void);
	int op_lddyd(void);
	int op_ldeyd(void);
	int op_ldhyd(void);
	int op_ldlyd(void);
	int op_ldyda(void);
	int op_ldydb(void);
	int op_ldydc(void);
	int op_ldydd(void);
	int op_ldyde(void);
	int op_ldydh(void);
	int op_ldydl(void);
	int op_ldydn(void);

	virtual int trap_ddcb(int){ return 0;};
	int op_tb0ixd(int);
	int op_tb1ixd(int);
	int op_tb2ixd(int);
	int op_tb3ixd(int);
	int op_tb4ixd(int);
	int op_tb5ixd(int);
	int op_tb6ixd(int);
	int op_tb7ixd(int);
	int op_rb0ixd(int);
	int op_rb1ixd(int);
	int op_rb2ixd(int);
	int op_rb3ixd(int);
	int op_rb4ixd(int);
	int op_rb5ixd(int);
	int op_rb6ixd(int);
	int op_rb7ixd(int);
	int op_sb0ixd(int);
	int op_sb1ixd(int);
	int op_sb2ixd(int);
	int op_sb3ixd(int);
	int op_sb4ixd(int);
	int op_sb5ixd(int);
	int op_sb6ixd(int);
	int op_sb7ixd(int);
	int op_rlcixd(int);
	int op_rrcixd(int);
	int op_rlixd(int);
	int op_rrixd(int);
	int op_slaixd(int);
	int op_sraixd(int);
	int op_srlixd(int);

	virtual int trap_fdcb(int){return 0;};
	int op_tb0iyd(int);
	int op_tb1iyd(int);
	int op_tb2iyd(int);
	int op_tb3iyd(int);
	int op_tb4iyd(int);
	int op_tb5iyd(int);
	int op_tb6iyd(int);
	int op_tb7iyd(int);
	int op_rb0iyd(int);
	int op_rb1iyd(int);
	int op_rb2iyd(int);
	int op_rb3iyd(int);
	int op_rb4iyd(int);
	int op_rb5iyd(int);
	int op_rb6iyd(int);
	int op_rb7iyd(int);
	int op_sb0iyd(int);
	int op_sb1iyd(int);
	int op_sb2iyd(int);
	int op_sb3iyd(int);
	int op_sb4iyd(int);
	int op_sb5iyd(int);
	int op_sb6iyd(int);
	int op_sb7iyd(int);
	int op_rlciyd(int);
	int op_rrciyd(int);
	int op_rliyd(int);
	int op_rriyd(int);
	int op_slaiyd(int);
	int op_sraiyd(int);
	int op_srliyd(int);

#ifdef WANT_DISASS
	struct opt {
		int (z80::*fun) (const char *s, WORD p);
		char *text;
	};
	int addr;
	static const char *unkown;
	static const char *reg[];
	static const char *regix;
	static const char *regiy;

	static const struct opt optab[256];

	int opout(const char *s, WORD p);
	int nout(const char *s, WORD p);
	int iout(const char *s, WORD p);
	int rout(const char *s, WORD p);
	int nnout(const char *s, WORD p);
	int inout(const char *s, WORD p);
	int cbop(const char *s, WORD p);
	int edop(const char *s, WORD p);
	int ddfd(const char *s, WORD p);
#endif /* WANT_DISASS */

private:

};

#endif //__ZILOG_H__