/*
 * Z80SIM  -  a	Z80-CPU	simulator
 *
 * Copyright (C) 1987-2006 by Udo Munk
 *
 * History:
 * 28-SEP-87 Development on TARGON/35 with AT&T Unix System V.3
 * 11-JAN-89 Release 1.1
 * 08-FEB-89 Release 1.2
 * 13-MAR-89 Release 1.3
 * 09-FEB-90 Release 1.4 Ported to TARGON/31 M10/30
 * 20-DEC-90 Release 1.5 Ported to COHERENT 3.0
 * 10-JUN-92 Release 1.6 long casting problem solved with COHERENT 3.2
 *			 and some optimization
 * 25-JUN-92 Release 1.7 comments in english
 * 04-OCT-06 Release 1.8 modified to compile on modern POSIX OS's
 * 27-MAR-07 Release 1.8-cpp-1.0 Ported to CPP by J.Sievers <JSievers@nadhh.Hanse.de>
 */
#include "stdafx.h"
#include <io.h>
#include <fcntl.h>
//#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include "zilog.h"

const char z80::version[] = CPPPORT;
void z80::reset(void)
{
	hlt = 0;
	PC = 0;
	I = 0;
	R = 0;
	IFF = 0;
	int_mode = IM0;
	int_type = INT_NONE;
	bus = FALSE;
}

void z80::nmi(void) { int_type |= INT_NMI; };
void z80::intr(void) { int_type |= INT_INT;};

const z80::optf_t z80::op_sim[256] = {
		&z80::op_nop,			/* 0x00	*/
		&z80::op_ldbcnn,		/* 0x01	*/
		&z80::op_ldbca,			/* 0x02	*/
		&z80::op_incbc,			/* 0x03	*/
		&z80::op_incb,			/* 0x04	*/
		&z80::op_decb,			/* 0x05	*/
		&z80::op_ldbn,			/* 0x06	*/
		&z80::op_rlca,			/* 0x07	*/
		&z80::op_exafaf,		/* 0x08	*/
		&z80::op_adhlbc,		/* 0x09	*/
		&z80::op_ldabc,			/* 0x0a	*/
		&z80::op_decbc,			/* 0x0b	*/
		&z80::op_incc,			/* 0x0c	*/
		&z80::op_decc,			/* 0x0d	*/
		&z80::op_ldcn,			/* 0x0e	*/
		&z80::op_rrca,			/* 0x0f	*/
		&z80::op_djnz,			/* 0x10	*/
		&z80::op_lddenn,		/* 0x11	*/
		&z80::op_lddea,			/* 0x12	*/
		&z80::op_incde,			/* 0x13	*/
		&z80::op_incd,			/* 0x14	*/
		&z80::op_decd,			/* 0x15	*/
		&z80::op_lddn,			/* 0x16	*/
		&z80::op_rla,			/* 0x17	*/
		&z80::op_jr,			/* 0x18	*/
		&z80::op_adhlde,		/* 0x19	*/
		&z80::op_ldade,			/* 0x1a	*/
		&z80::op_decde,			/* 0x1b	*/
		&z80::op_ince,			/* 0x1c	*/
		&z80::op_dece,			/* 0x1d	*/
		&z80::op_lden,			/* 0x1e	*/
		&z80::op_rra,			/* 0x1f	*/
		&z80::op_jrnz,			/* 0x20	*/
		&z80::op_ldhlnn,		/* 0x21	*/
		&z80::op_ldinhl,		/* 0x22	*/
		&z80::op_inchl,			/* 0x23	*/
		&z80::op_inch,			/* 0x24	*/
		&z80::op_dech,			/* 0x25	*/
		&z80::op_ldhn,			/* 0x26	*/
		&z80::op_daa,			/* 0x27	*/
		&z80::op_jrz,			/* 0x28	*/
		&z80::op_adhlhl,		/* 0x29	*/
		&z80::op_ldhlin,		/* 0x2a	*/
		&z80::op_dechl,			/* 0x2b	*/
		&z80::op_incl,			/* 0x2c	*/
		&z80::op_decl,			/* 0x2d	*/
		&z80::op_ldln,			/* 0x2e	*/
		&z80::op_cpl,			/* 0x2f	*/
		&z80::op_jrnc,			/* 0x30	*/
		&z80::op_ldspnn,		/* 0x31	*/
		&z80::op_ldnna,			/* 0x32	*/
		&z80::op_incsp,			/* 0x33	*/
		&z80::op_incihl,		/* 0x34	*/
		&z80::op_decihl,		/* 0x35	*/
		&z80::op_ldhl1,			/* 0x36	*/
		&z80::op_scf,			/* 0x37	*/
		&z80::op_jrc,			/* 0x38	*/
		&z80::op_adhlsp,		/* 0x39	*/
		&z80::op_ldann,			/* 0x3a	*/
		&z80::op_decsp,			/* 0x3b	*/
		&z80::op_inca,			/* 0x3c	*/
		&z80::op_deca,			/* 0x3d	*/
		&z80::op_ldan,			/* 0x3e	*/
		&z80::op_ccf,			/* 0x3f	*/
		&z80::op_ldbb,			/* 0x40	*/
		&z80::op_ldbc,			/* 0x41	*/
		&z80::op_ldbd,			/* 0x42	*/
		&z80::op_ldbe,			/* 0x43	*/
		&z80::op_ldbh,			/* 0x44	*/
		&z80::op_ldbl,			/* 0x45	*/
		&z80::op_ldbhl,			/* 0x46	*/
		&z80::op_ldba,			/* 0x47	*/
		&z80::op_ldcb,			/* 0x48	*/
		&z80::op_ldcc,			/* 0x49	*/
		&z80::op_ldcd,			/* 0x4a	*/
		&z80::op_ldce,			/* 0x4b	*/
		&z80::op_ldch,			/* 0x4c	*/
		&z80::op_ldcl,			/* 0x4d	*/
		&z80::op_ldchl,			/* 0x4e	*/
		&z80::op_ldca,			/* 0x4f	*/
		&z80::op_lddb,			/* 0x50	*/
		&z80::op_lddc,			/* 0x51	*/
		&z80::op_lddd,			/* 0x52	*/
		&z80::op_ldde,			/* 0x53	*/
		&z80::op_lddh,			/* 0x54	*/
		&z80::op_lddl,			/* 0x55	*/
		&z80::op_lddhl,			/* 0x56	*/
		&z80::op_ldda,			/* 0x57	*/
		&z80::op_ldeb,			/* 0x58	*/
		&z80::op_ldec,			/* 0x59	*/
		&z80::op_lded,			/* 0x5a	*/
		&z80::op_ldee,			/* 0x5b	*/
		&z80::op_ldeh,			/* 0x5c	*/
		&z80::op_ldel,			/* 0x5d	*/
		&z80::op_ldehl,			/* 0x5e	*/
		&z80::op_ldea,			/* 0x5f	*/
		&z80::op_ldhb,			/* 0x60	*/
		&z80::op_ldhc,			/* 0x61	*/
		&z80::op_ldhd,			/* 0x62	*/
		&z80::op_ldhe,			/* 0x63	*/
		&z80::op_ldhh,			/* 0x64	*/
		&z80::op_ldhl,			/* 0x65	*/
		&z80::op_ldhhl,			/* 0x66	*/
		&z80::op_ldha,			/* 0x67	*/
		&z80::op_ldlb,			/* 0x68	*/
		&z80::op_ldlc,			/* 0x69	*/
		&z80::op_ldld,			/* 0x6a	*/
		&z80::op_ldle,			/* 0x6b	*/
		&z80::op_ldlh,			/* 0x6c	*/
		&z80::op_ldll,			/* 0x6d	*/
		&z80::op_ldlhl,			/* 0x6e	*/
		&z80::op_ldla,			/* 0x6f	*/
		&z80::op_ldhlb,			/* 0x70	*/
		&z80::op_ldhlc,			/* 0x71	*/
		&z80::op_ldhld,			/* 0x72	*/
		&z80::op_ldhle,			/* 0x73	*/
		&z80::op_ldhlh,			/* 0x74	*/
		&z80::op_ldhll,			/* 0x75	*/
		&z80::op_halt,			/* 0x76	*/
		&z80::op_ldhla,			/* 0x77	*/
		&z80::op_ldab,			/* 0x78	*/
		&z80::op_ldac,			/* 0x79	*/
		&z80::op_ldad,			/* 0x7a	*/
		&z80::op_ldae,			/* 0x7b	*/
		&z80::op_ldah,			/* 0x7c	*/
		&z80::op_ldal,			/* 0x7d	*/
		&z80::op_ldahl,			/* 0x7e	*/
		&z80::op_ldaa,			/* 0x7f	*/
		&z80::op_addb,			/* 0x80	*/
		&z80::op_addc,			/* 0x81	*/
		&z80::op_addd,			/* 0x82	*/
		&z80::op_adde,			/* 0x83	*/
		&z80::op_addh,			/* 0x84	*/
		&z80::op_addl,			/* 0x85	*/
		&z80::op_addhl,			/* 0x86	*/
		&z80::op_adda,			/* 0x87	*/
		&z80::op_adcb,			/* 0x88	*/
		&z80::op_adcc,			/* 0x89	*/
		&z80::op_adcd,			/* 0x8a	*/
		&z80::op_adce,			/* 0x8b	*/
		&z80::op_adch,			/* 0x8c	*/
		&z80::op_adcl,			/* 0x8d	*/
		&z80::op_adchl,			/* 0x8e	*/
		&z80::op_adca,			/* 0x8f	*/
		&z80::op_subb,			/* 0x90	*/
		&z80::op_subc,			/* 0x91	*/
		&z80::op_subd,			/* 0x92	*/
		&z80::op_sube,			/* 0x93	*/
		&z80::op_subh,			/* 0x94	*/
		&z80::op_subl,			/* 0x95	*/
		&z80::op_subhl,			/* 0x96	*/
		&z80::op_suba,			/* 0x97	*/
		&z80::op_sbcb,			/* 0x98	*/
		&z80::op_sbcc,			/* 0x99	*/
		&z80::op_sbcd,			/* 0x9a	*/
		&z80::op_sbce,			/* 0x9b	*/
		&z80::op_sbch,			/* 0x9c	*/
		&z80::op_sbcl,			/* 0x9d	*/
		&z80::op_sbchl,			/* 0x9e	*/
		&z80::op_sbca,			/* 0x9f	*/
		&z80::op_andb,			/* 0xa0	*/
		&z80::op_andc,			/* 0xa1	*/
		&z80::op_andd,			/* 0xa2	*/
		&z80::op_ande,			/* 0xa3	*/
		&z80::op_andh,			/* 0xa4	*/
		&z80::op_andl,			/* 0xa5	*/
		&z80::op_andhl,			/* 0xa6	*/
		&z80::op_anda,			/* 0xa7	*/
		&z80::op_xorb,			/* 0xa8	*/
		&z80::op_xorc,			/* 0xa9	*/
		&z80::op_xord,			/* 0xaa	*/
		&z80::op_xore,			/* 0xab	*/
		&z80::op_xorh,			/* 0xac	*/
		&z80::op_xorl,			/* 0xad	*/
		&z80::op_xorhl,			/* 0xae	*/
		&z80::op_xora,			/* 0xaf	*/
		&z80::op_orb,			/* 0xb0	*/
		&z80::op_orc,			/* 0xb1	*/
		&z80::op_ord,			/* 0xb2	*/
		&z80::op_ore,			/* 0xb3	*/
		&z80::op_orh,			/* 0xb4	*/
		&z80::op_orl,			/* 0xb5	*/
		&z80::op_orhl,			/* 0xb6	*/
		&z80::op_ora,			/* 0xb7	*/
		&z80::op_cpb,			/* 0xb8	*/
		&z80::op_cpc,			/* 0xb9	*/
		&z80::op_cpd,			/* 0xba	*/
		&z80::op_cpe,			/* 0xbb	*/
		&z80::op_cph,			/* 0xbc	*/
		&z80::op_cplr,			/* 0xbd	*/
		&z80::op_cphl,			/* 0xbe	*/
		&z80::op_cpa,			/* 0xbf	*/
		&z80::op_retnz,			/* 0xc0	*/
		&z80::op_popbc,			/* 0xc1	*/
		&z80::op_jpnz,			/* 0xc2	*/
		&z80::op_jp,			/* 0xc3	*/
		&z80::op_calnz,			/* 0xc4	*/
		&z80::op_pushbc,		/* 0xc5	*/
		&z80::op_addn,			/* 0xc6	*/
		&z80::op_rst00,			/* 0xc7	*/
		&z80::op_retz,			/* 0xc8	*/
		&z80::op_ret,			/* 0xc9	*/
		&z80::op_jpz,			/* 0xca	*/
		&z80::op_cb_handel,		/* 0xcb	*/
		&z80::op_calz,			/* 0xcc	*/
		&z80::op_call,			/* 0xcd	*/
		&z80::op_adcn,			/* 0xce	*/
		&z80::op_rst08,			/* 0xcf	*/
		&z80::op_retnc,			/* 0xd0	*/
		&z80::op_popde,			/* 0xd1	*/
		&z80::op_jpnc,			/* 0xd2	*/
		&z80::op_out,			/* 0xd3	*/
		&z80::op_calnc,			/* 0xd4	*/
		&z80::op_pushde,		/* 0xd5	*/
		&z80::op_subn,			/* 0xd6	*/
		&z80::op_rst10,			/* 0xd7	*/
		&z80::op_retc,			/* 0xd8	*/
		&z80::op_exx,			/* 0xd9	*/
		&z80::op_jpc,			/* 0xda	*/
		&z80::op_in,			/* 0xdb	*/
		&z80::op_calc,			/* 0xdc	*/
		&z80::op_dd_handel,		/* 0xdd	*/
		&z80::op_sbcn,			/* 0xde	*/
		&z80::op_rst18,			/* 0xdf	*/
		&z80::op_retpo,			/* 0xe0	*/
		&z80::op_pophl,			/* 0xe1	*/
		&z80::op_jppo,			/* 0xe2	*/
		&z80::op_exsphl,		/* 0xe3	*/
		&z80::op_calpo,			/* 0xe4	*/
		&z80::op_pushhl,		/* 0xe5	*/
		&z80::op_andn,			/* 0xe6	*/
		&z80::op_rst20,			/* 0xe7	*/
		&z80::op_retpe,			/* 0xe8	*/
		&z80::op_jphl,			/* 0xe9	*/
		&z80::op_jppe,			/* 0xea	*/
		&z80::op_exdehl,		/* 0xeb	*/
		&z80::op_calpe,			/* 0xec	*/
		&z80::op_ed_handel,		/* 0xed	*/
		&z80::op_xorn,			/* 0xee	*/
		&z80::op_rst28,			/* 0xef	*/
		&z80::op_retp,			/* 0xf0	*/
		&z80::op_popaf,			/* 0xf1	*/
		&z80::op_jpp,			/* 0xf2	*/
		&z80::op_di,			/* 0xf3	*/
		&z80::op_calp,			/* 0xf4	*/
		&z80::op_pushaf,		/* 0xf5	*/
		&z80::op_orn,			/* 0xf6	*/
		&z80::op_rst30,			/* 0xf7	*/
		&z80::op_retm,			/* 0xf8	*/
		&z80::op_ldsphl,		/* 0xf9	*/
		&z80::op_jpm,			/* 0xfa	*/
		&z80::op_ei,			/* 0xfb	*/
		&z80::op_calm,			/* 0xfc	*/
		&z80::op_fd_handel,		/* 0xfd	*/
		&z80::op_cpn,			/* 0xfe	*/
		&z80::op_rst38			/* 0xff	*/
	};

const z80::optf_t z80::op_cb[256] =	{
		&z80::op_rlcb,			/* 0x00 */
		&z80::op_rlcc,			/* 0x01	*/
		&z80::op_rlcd,			/* 0x02	*/
		&z80::op_rlce,			/* 0x03	*/
		&z80::op_rlch,			/* 0x04	*/
		&z80::op_rlcl,			/* 0x05	*/
		&z80::op_rlchl,			/* 0x06	*/
		&z80::op_rlcra,			/* 0x07	*/
		&z80::op_rrcb,			/* 0x08	*/
		&z80::op_rrcc,			/* 0x09	*/
		&z80::op_rrcd,			/* 0x0a	*/
		&z80::op_rrce,			/* 0x0b	*/
		&z80::op_rrch,			/* 0x0c	*/
		&z80::op_rrcl,			/* 0x0d	*/
		&z80::op_rrchl,			/* 0x0e	*/
		&z80::op_rrcra,			/* 0x0f	*/
		&z80::op_rlb,				/* 0x10	*/
		&z80::op_rlc,				/* 0x11	*/
		&z80::op_rld,				/* 0x12	*/
		&z80::op_rle,				/* 0x13	*/
		&z80::op_rlh,				/* 0x14	*/
		&z80::op_rll,				/* 0x15	*/
		&z80::op_rlhl,			/* 0x16	*/
		&z80::op_rlra,			/* 0x17	*/
		&z80::op_rrb,				/* 0x18	*/
		&z80::op_rrc,				/* 0x19	*/
		&z80::op_rrd,				/* 0x1a	*/
		&z80::op_rre,				/* 0x1b	*/
		&z80::op_rrh,				/* 0x1c	*/
		&z80::op_rrl,				/* 0x1d	*/
		&z80::op_rrhl,			/* 0x1e	*/
		&z80::op_rrra,			/* 0x1f	*/
		&z80::op_slab,			/* 0x20	*/
		&z80::op_slac,			/* 0x21	*/
		&z80::op_slad,			/* 0x22	*/
		&z80::op_slae,			/* 0x23	*/
		&z80::op_slah,			/* 0x24	*/
		&z80::op_slal,			/* 0x25	*/
		&z80::op_slahl,			/* 0x26	*/
		&z80::op_slaa,			/* 0x27	*/
		&z80::op_srab,			/* 0x28	*/
		&z80::op_srac,			/* 0x29	*/
		&z80::op_srad,			/* 0x2a	*/
		&z80::op_srae,			/* 0x2b	*/
		&z80::op_srah,			/* 0x2c	*/
		&z80::op_sral,			/* 0x2d	*/
		&z80::op_srahl,			/* 0x2e	*/
		&z80::op_sraa,			/* 0x2f	*/
		&z80::trap_cb,			/* 0x30	*/
		&z80::trap_cb,			/* 0x31	*/
		&z80::trap_cb,			/* 0x32	*/
		&z80::trap_cb,			/* 0x33	*/
		&z80::trap_cb,			/* 0x34	*/
		&z80::trap_cb,			/* 0x35	*/
		&z80::trap_cb,			/* 0x36	*/
		&z80::trap_cb,			/* 0x37	*/
		&z80::op_srlb,			/* 0x38	*/
		&z80::op_srlc,			/* 0x39	*/
		&z80::op_srld,			/* 0x3a	*/
		&z80::op_srle,			/* 0x3b	*/
		&z80::op_srlh,			/* 0x3c	*/
		&z80::op_srll,			/* 0x3d	*/
		&z80::op_srlhl,			/* 0x3e	*/
		&z80::op_srla,			/* 0x3f	*/
		&z80::op_tb0b,			/* 0x40	*/
		&z80::op_tb0c,			/* 0x41	*/
		&z80::op_tb0d,			/* 0x42	*/
		&z80::op_tb0e,			/* 0x43	*/
		&z80::op_tb0h,			/* 0x44	*/
		&z80::op_tb0l,			/* 0x45	*/
		&z80::op_tb0hl,			/* 0x46	*/
		&z80::op_tb0a,			/* 0x47	*/
		&z80::op_tb1b,			/* 0x48	*/
		&z80::op_tb1c,			/* 0x49	*/
		&z80::op_tb1d,			/* 0x4a	*/
		&z80::op_tb1e,			/* 0x4b	*/
		&z80::op_tb1h,			/* 0x4c	*/
		&z80::op_tb1l,			/* 0x4d	*/
		&z80::op_tb1hl,			/* 0x4e	*/
		&z80::op_tb1a,			/* 0x4f	*/
		&z80::op_tb2b,			/* 0x50	*/
		&z80::op_tb2c,			/* 0x51	*/
		&z80::op_tb2d,			/* 0x52	*/
		&z80::op_tb2e,			/* 0x53	*/
		&z80::op_tb2h,			/* 0x54	*/
		&z80::op_tb2l,			/* 0x55	*/
		&z80::op_tb2hl,			/* 0x56	*/
		&z80::op_tb2a,			/* 0x57	*/
		&z80::op_tb3b,			/* 0x58	*/
		&z80::op_tb3c,			/* 0x59	*/
		&z80::op_tb3d,			/* 0x5a	*/
		&z80::op_tb3e,			/* 0x5b	*/
		&z80::op_tb3h,			/* 0x5c	*/
		&z80::op_tb3l,			/* 0x5d	*/
		&z80::op_tb3hl,			/* 0x5e	*/
		&z80::op_tb3a,			/* 0x5f	*/
		&z80::op_tb4b,			/* 0x60	*/
		&z80::op_tb4c,			/* 0x61	*/
		&z80::op_tb4d,			/* 0x62	*/
		&z80::op_tb4e,			/* 0x63	*/
		&z80::op_tb4h,			/* 0x64	*/
		&z80::op_tb4l,			/* 0x65	*/
		&z80::op_tb4hl,			/* 0x66	*/
		&z80::op_tb4a,			/* 0x67	*/
		&z80::op_tb5b,			/* 0x68	*/
		&z80::op_tb5c,			/* 0x69	*/
		&z80::op_tb5d,			/* 0x6a	*/
		&z80::op_tb5e,			/* 0x6b	*/
		&z80::op_tb5h,			/* 0x6c	*/
		&z80::op_tb5l,			/* 0x6d	*/
		&z80::op_tb5hl,			/* 0x6e	*/
		&z80::op_tb5a,			/* 0x6f	*/
		&z80::op_tb6b,			/* 0x70	*/
		&z80::op_tb6c,			/* 0x71	*/
		&z80::op_tb6d,			/* 0x72	*/
		&z80::op_tb6e,			/* 0x73	*/
		&z80::op_tb6h,			/* 0x74	*/
		&z80::op_tb6l,			/* 0x75	*/
		&z80::op_tb6hl,			/* 0x76	*/
		&z80::op_tb6a,			/* 0x77	*/
		&z80::op_tb7b,			/* 0x78	*/
		&z80::op_tb7c,			/* 0x79	*/
		&z80::op_tb7d,			/* 0x7a	*/
		&z80::op_tb7e,			/* 0x7b	*/
		&z80::op_tb7h,			/* 0x7c	*/
		&z80::op_tb7l,			/* 0x7d	*/
		&z80::op_tb7hl,			/* 0x7e	*/
		&z80::op_tb7a,			/* 0x7f	*/
		&z80::op_rb0b,			/* 0x80	*/
		&z80::op_rb0c,			/* 0x81	*/
		&z80::op_rb0d,			/* 0x82	*/
		&z80::op_rb0e,			/* 0x83	*/
		&z80::op_rb0h,			/* 0x84	*/
		&z80::op_rb0l,			/* 0x85	*/
		&z80::op_rb0hl,			/* 0x86	*/
		&z80::op_rb0a,			/* 0x87	*/
		&z80::op_rb1b,			/* 0x88	*/
		&z80::op_rb1c,			/* 0x89	*/
		&z80::op_rb1d,			/* 0x8a	*/
		&z80::op_rb1e,			/* 0x8b	*/
		&z80::op_rb1h,			/* 0x8c	*/
		&z80::op_rb1l,			/* 0x8d	*/
		&z80::op_rb1hl,			/* 0x8e	*/
		&z80::op_rb1a,			/* 0x8f	*/
		&z80::op_rb2b,			/* 0x90	*/
		&z80::op_rb2c,			/* 0x91	*/
		&z80::op_rb2d,			/* 0x92	*/
		&z80::op_rb2e,			/* 0x93	*/
		&z80::op_rb2h,			/* 0x94	*/
		&z80::op_rb2l,			/* 0x95	*/
		&z80::op_rb2hl,			/* 0x96	*/
		&z80::op_rb2a,			/* 0x97	*/
		&z80::op_rb3b,			/* 0x98	*/
		&z80::op_rb3c,			/* 0x99	*/
		&z80::op_rb3d,			/* 0x9a	*/
		&z80::op_rb3e,			/* 0x9b	*/
		&z80::op_rb3h,			/* 0x9c	*/
		&z80::op_rb3l,			/* 0x9d	*/
		&z80::op_rb3hl,			/* 0x9e	*/
		&z80::op_rb3a,			/* 0x9f	*/
		&z80::op_rb4b,			/* 0xa0	*/
		&z80::op_rb4c,			/* 0xa1	*/
		&z80::op_rb4d,			/* 0xa2	*/
		&z80::op_rb4e,			/* 0xa3	*/
		&z80::op_rb4h,			/* 0xa4	*/
		&z80::op_rb4l,			/* 0xa5	*/
		&z80::op_rb4hl,			/* 0xa6	*/
		&z80::op_rb4a,			/* 0xa7	*/
		&z80::op_rb5b,			/* 0xa8	*/
		&z80::op_rb5c,			/* 0xa9	*/
		&z80::op_rb5d,			/* 0xaa	*/
		&z80::op_rb5e,			/* 0xab	*/
		&z80::op_rb5h,			/* 0xac	*/
		&z80::op_rb5l,			/* 0xad	*/
		&z80::op_rb5hl,			/* 0xae	*/
		&z80::op_rb5a,			/* 0xaf	*/
		&z80::op_rb6b,			/* 0xb0	*/
		&z80::op_rb6c,			/* 0xb1	*/
		&z80::op_rb6d,			/* 0xb2	*/
		&z80::op_rb6e,			/* 0xb3	*/
		&z80::op_rb6h,			/* 0xb4	*/
		&z80::op_rb6l,			/* 0xb5	*/
		&z80::op_rb6hl,			/* 0xb6	*/
		&z80::op_rb6a,			/* 0xb7	*/
		&z80::op_rb7b,			/* 0xb8	*/
		&z80::op_rb7c,			/* 0xb9	*/
		&z80::op_rb7d,			/* 0xba	*/
		&z80::op_rb7e,			/* 0xbb	*/
		&z80::op_rb7h,			/* 0xbc	*/
		&z80::op_rb7l,			/* 0xbd	*/
		&z80::op_rb7hl,			/* 0xbe	*/
		&z80::op_rb7a,			/* 0xbf	*/
		&z80::op_sb0b,			/* 0xc0	*/
		&z80::op_sb0c,			/* 0xc1	*/
		&z80::op_sb0d,			/* 0xc2	*/
		&z80::op_sb0e,			/* 0xc3	*/
		&z80::op_sb0h,			/* 0xc4	*/
		&z80::op_sb0l,			/* 0xc5	*/
		&z80::op_sb0hl,			/* 0xc6	*/
		&z80::op_sb0a,			/* 0xc7	*/
		&z80::op_sb1b,			/* 0xc8	*/
		&z80::op_sb1c,			/* 0xc9	*/
		&z80::op_sb1d,			/* 0xca	*/
		&z80::op_sb1e,			/* 0xcb	*/
		&z80::op_sb1h,			/* 0xcc	*/
		&z80::op_sb1l,			/* 0xcd	*/
		&z80::op_sb1hl,			/* 0xce	*/
		&z80::op_sb1a,			/* 0xcf	*/
		&z80::op_sb2b,			/* 0xd0	*/
		&z80::op_sb2c,			/* 0xd1	*/
		&z80::op_sb2d,			/* 0xd2	*/
		&z80::op_sb2e,			/* 0xd3	*/
		&z80::op_sb2h,			/* 0xd4	*/
		&z80::op_sb2l,			/* 0xd5	*/
		&z80::op_sb2hl,			/* 0xd6	*/
		&z80::op_sb2a,			/* 0xd7	*/
		&z80::op_sb3b,			/* 0xd8	*/
		&z80::op_sb3c,			/* 0xd9	*/
		&z80::op_sb3d,			/* 0xda	*/
		&z80::op_sb3e,			/* 0xdb	*/
		&z80::op_sb3h,			/* 0xdc	*/
		&z80::op_sb3l,			/* 0xdd	*/
		&z80::op_sb3hl,			/* 0xde	*/
		&z80::op_sb3a,			/* 0xdf	*/
		&z80::op_sb4b,			/* 0xe0	*/
		&z80::op_sb4c,			/* 0xe1	*/
		&z80::op_sb4d,			/* 0xe2	*/
		&z80::op_sb4e,			/* 0xe3	*/
		&z80::op_sb4h,			/* 0xe4	*/
		&z80::op_sb4l,			/* 0xe5	*/
		&z80::op_sb4hl,			/* 0xe6	*/
		&z80::op_sb4a,			/* 0xe7	*/
		&z80::op_sb5b,			/* 0xe8	*/
		&z80::op_sb5c,			/* 0xe9	*/
		&z80::op_sb5d,			/* 0xea	*/
		&z80::op_sb5e,			/* 0xeb	*/
		&z80::op_sb5h,			/* 0xec	*/
		&z80::op_sb5l,			/* 0xed	*/
		&z80::op_sb5hl,			/* 0xee	*/
		&z80::op_sb5a,			/* 0xef	*/
		&z80::op_sb6b,			/* 0xf0	*/
		&z80::op_sb6c,			/* 0xf1	*/
		&z80::op_sb6d,			/* 0xf2	*/
		&z80::op_sb6e,			/* 0xf3	*/
		&z80::op_sb6h,			/* 0xf4	*/
		&z80::op_sb6l,			/* 0xf5	*/
		&z80::op_sb6hl,			/* 0xf6	*/
		&z80::op_sb6a,			/* 0xf7	*/
		&z80::op_sb7b,			/* 0xf8	*/
		&z80::op_sb7c,			/* 0xf9	*/
		&z80::op_sb7d,			/* 0xfa	*/
		&z80::op_sb7e,			/* 0xfb	*/
		&z80::op_sb7h,			/* 0xfc	*/
		&z80::op_sb7l,			/* 0xfd	*/
		&z80::op_sb7hl,			/* 0xfe	*/
		&z80::op_sb7a			/* 0xff	*/
	};

const z80::optf_t z80::op_dd[256] =	{
		&z80::trap_dd,			/* 0x00	*/
		&z80::trap_dd,			/* 0x01	*/
		&z80::trap_dd,			/* 0x02	*/
		&z80::trap_dd,			/* 0x03	*/
		&z80::trap_dd,			/* 0x04	*/
		&z80::trap_dd,			/* 0x05	*/
		&z80::trap_dd,			/* 0x06	*/
		&z80::trap_dd,			/* 0x07	*/
		&z80::trap_dd,			/* 0x08	*/
		&z80::op_addxb,			/* 0x09	*/
		&z80::trap_dd,			/* 0x0a	*/
		&z80::trap_dd,			/* 0x0b	*/
		&z80::trap_dd,			/* 0x0c	*/
		&z80::trap_dd,			/* 0x0d	*/
		&z80::trap_dd,			/* 0x0e	*/
		&z80::trap_dd,			/* 0x0f	*/
		&z80::trap_dd,			/* 0x10	*/
		&z80::trap_dd,			/* 0x11	*/
		&z80::trap_dd,			/* 0x12	*/
		&z80::trap_dd,			/* 0x13	*/
		&z80::trap_dd,			/* 0x14	*/
		&z80::trap_dd,			/* 0x15	*/
		&z80::trap_dd,			/* 0x16	*/
		&z80::trap_dd,			/* 0x17	*/
		&z80::trap_dd,			/* 0x18	*/
		&z80::op_addxd,			/* 0x19	*/
		&z80::trap_dd,			/* 0x1a	*/
		&z80::trap_dd,			/* 0x1b	*/
		&z80::trap_dd,			/* 0x1c	*/
		&z80::trap_dd,			/* 0x1d	*/
		&z80::trap_dd,			/* 0x1e	*/
		&z80::trap_dd,			/* 0x1f	*/
		&z80::trap_dd,			/* 0x20	*/
		&z80::op_ldixnn,		/* 0x21	*/
		&z80::op_ldinx,			/* 0x22	*/
		&z80::op_incix,			/* 0x23	*/
		&z80::trap_dd,			/* 0x24	*/
		&z80::trap_dd,			/* 0x25	*/
		&z80::trap_dd,			/* 0x26	*/
		&z80::trap_dd,			/* 0x27	*/
		&z80::trap_dd,			/* 0x28	*/
		&z80::op_addxx,			/* 0x29	*/
		&z80::op_ldixinn,		/* 0x2a	*/
		&z80::op_decix,			/* 0x2b	*/
		&z80::trap_dd,			/* 0x2c	*/
		&z80::trap_dd,			/* 0x2d	*/
		&z80::trap_dd,			/* 0x2e	*/
		&z80::trap_dd,			/* 0x2f	*/
		&z80::trap_dd,			/* 0x30	*/
		&z80::trap_dd,			/* 0x31	*/
		&z80::trap_dd,			/* 0x32	*/
		&z80::trap_dd,			/* 0x33	*/
		&z80::op_incxd,			/* 0x34	*/
		&z80::op_decxd,			/* 0x35	*/
		&z80::op_ldxdn,			/* 0x36	*/
		&z80::trap_dd,			/* 0x37	*/
		&z80::trap_dd,			/* 0x38	*/
		&z80::op_addxs,			/* 0x39	*/
		&z80::trap_dd,			/* 0x3a	*/
		&z80::trap_dd,			/* 0x3b	*/
		&z80::trap_dd,			/* 0x3c	*/
		&z80::trap_dd,			/* 0x3d	*/
		&z80::trap_dd,			/* 0x3e	*/
		&z80::trap_dd,			/* 0x3f	*/
		&z80::trap_dd,			/* 0x40	*/
		&z80::trap_dd,			/* 0x41	*/
		&z80::trap_dd,			/* 0x42	*/
		&z80::trap_dd,			/* 0x43	*/
		&z80::trap_dd,			/* 0x44	*/
		&z80::trap_dd,			/* 0x45	*/
		&z80::op_ldbxd,			/* 0x46	*/
		&z80::trap_dd,			/* 0x47	*/
		&z80::trap_dd,			/* 0x48	*/
		&z80::trap_dd,			/* 0x49	*/
		&z80::trap_dd,			/* 0x4a	*/
		&z80::trap_dd,			/* 0x4b	*/
		&z80::trap_dd,			/* 0x4c	*/
		&z80::trap_dd,			/* 0x4d	*/
		&z80::op_ldcxd,			/* 0x4e	*/
		&z80::trap_dd,			/* 0x4f	*/
		&z80::trap_dd,			/* 0x50	*/
		&z80::trap_dd,			/* 0x51	*/
		&z80::trap_dd,			/* 0x52	*/
		&z80::trap_dd,			/* 0x53	*/
		&z80::trap_dd,			/* 0x54	*/
		&z80::trap_dd,			/* 0x55	*/
		&z80::op_lddxd,			/* 0x56	*/
		&z80::trap_dd,			/* 0x57	*/
		&z80::trap_dd,			/* 0x58	*/
		&z80::trap_dd,			/* 0x59	*/
		&z80::trap_dd,			/* 0x5a	*/
		&z80::trap_dd,			/* 0x5b	*/
		&z80::trap_dd,			/* 0x5c	*/
		&z80::trap_dd,			/* 0x5d	*/
		&z80::op_ldexd,			/* 0x5e	*/
		&z80::trap_dd,			/* 0x5f	*/
		&z80::trap_dd,			/* 0x60	*/
		&z80::trap_dd,			/* 0x61	*/
		&z80::trap_dd,			/* 0x62	*/
		&z80::trap_dd,			/* 0x63	*/
		&z80::trap_dd,			/* 0x64	*/
		&z80::trap_dd,			/* 0x65	*/
		&z80::op_ldhxd,			/* 0x66	*/
		&z80::trap_dd,			/* 0x67	*/
		&z80::trap_dd,			/* 0x68	*/
		&z80::trap_dd,			/* 0x69	*/
		&z80::trap_dd,			/* 0x6a	*/
		&z80::trap_dd,			/* 0x6b	*/
		&z80::trap_dd,			/* 0x6c	*/
		&z80::trap_dd,			/* 0x6d	*/
		&z80::op_ldlxd,			/* 0x6e	*/
		&z80::trap_dd,			/* 0x6f	*/
		&z80::op_ldxdb,			/* 0x70	*/
		&z80::op_ldxdc,			/* 0x71	*/
		&z80::op_ldxdd,			/* 0x72	*/
		&z80::op_ldxde,			/* 0x73	*/
		&z80::op_ldxdh,			/* 0x74	*/
		&z80::op_ldxdl,			/* 0x75	*/
		&z80::trap_dd,			/* 0x76	*/
		&z80::op_ldxda,			/* 0x77	*/
		&z80::trap_dd,			/* 0x78	*/
		&z80::trap_dd,			/* 0x79	*/
		&z80::trap_dd,			/* 0x7a	*/
		&z80::trap_dd,			/* 0x7b	*/
		&z80::trap_dd,			/* 0x7c	*/
		&z80::trap_dd,			/* 0x7d	*/
		&z80::op_ldaxd,			/* 0x7e	*/
		&z80::trap_dd,			/* 0x7f	*/
		&z80::trap_dd,			/* 0x80	*/
		&z80::trap_dd,			/* 0x81	*/
		&z80::trap_dd,			/* 0x82	*/
		&z80::trap_dd,			/* 0x83	*/
		&z80::trap_dd,			/* 0x84	*/
		&z80::trap_dd,			/* 0x85	*/
		&z80::op_adaxd,			/* 0x86	*/
		&z80::trap_dd,			/* 0x87	*/
		&z80::trap_dd,			/* 0x88	*/
		&z80::trap_dd,			/* 0x89	*/
		&z80::trap_dd,			/* 0x8a	*/
		&z80::trap_dd,			/* 0x8b	*/
		&z80::trap_dd,			/* 0x8c	*/
		&z80::trap_dd,			/* 0x8d	*/
		&z80::op_acaxd,			/* 0x8e	*/
		&z80::trap_dd,			/* 0x8f	*/
		&z80::trap_dd,			/* 0x90	*/
		&z80::trap_dd,			/* 0x91	*/
		&z80::trap_dd,			/* 0x92	*/
		&z80::trap_dd,			/* 0x93	*/
		&z80::trap_dd,			/* 0x94	*/
		&z80::trap_dd,			/* 0x95	*/
		&z80::op_suaxd,			/* 0x96	*/
		&z80::trap_dd,			/* 0x97	*/
		&z80::trap_dd,			/* 0x98	*/
		&z80::trap_dd,			/* 0x99	*/
		&z80::trap_dd,			/* 0x9a	*/
		&z80::trap_dd,			/* 0x9b	*/
		&z80::trap_dd,			/* 0x9c	*/
		&z80::trap_dd,			/* 0x9d	*/
		&z80::op_scaxd,			/* 0x9e	*/
		&z80::trap_dd,			/* 0x9f	*/
		&z80::trap_dd,			/* 0xa0	*/
		&z80::trap_dd,			/* 0xa1	*/
		&z80::trap_dd,			/* 0xa2	*/
		&z80::trap_dd,			/* 0xa3	*/
		&z80::trap_dd,			/* 0xa4	*/
		&z80::trap_dd,			/* 0xa5	*/
		&z80::op_andxd,			/* 0xa6	*/
		&z80::trap_dd,			/* 0xa7	*/
		&z80::trap_dd,			/* 0xa8	*/
		&z80::trap_dd,			/* 0xa9	*/
		&z80::trap_dd,			/* 0xaa	*/
		&z80::trap_dd,			/* 0xab	*/
		&z80::trap_dd,			/* 0xac	*/
		&z80::trap_dd,			/* 0xad	*/
		&z80::op_xorxd,			/* 0xae	*/
		&z80::trap_dd,			/* 0xaf	*/
		&z80::trap_dd,			/* 0xb0	*/
		&z80::trap_dd,			/* 0xb1	*/
		&z80::trap_dd,			/* 0xb2	*/
		&z80::trap_dd,			/* 0xb3	*/
		&z80::trap_dd,			/* 0xb4	*/
		&z80::trap_dd,			/* 0xb5	*/
		&z80::op_orxd,			/* 0xb6	*/
		&z80::trap_dd,			/* 0xb7	*/
		&z80::trap_dd,			/* 0xb8	*/
		&z80::trap_dd,			/* 0xb9	*/
		&z80::trap_dd,			/* 0xba	*/
		&z80::trap_dd,			/* 0xbb	*/
		&z80::trap_dd,			/* 0xbc	*/
		&z80::trap_dd,			/* 0xbd	*/
		&z80::op_cpxd,			/* 0xbe	*/
		&z80::trap_dd,			/* 0xbf	*/
		&z80::trap_dd,			/* 0xc0	*/
		&z80::trap_dd,			/* 0xc1	*/
		&z80::trap_dd,			/* 0xc2	*/
		&z80::trap_dd,			/* 0xc3	*/
		&z80::trap_dd,			/* 0xc4	*/
		&z80::trap_dd,			/* 0xc5	*/
		&z80::trap_dd,			/* 0xc6	*/
		&z80::trap_dd,			/* 0xc7	*/
		&z80::trap_dd,			/* 0xc8	*/
		&z80::trap_dd,			/* 0xc9	*/
		&z80::trap_dd,			/* 0xca	*/
		&z80::op_ddcb_handel,	/* 0xcb	*/
		&z80::trap_dd,			/* 0xcc	*/
		&z80::trap_dd,			/* 0xcd	*/
		&z80::trap_dd,			/* 0xce	*/
		&z80::trap_dd,			/* 0xcf	*/
		&z80::trap_dd,			/* 0xd0	*/
		&z80::trap_dd,			/* 0xd1	*/
		&z80::trap_dd,			/* 0xd2	*/
		&z80::trap_dd,			/* 0xd3	*/
		&z80::trap_dd,			/* 0xd4	*/
		&z80::trap_dd,			/* 0xd5	*/
		&z80::trap_dd,			/* 0xd6	*/
		&z80::trap_dd,			/* 0xd7	*/
		&z80::trap_dd,			/* 0xd8	*/
		&z80::trap_dd,			/* 0xd9	*/
		&z80::trap_dd,			/* 0xda	*/
		&z80::trap_dd,			/* 0xdb	*/
		&z80::trap_dd,			/* 0xdc	*/
		&z80::trap_dd,			/* 0xdd	*/
		&z80::trap_dd,			/* 0xde	*/
		&z80::trap_dd,			/* 0xdf	*/
		&z80::trap_dd,			/* 0xe0	*/
		&z80::op_popix,			/* 0xe1	*/
		&z80::trap_dd,			/* 0xe2	*/
		&z80::op_exspx,			/* 0xe3	*/
		&z80::trap_dd,			/* 0xe4	*/
		&z80::op_pusix,			/* 0xe5	*/
		&z80::trap_dd,			/* 0xe6	*/
		&z80::trap_dd,			/* 0xe7	*/
		&z80::trap_dd,			/* 0xe8	*/
		&z80::op_jpix,			/* 0xe9	*/
		&z80::trap_dd,			/* 0xea	*/
		&z80::trap_dd,			/* 0xeb	*/
		&z80::trap_dd,			/* 0xec	*/
		&z80::trap_dd,			/* 0xed	*/
		&z80::trap_dd,			/* 0xee	*/
		&z80::trap_dd,			/* 0xef	*/
		&z80::trap_dd,			/* 0xf0	*/
		&z80::trap_dd,			/* 0xf1	*/
		&z80::trap_dd,			/* 0xf2	*/
		&z80::trap_dd,			/* 0xf3	*/
		&z80::trap_dd,			/* 0xf4	*/
		&z80::trap_dd,			/* 0xf5	*/
		&z80::trap_dd,			/* 0xf6	*/
		&z80::trap_dd,			/* 0xf7	*/
		&z80::trap_dd,			/* 0xf8	*/
		&z80::op_ldspx,			/* 0xf9	*/
		&z80::trap_dd,			/* 0xfa	*/
		&z80::trap_dd,			/* 0xfb	*/
		&z80::trap_dd,			/* 0xfc	*/
		&z80::trap_dd,			/* 0xfd	*/
		&z80::trap_dd,			/* 0xfe	*/
		&z80::trap_dd			/* 0xff	*/
	};

const z80::optf_t z80::op_ed[256] = {
		&z80::trap_ed,			/* 0x00	*/
		&z80::trap_ed,			/* 0x01	*/
		&z80::trap_ed,			/* 0x02	*/
		&z80::trap_ed,			/* 0x03	*/
		&z80::trap_ed,			/* 0x04	*/
		&z80::trap_ed,			/* 0x05	*/
		&z80::trap_ed,			/* 0x06	*/
		&z80::trap_ed,			/* 0x07	*/
		&z80::trap_ed,			/* 0x08	*/
		&z80::trap_ed,			/* 0x09	*/
		&z80::trap_ed,			/* 0x0a	*/
		&z80::trap_ed,			/* 0x0b	*/
		&z80::trap_ed,			/* 0x0c	*/
		&z80::trap_ed,			/* 0x0d	*/
		&z80::trap_ed,			/* 0x0e	*/
		&z80::trap_ed,			/* 0x0f	*/
		&z80::trap_ed,			/* 0x10	*/
		&z80::trap_ed,			/* 0x11	*/
		&z80::trap_ed,			/* 0x12	*/
		&z80::trap_ed,			/* 0x13	*/
		&z80::trap_ed,			/* 0x14	*/
		&z80::trap_ed,			/* 0x15	*/
		&z80::trap_ed,			/* 0x16	*/
		&z80::trap_ed,			/* 0x17	*/
		&z80::trap_ed,			/* 0x18	*/
		&z80::trap_ed,			/* 0x19	*/
		&z80::trap_ed,			/* 0x1a	*/
		&z80::trap_ed,			/* 0x1b	*/
		&z80::trap_ed,			/* 0x1c	*/
		&z80::trap_ed,			/* 0x1d	*/
		&z80::trap_ed,			/* 0x1e	*/
		&z80::trap_ed,			/* 0x1f	*/
		&z80::trap_ed,			/* 0x20	*/
		&z80::trap_ed,			/* 0x21	*/
		&z80::trap_ed,			/* 0x22	*/
		&z80::trap_ed,			/* 0x23	*/
		&z80::trap_ed,			/* 0x24	*/
		&z80::trap_ed,			/* 0x25	*/
		&z80::trap_ed,			/* 0x26	*/
		&z80::trap_ed,			/* 0x27	*/
		&z80::trap_ed,			/* 0x28	*/
		&z80::trap_ed,			/* 0x29	*/
		&z80::trap_ed,			/* 0x2a	*/
		&z80::trap_ed,			/* 0x2b	*/
		&z80::trap_ed,			/* 0x2c	*/
		&z80::trap_ed,			/* 0x2d	*/
		&z80::trap_ed,			/* 0x2e	*/
		&z80::trap_ed,			/* 0x2f	*/
		&z80::trap_ed,			/* 0x30	*/
		&z80::trap_ed,			/* 0x31	*/
		&z80::trap_ed,			/* 0x32	*/
		&z80::trap_ed,			/* 0x33	*/
		&z80::trap_ed,			/* 0x34	*/
		&z80::trap_ed,			/* 0x35	*/
		&z80::trap_ed,			/* 0x36	*/
		&z80::trap_ed,			/* 0x37	*/
		&z80::trap_ed,			/* 0x38	*/
		&z80::trap_ed,			/* 0x39	*/
		&z80::trap_ed,			/* 0x3a	*/
		&z80::trap_ed,			/* 0x3b	*/
		&z80::trap_ed,			/* 0x3c	*/
		&z80::trap_ed,			/* 0x3d	*/
		&z80::trap_ed,			/* 0x3e	*/
		&z80::trap_ed,			/* 0x3f	*/
		&z80::op_inbic,			/* 0x40	*/
		&z80::op_outcb,			/* 0x41	*/
		&z80::op_sbchb,			/* 0x42	*/
		&z80::op_ldinbc,			/* 0x43	*/
		&z80::op_neg,				/* 0x44	*/
		&z80::op_retn,			/* 0x45	*/
		&z80::op_im0,				/* 0x46	*/
		&z80::op_ldia,			/* 0x47	*/
		&z80::op_incic,			/* 0x48	*/
		&z80::op_outcc,			/* 0x49	*/
		&z80::op_adchb,			/* 0x4a	*/
		&z80::op_ldbcinn,			/* 0x4b	*/
		&z80::trap_ed,			/* 0x4c	*/
		&z80::op_reti,			/* 0x4d	*/
		&z80::trap_ed,			/* 0x4e	*/
		&z80::op_ldra,			/* 0x4f	*/
		&z80::op_indic,			/* 0x50	*/
		&z80::op_outcd,			/* 0x51	*/
		&z80::op_sbchd,			/* 0x52	*/
		&z80::op_ldinde,			/* 0x53	*/
		&z80::trap_ed,			/* 0x54	*/
		&z80::trap_ed,			/* 0x55	*/
		&z80::op_im1,				/* 0x56	*/
		&z80::op_ldai,			/* 0x57	*/
		&z80::op_ineic,			/* 0x58	*/
		&z80::op_outce,			/* 0x59	*/
		&z80::op_adchd,			/* 0x5a	*/
		&z80::op_lddeinn,			/* 0x5b	*/
		&z80::trap_ed,			/* 0x5c	*/
		&z80::trap_ed,			/* 0x5d	*/
		&z80::op_im2,				/* 0x5e	*/
		&z80::op_ldar,			/* 0x5f	*/
		&z80::op_inhic,			/* 0x60	*/
		&z80::op_outch,			/* 0x61	*/
		&z80::op_sbchh,			/* 0x62	*/
		&z80::trap_ed,			/* 0x63	*/
		&z80::trap_ed,			/* 0x64	*/
		&z80::trap_ed,			/* 0x65	*/
		&z80::trap_ed,			/* 0x66	*/
		&z80::op_oprrd,			/* 0x67	*/
		&z80::op_inlic,			/* 0x68	*/
		&z80::op_outcl,			/* 0x69	*/
		&z80::op_adchh,			/* 0x6a	*/
		&z80::trap_ed,			/* 0x6b	*/
		&z80::trap_ed,			/* 0x6c	*/
		&z80::trap_ed,			/* 0x6d	*/
		&z80::trap_ed,			/* 0x6e	*/
		&z80::op_oprld,			/* 0x6f	*/
		&z80::trap_ed,			/* 0x70	*/
		&z80::trap_ed,			/* 0x71	*/
		&z80::op_sbchs,			/* 0x72	*/
		&z80::op_ldinsp,			/* 0x73	*/
		&z80::trap_ed,			/* 0x74	*/
		&z80::trap_ed,			/* 0x75	*/
		&z80::trap_ed,			/* 0x76	*/
		&z80::trap_ed,			/* 0x77	*/
		&z80::op_inaic,			/* 0x78	*/
		&z80::op_outca,			/* 0x79	*/
		&z80::op_adchs,			/* 0x7a	*/
		&z80::op_ldspinn,			/* 0x7b	*/
		&z80::trap_ed,			/* 0x7c	*/
		&z80::trap_ed,			/* 0x7d	*/
		&z80::trap_ed,			/* 0x7e	*/
		&z80::trap_ed,			/* 0x7f	*/
		&z80::trap_ed,			/* 0x80	*/
		&z80::trap_ed,			/* 0x81	*/
		&z80::trap_ed,			/* 0x82	*/
		&z80::trap_ed,			/* 0x83	*/
		&z80::trap_ed,			/* 0x84	*/
		&z80::trap_ed,			/* 0x85	*/
		&z80::trap_ed,			/* 0x86	*/
		&z80::trap_ed,			/* 0x87	*/
		&z80::trap_ed,			/* 0x88	*/
		&z80::trap_ed,			/* 0x89	*/
		&z80::trap_ed,			/* 0x8a	*/
		&z80::trap_ed,			/* 0x8b	*/
		&z80::trap_ed,			/* 0x8c	*/
		&z80::trap_ed,			/* 0x8d	*/
		&z80::trap_ed,			/* 0x8e	*/
		&z80::trap_ed,			/* 0x8f	*/
		&z80::trap_ed,			/* 0x90	*/
		&z80::trap_ed,			/* 0x91	*/
		&z80::trap_ed,			/* 0x92	*/
		&z80::trap_ed,			/* 0x93	*/
		&z80::trap_ed,			/* 0x94	*/
		&z80::trap_ed,			/* 0x95	*/
		&z80::trap_ed,			/* 0x96	*/
		&z80::trap_ed,			/* 0x97	*/
		&z80::trap_ed,			/* 0x98	*/
		&z80::trap_ed,			/* 0x99	*/
		&z80::trap_ed,			/* 0x9a	*/
		&z80::trap_ed,			/* 0x9b	*/
		&z80::trap_ed,			/* 0x9c	*/
		&z80::trap_ed,			/* 0x9d	*/
		&z80::trap_ed,			/* 0x9e	*/
		&z80::trap_ed,			/* 0x9f	*/
		&z80::op_ldi,				/* 0xa0	*/
		&z80::op_cpi,				/* 0xa1	*/
		&z80::op_ini,				/* 0xa2	*/
		&z80::op_outi,			/* 0xa3	*/
		&z80::trap_ed,			/* 0xa4	*/
		&z80::trap_ed,			/* 0xa5	*/
		&z80::trap_ed,			/* 0xa6	*/
		&z80::trap_ed,			/* 0xa7	*/
		&z80::op_ldd,				/* 0xa8	*/
		&z80::op_cpdop,			/* 0xa9	*/
		&z80::op_ind,				/* 0xaa	*/
		&z80::op_outd,			/* 0xab	*/
		&z80::trap_ed,			/* 0xac	*/
		&z80::trap_ed,			/* 0xad	*/
		&z80::trap_ed,			/* 0xae	*/
		&z80::trap_ed,			/* 0xaf	*/
		&z80::op_ldir,			/* 0xb0	*/
		&z80::op_cpir,			/* 0xb1	*/
		&z80::op_inir,			/* 0xb2	*/
		&z80::op_otir,			/* 0xb3	*/
		&z80::trap_ed,			/* 0xb4	*/
		&z80::trap_ed,			/* 0xb5	*/
		&z80::trap_ed,			/* 0xb6	*/
		&z80::trap_ed,			/* 0xb7	*/
		&z80::op_lddr,			/* 0xb8	*/
		&z80::op_cpdr,			/* 0xb9	*/
		&z80::op_indr,			/* 0xba	*/
		&z80::op_otdr,			/* 0xbb	*/
		&z80::trap_ed,			/* 0xbc	*/
		&z80::trap_ed,			/* 0xbd	*/
		&z80::trap_ed,			/* 0xbe	*/
		&z80::trap_ed,			/* 0xbf	*/
		&z80::trap_ed,			/* 0xc0	*/
		&z80::trap_ed,			/* 0xc1	*/
		&z80::trap_ed,			/* 0xc2	*/
		&z80::trap_ed,			/* 0xc3	*/
		&z80::trap_ed,			/* 0xc4	*/
		&z80::trap_ed,			/* 0xc5	*/
		&z80::trap_ed,			/* 0xc6	*/
		&z80::trap_ed,			/* 0xc7	*/
		&z80::trap_ed,			/* 0xc8	*/
		&z80::trap_ed,			/* 0xc9	*/
		&z80::trap_ed,			/* 0xca	*/
		&z80::trap_ed,			/* 0xcb	*/
		&z80::trap_ed,			/* 0xcc	*/
		&z80::trap_ed,			/* 0xcd	*/
		&z80::trap_ed,			/* 0xce	*/
		&z80::trap_ed,			/* 0xcf	*/
		&z80::trap_ed,			/* 0xd0	*/
		&z80::trap_ed,			/* 0xd1	*/
		&z80::trap_ed,			/* 0xd2	*/
		&z80::trap_ed,			/* 0xd3	*/
		&z80::trap_ed,			/* 0xd4	*/
		&z80::trap_ed,			/* 0xd5	*/
		&z80::trap_ed,			/* 0xd6	*/
		&z80::trap_ed,			/* 0xd7	*/
		&z80::trap_ed,			/* 0xd8	*/
		&z80::trap_ed,			/* 0xd9	*/
		&z80::trap_ed,			/* 0xda	*/
		&z80::trap_ed,			/* 0xdb	*/
		&z80::trap_ed,			/* 0xdc	*/
		&z80::trap_ed,			/* 0xdd	*/
		&z80::trap_ed,			/* 0xde	*/
		&z80::trap_ed,			/* 0xdf	*/
		&z80::trap_ed,			/* 0xe0	*/
		&z80::trap_ed,			/* 0xe1	*/
		&z80::trap_ed,			/* 0xe2	*/
		&z80::trap_ed,			/* 0xe3	*/
		&z80::trap_ed,			/* 0xe4	*/
		&z80::trap_ed,			/* 0xe5	*/
		&z80::trap_ed,			/* 0xe6	*/
		&z80::trap_ed,			/* 0xe7	*/
		&z80::trap_ed,			/* 0xe8	*/
		&z80::trap_ed,			/* 0xe9	*/
		&z80::trap_ed,			/* 0xea	*/
		&z80::trap_ed,			/* 0xeb	*/
		&z80::trap_ed,			/* 0xec	*/
		&z80::trap_ed,			/* 0xed	*/
		&z80::trap_ed,			/* 0xee	*/
		&z80::trap_ed,			/* 0xef	*/
		&z80::trap_ed,			/* 0xf0	*/
		&z80::trap_ed,			/* 0xf1	*/
		&z80::trap_ed,			/* 0xf2	*/
		&z80::trap_ed,			/* 0xf3	*/
		&z80::trap_ed,			/* 0xf4	*/
		&z80::trap_ed,			/* 0xf5	*/
		&z80::trap_ed,			/* 0xf6	*/
		&z80::trap_ed,			/* 0xf7	*/
		&z80::trap_ed,			/* 0xf8	*/
		&z80::trap_ed,			/* 0xf9	*/
		&z80::trap_ed,			/* 0xfa	*/
		&z80::trap_ed,			/* 0xfb	*/
		&z80::trap_ed,			/* 0xfc	*/
		&z80::trap_ed,			/* 0xfd	*/
		&z80::trap_ed,			/* 0xfe	*/
		&z80::trap_ed				/* 0xff	*/
	};

const z80::optf_t z80::op_fd[256] = {
		&z80::trap_fd,			/* 0x00	*/
		&z80::trap_fd,			/* 0x01	*/
		&z80::trap_fd,			/* 0x02	*/
		&z80::trap_fd,			/* 0x03	*/
		&z80::trap_fd,			/* 0x04	*/
		&z80::trap_fd,			/* 0x05	*/
		&z80::trap_fd,			/* 0x06	*/
		&z80::trap_fd,			/* 0x07	*/
		&z80::trap_fd,			/* 0x08	*/
		&z80::op_addyb,			/* 0x09	*/
		&z80::trap_fd,			/* 0x0a	*/
		&z80::trap_fd,			/* 0x0b	*/
		&z80::trap_fd,			/* 0x0c	*/
		&z80::trap_fd,			/* 0x0d	*/
		&z80::trap_fd,			/* 0x0e	*/
		&z80::trap_fd,			/* 0x0f	*/
		&z80::trap_fd,			/* 0x10	*/
		&z80::trap_fd,			/* 0x11	*/
		&z80::trap_fd,			/* 0x12	*/
		&z80::trap_fd,			/* 0x13	*/
		&z80::trap_fd,			/* 0x14	*/
		&z80::trap_fd,			/* 0x15	*/
		&z80::trap_fd,			/* 0x16	*/
		&z80::trap_fd,			/* 0x17	*/
		&z80::trap_fd,			/* 0x18	*/
		&z80::op_addyd,			/* 0x19	*/
		&z80::trap_fd,			/* 0x1a	*/
		&z80::trap_fd,			/* 0x1b	*/
		&z80::trap_fd,			/* 0x1c	*/
		&z80::trap_fd,			/* 0x1d	*/
		&z80::trap_fd,			/* 0x1e	*/
		&z80::trap_fd,			/* 0x1f	*/
		&z80::trap_fd,			/* 0x20	*/
		&z80::op_ldiynn,			/* 0x21	*/
		&z80::op_ldiny,			/* 0x22	*/
		&z80::op_inciy,			/* 0x23	*/
		&z80::trap_fd,			/* 0x24	*/
		&z80::trap_fd,			/* 0x25	*/
		&z80::trap_fd,			/* 0x26	*/
		&z80::trap_fd,			/* 0x27	*/
		&z80::trap_fd,			/* 0x28	*/
		&z80::op_addyy,			/* 0x29	*/
		&z80::op_ldiyinn,			/* 0x2a	*/
		&z80::op_deciy,			/* 0x2b	*/
		&z80::trap_fd,			/* 0x2c	*/
		&z80::trap_fd,			/* 0x2d	*/
		&z80::trap_fd,			/* 0x2e	*/
		&z80::trap_fd,			/* 0x2f	*/
		&z80::trap_fd,			/* 0x30	*/
		&z80::trap_fd,			/* 0x31	*/
		&z80::trap_fd,			/* 0x32	*/
		&z80::trap_fd,			/* 0x33	*/
		&z80::op_incyd,			/* 0x34	*/
		&z80::op_decyd,			/* 0x35	*/
		&z80::op_ldydn,			/* 0x36	*/
		&z80::trap_fd,			/* 0x37	*/
		&z80::trap_fd,			/* 0x38	*/
		&z80::op_addys,			/* 0x39	*/
		&z80::trap_fd,			/* 0x3a	*/
		&z80::trap_fd,			/* 0x3b	*/
		&z80::trap_fd,			/* 0x3c	*/
		&z80::trap_fd,			/* 0x3d	*/
		&z80::trap_fd,			/* 0x3e	*/
		&z80::trap_fd,			/* 0x3f	*/
		&z80::trap_fd,			/* 0x40	*/
		&z80::trap_fd,			/* 0x41	*/
		&z80::trap_fd,			/* 0x42	*/
		&z80::trap_fd,			/* 0x43	*/
		&z80::trap_fd,			/* 0x44	*/
		&z80::trap_fd,			/* 0x45	*/
		&z80::op_ldbyd,			/* 0x46	*/
		&z80::trap_fd,			/* 0x47	*/
		&z80::trap_fd,			/* 0x48	*/
		&z80::trap_fd,			/* 0x49	*/
		&z80::trap_fd,			/* 0x4a	*/
		&z80::trap_fd,			/* 0x4b	*/
		&z80::trap_fd,			/* 0x4c	*/
		&z80::trap_fd,			/* 0x4d	*/
		&z80::op_ldcyd,			/* 0x4e	*/
		&z80::trap_fd,			/* 0x4f	*/
		&z80::trap_fd,			/* 0x50	*/
		&z80::trap_fd,			/* 0x51	*/
		&z80::trap_fd,			/* 0x52	*/
		&z80::trap_fd,			/* 0x53	*/
		&z80::trap_fd,			/* 0x54	*/
		&z80::trap_fd,			/* 0x55	*/
		&z80::op_lddyd,			/* 0x56	*/
		&z80::trap_fd,			/* 0x57	*/
		&z80::trap_fd,			/* 0x58	*/
		&z80::trap_fd,			/* 0x59	*/
		&z80::trap_fd,			/* 0x5a	*/
		&z80::trap_fd,			/* 0x5b	*/
		&z80::trap_fd,			/* 0x5c	*/
		&z80::trap_fd,			/* 0x5d	*/
		&z80::op_ldeyd,			/* 0x5e	*/
		&z80::trap_fd,			/* 0x5f	*/
		&z80::trap_fd,			/* 0x60	*/
		&z80::trap_fd,			/* 0x61	*/
		&z80::trap_fd,			/* 0x62	*/
		&z80::trap_fd,			/* 0x63	*/
		&z80::trap_fd,			/* 0x64	*/
		&z80::trap_fd,			/* 0x65	*/
		&z80::op_ldhyd,			/* 0x66	*/
		&z80::trap_fd,			/* 0x67	*/
		&z80::trap_fd,			/* 0x68	*/
		&z80::trap_fd,			/* 0x69	*/
		&z80::trap_fd,			/* 0x6a	*/
		&z80::trap_fd,			/* 0x6b	*/
		&z80::trap_fd,			/* 0x6c	*/
		&z80::trap_fd,			/* 0x6d	*/
		&z80::op_ldlyd,			/* 0x6e	*/
		&z80::trap_fd,			/* 0x6f	*/
		&z80::op_ldydb,			/* 0x70	*/
		&z80::op_ldydc,			/* 0x71	*/
		&z80::op_ldydd,			/* 0x72	*/
		&z80::op_ldyde,			/* 0x73	*/
		&z80::op_ldydh,			/* 0x74	*/
		&z80::op_ldydl,			/* 0x75	*/
		&z80::trap_fd,			/* 0x76	*/
		&z80::op_ldyda,			/* 0x77	*/
		&z80::trap_fd,			/* 0x78	*/
		&z80::trap_fd,			/* 0x79	*/
		&z80::trap_fd,			/* 0x7a	*/
		&z80::trap_fd,			/* 0x7b	*/
		&z80::trap_fd,			/* 0x7c	*/
		&z80::trap_fd,			/* 0x7d	*/
		&z80::op_ldayd,			/* 0x7e	*/
		&z80::trap_fd,			/* 0x7f	*/
		&z80::trap_fd,			/* 0x80	*/
		&z80::trap_fd,			/* 0x81	*/
		&z80::trap_fd,			/* 0x82	*/
		&z80::trap_fd,			/* 0x83	*/
		&z80::trap_fd,			/* 0x84	*/
		&z80::trap_fd,			/* 0x85	*/
		&z80::op_adayd,			/* 0x86	*/
		&z80::trap_fd,			/* 0x87	*/
		&z80::trap_fd,			/* 0x88	*/
		&z80::trap_fd,			/* 0x89	*/
		&z80::trap_fd,			/* 0x8a	*/
		&z80::trap_fd,			/* 0x8b	*/
		&z80::trap_fd,			/* 0x8c	*/
		&z80::trap_fd,			/* 0x8d	*/
		&z80::op_acayd,			/* 0x8e	*/
		&z80::trap_fd,			/* 0x8f	*/
		&z80::trap_fd,			/* 0x90	*/
		&z80::trap_fd,			/* 0x91	*/
		&z80::trap_fd,			/* 0x92	*/
		&z80::trap_fd,			/* 0x93	*/
		&z80::trap_fd,			/* 0x94	*/
		&z80::trap_fd,			/* 0x95	*/
		&z80::op_suayd,			/* 0x96	*/
		&z80::trap_fd,			/* 0x97	*/
		&z80::trap_fd,			/* 0x98	*/
		&z80::trap_fd,			/* 0x99	*/
		&z80::trap_fd,			/* 0x9a	*/
		&z80::trap_fd,			/* 0x9b	*/
		&z80::trap_fd,			/* 0x9c	*/
		&z80::trap_fd,			/* 0x9d	*/
		&z80::op_scayd,			/* 0x9e	*/
		&z80::trap_fd,			/* 0x9f	*/
		&z80::trap_fd,			/* 0xa0	*/
		&z80::trap_fd,			/* 0xa1	*/
		&z80::trap_fd,			/* 0xa2	*/
		&z80::trap_fd,			/* 0xa3	*/
		&z80::trap_fd,			/* 0xa4	*/
		&z80::trap_fd,			/* 0xa5	*/
		&z80::op_andyd,			/* 0xa6	*/
		&z80::trap_fd,			/* 0xa7	*/
		&z80::trap_fd,			/* 0xa8	*/
		&z80::trap_fd,			/* 0xa9	*/
		&z80::trap_fd,			/* 0xaa	*/
		&z80::trap_fd,			/* 0xab	*/
		&z80::trap_fd,			/* 0xac	*/
		&z80::trap_fd,			/* 0xad	*/
		&z80::op_xoryd,			/* 0xae	*/
		&z80::trap_fd,			/* 0xaf	*/
		&z80::trap_fd,			/* 0xb0	*/
		&z80::trap_fd,			/* 0xb1	*/
		&z80::trap_fd,			/* 0xb2	*/
		&z80::trap_fd,			/* 0xb3	*/
		&z80::trap_fd,			/* 0xb4	*/
		&z80::trap_fd,			/* 0xb5	*/
		&z80::op_oryd,			/* 0xb6	*/
		&z80::trap_fd,			/* 0xb7	*/
		&z80::trap_fd,			/* 0xb8	*/
		&z80::trap_fd,			/* 0xb9	*/
		&z80::trap_fd,			/* 0xba	*/
		&z80::trap_fd,			/* 0xbb	*/
		&z80::trap_fd,			/* 0xbc	*/
		&z80::trap_fd,			/* 0xbd	*/
		&z80::op_cpyd,			/* 0xbe	*/
		&z80::trap_fd,			/* 0xbf	*/
		&z80::trap_fd,			/* 0xc0	*/
		&z80::trap_fd,			/* 0xc1	*/
		&z80::trap_fd,			/* 0xc2	*/
		&z80::trap_fd,			/* 0xc3	*/
		&z80::trap_fd,			/* 0xc4	*/
		&z80::trap_fd,			/* 0xc5	*/
		&z80::trap_fd,			/* 0xc6	*/
		&z80::trap_fd,			/* 0xc7	*/
		&z80::trap_fd,			/* 0xc8	*/
		&z80::trap_fd,			/* 0xc9	*/
		&z80::trap_fd,			/* 0xca	*/
		&z80::op_fdcb_handel,			/* 0xcb	*/
		&z80::trap_fd,			/* 0xcc	*/
		&z80::trap_fd,			/* 0xcd	*/
		&z80::trap_fd,			/* 0xce	*/
		&z80::trap_fd,			/* 0xcf	*/
		&z80::trap_fd,			/* 0xd0	*/
		&z80::trap_fd,			/* 0xd1	*/
		&z80::trap_fd,			/* 0xd2	*/
		&z80::trap_fd,			/* 0xd3	*/
		&z80::trap_fd,			/* 0xd4	*/
		&z80::trap_fd,			/* 0xd5	*/
		&z80::trap_fd,			/* 0xd6	*/
		&z80::trap_fd,			/* 0xd7	*/
		&z80::trap_fd,			/* 0xd8	*/
		&z80::trap_fd,			/* 0xd9	*/
		&z80::trap_fd,			/* 0xda	*/
		&z80::trap_fd,			/* 0xdb	*/
		&z80::trap_fd,			/* 0xdc	*/
		&z80::trap_fd,			/* 0xdd	*/
		&z80::trap_fd,			/* 0xde	*/
		&z80::trap_fd,			/* 0xdf	*/
		&z80::trap_fd,			/* 0xe0	*/
		&z80::op_popiy,			/* 0xe1	*/
		&z80::trap_fd,			/* 0xe2	*/
		&z80::op_exspy,			/* 0xe3	*/
		&z80::trap_fd,			/* 0xe4	*/
		&z80::op_pusiy,			/* 0xe5	*/
		&z80::trap_fd,			/* 0xe6	*/
		&z80::trap_fd,			/* 0xe7	*/
		&z80::trap_fd,			/* 0xe8	*/
		&z80::op_jpiy,			/* 0xe9	*/
		&z80::trap_fd,			/* 0xea	*/
		&z80::trap_fd,			/* 0xeb	*/
		&z80::trap_fd,			/* 0xec	*/
		&z80::trap_fd,			/* 0xed	*/
		&z80::trap_fd,			/* 0xee	*/
		&z80::trap_fd,			/* 0xef	*/
		&z80::trap_fd,			/* 0xf0	*/
		&z80::trap_fd,			/* 0xf1	*/
		&z80::trap_fd,			/* 0xf2	*/
		&z80::trap_fd,			/* 0xf3	*/
		&z80::trap_fd,			/* 0xf4	*/
		&z80::trap_fd,			/* 0xf5	*/
		&z80::trap_fd,			/* 0xf6	*/
		&z80::trap_fd,			/* 0xf7	*/
		&z80::trap_fd,			/* 0xf8	*/
		&z80::op_ldspy,			/* 0xf9	*/
		&z80::trap_fd,			/* 0xfa	*/
		&z80::trap_fd,			/* 0xfb	*/
		&z80::trap_fd,			/* 0xfc	*/
		&z80::trap_fd,			/* 0xfd	*/
		&z80::trap_fd,			/* 0xfe	*/
		&z80::trap_fd				/* 0xff	*/
	};

const z80::optf1_t z80::op_ddcb[256] = {
		&z80::trap_ddcb,			/* 0x00	*/
		&z80::trap_ddcb,			/* 0x01	*/
		&z80::trap_ddcb,			/* 0x02	*/
		&z80::trap_ddcb,			/* 0x03	*/
		&z80::trap_ddcb,			/* 0x04	*/
		&z80::trap_ddcb,			/* 0x05	*/
		&z80::op_rlcixd,			/* 0x06	*/
		&z80::trap_ddcb,			/* 0x07	*/
		&z80::trap_ddcb,			/* 0x08	*/
		&z80::trap_ddcb,			/* 0x09	*/
		&z80::trap_ddcb,			/* 0x0a	*/
		&z80::trap_ddcb,			/* 0x0b	*/
		&z80::trap_ddcb,			/* 0x0c	*/
		&z80::trap_ddcb,			/* 0x0d	*/
		&z80::op_rrcixd,			/* 0x0e	*/
		&z80::trap_ddcb,			/* 0x0f	*/
		&z80::trap_ddcb,			/* 0x10	*/
		&z80::trap_ddcb,			/* 0x11	*/
		&z80::trap_ddcb,			/* 0x12	*/
		&z80::trap_ddcb,			/* 0x13	*/
		&z80::trap_ddcb,			/* 0x14	*/
		&z80::trap_ddcb,			/* 0x15	*/
		&z80::op_rlixd,			/* 0x16	*/
		&z80::trap_ddcb,			/* 0x17	*/
		&z80::trap_ddcb,			/* 0x18	*/
		&z80::trap_ddcb,			/* 0x19	*/
		&z80::trap_ddcb,			/* 0x1a	*/
		&z80::trap_ddcb,			/* 0x1b	*/
		&z80::trap_ddcb,			/* 0x1c	*/
		&z80::trap_ddcb,			/* 0x1d	*/
		&z80::op_rrixd,			/* 0x1e	*/
		&z80::trap_ddcb,			/* 0x1f	*/
		&z80::trap_ddcb,			/* 0x20	*/
		&z80::trap_ddcb,			/* 0x21	*/
		&z80::trap_ddcb,			/* 0x22	*/
		&z80::trap_ddcb,			/* 0x23	*/
		&z80::trap_ddcb,			/* 0x24	*/
		&z80::trap_ddcb,			/* 0x25	*/
		&z80::op_slaixd,			/* 0x26	*/
		&z80::trap_ddcb,			/* 0x27	*/
		&z80::trap_ddcb,			/* 0x28	*/
		&z80::trap_ddcb,			/* 0x29	*/
		&z80::trap_ddcb,			/* 0x2a	*/
		&z80::trap_ddcb,			/* 0x2b	*/
		&z80::trap_ddcb,			/* 0x2c	*/
		&z80::trap_ddcb,			/* 0x2d	*/
		&z80::op_sraixd,			/* 0x2e	*/
		&z80::trap_ddcb,			/* 0x2f	*/
		&z80::trap_ddcb,			/* 0x30	*/
		&z80::trap_ddcb,			/* 0x31	*/
		&z80::trap_ddcb,			/* 0x32	*/
		&z80::trap_ddcb,			/* 0x33	*/
		&z80::trap_ddcb,			/* 0x34	*/
		&z80::trap_ddcb,			/* 0x35	*/
		&z80::trap_ddcb,			/* 0x36	*/
		&z80::trap_ddcb,			/* 0x37	*/
		&z80::trap_ddcb,			/* 0x38	*/
		&z80::trap_ddcb,			/* 0x39	*/
		&z80::trap_ddcb,			/* 0x3a	*/
		&z80::trap_ddcb,			/* 0x3b	*/
		&z80::trap_ddcb,			/* 0x3c	*/
		&z80::trap_ddcb,			/* 0x3d	*/
		&z80::op_srlixd,			/* 0x3e	*/
		&z80::trap_ddcb,			/* 0x3f	*/
		&z80::trap_ddcb,			/* 0x40	*/
		&z80::trap_ddcb,			/* 0x41	*/
		&z80::trap_ddcb,			/* 0x42	*/
		&z80::trap_ddcb,			/* 0x43	*/
		&z80::trap_ddcb,			/* 0x44	*/
		&z80::trap_ddcb,			/* 0x45	*/
		&z80::op_tb0ixd,			/* 0x46	*/
		&z80::trap_ddcb,			/* 0x47	*/
		&z80::trap_ddcb,			/* 0x48	*/
		&z80::trap_ddcb,			/* 0x49	*/
		&z80::trap_ddcb,			/* 0x4a	*/
		&z80::trap_ddcb,			/* 0x4b	*/
		&z80::trap_ddcb,			/* 0x4c	*/
		&z80::trap_ddcb,			/* 0x4d	*/
		&z80::op_tb1ixd,			/* 0x4e	*/
		&z80::trap_ddcb,			/* 0x4f	*/
		&z80::trap_ddcb,			/* 0x50	*/
		&z80::trap_ddcb,			/* 0x51	*/
		&z80::trap_ddcb,			/* 0x52	*/
		&z80::trap_ddcb,			/* 0x53	*/
		&z80::trap_ddcb,			/* 0x54	*/
		&z80::trap_ddcb,			/* 0x55	*/
		&z80::op_tb2ixd,			/* 0x56	*/
		&z80::trap_ddcb,			/* 0x57	*/
		&z80::trap_ddcb,			/* 0x58	*/
		&z80::trap_ddcb,			/* 0x59	*/
		&z80::trap_ddcb,			/* 0x5a	*/
		&z80::trap_ddcb,			/* 0x5b	*/
		&z80::trap_ddcb,			/* 0x5c	*/
		&z80::trap_ddcb,			/* 0x5d	*/
		&z80::op_tb3ixd,			/* 0x5e	*/
		&z80::trap_ddcb,			/* 0x5f	*/
		&z80::trap_ddcb,			/* 0x60	*/
		&z80::trap_ddcb,			/* 0x61	*/
		&z80::trap_ddcb,			/* 0x62	*/
		&z80::trap_ddcb,			/* 0x63	*/
		&z80::trap_ddcb,			/* 0x64	*/
		&z80::trap_ddcb,			/* 0x65	*/
		&z80::op_tb4ixd,			/* 0x66	*/
		&z80::trap_ddcb,			/* 0x67	*/
		&z80::trap_ddcb,			/* 0x68	*/
		&z80::trap_ddcb,			/* 0x69	*/
		&z80::trap_ddcb,			/* 0x6a	*/
		&z80::trap_ddcb,			/* 0x6b	*/
		&z80::trap_ddcb,			/* 0x6c	*/
		&z80::trap_ddcb,			/* 0x6d	*/
		&z80::op_tb5ixd,			/* 0x6e	*/
		&z80::trap_ddcb,			/* 0x6f	*/
		&z80::trap_ddcb,			/* 0x70	*/
		&z80::trap_ddcb,			/* 0x71	*/
		&z80::trap_ddcb,			/* 0x72	*/
		&z80::trap_ddcb,			/* 0x73	*/
		&z80::trap_ddcb,			/* 0x74	*/
		&z80::trap_ddcb,			/* 0x75	*/
		&z80::op_tb6ixd,			/* 0x76	*/
		&z80::trap_ddcb,			/* 0x77	*/
		&z80::trap_ddcb,			/* 0x78	*/
		&z80::trap_ddcb,			/* 0x79	*/
		&z80::trap_ddcb,			/* 0x7a	*/
		&z80::trap_ddcb,			/* 0x7b	*/
		&z80::trap_ddcb,			/* 0x7c	*/
		&z80::trap_ddcb,			/* 0x7d	*/
		&z80::op_tb7ixd,			/* 0x7e	*/
		&z80::trap_ddcb,			/* 0x7f	*/
		&z80::trap_ddcb,			/* 0x80	*/
		&z80::trap_ddcb,			/* 0x81	*/
		&z80::trap_ddcb,			/* 0x82	*/
		&z80::trap_ddcb,			/* 0x83	*/
		&z80::trap_ddcb,			/* 0x84	*/
		&z80::trap_ddcb,			/* 0x85	*/
		&z80::op_rb0ixd,			/* 0x86	*/
		&z80::trap_ddcb,			/* 0x87	*/
		&z80::trap_ddcb,			/* 0x88	*/
		&z80::trap_ddcb,			/* 0x89	*/
		&z80::trap_ddcb,			/* 0x8a	*/
		&z80::trap_ddcb,			/* 0x8b	*/
		&z80::trap_ddcb,			/* 0x8c	*/
		&z80::trap_ddcb,			/* 0x8d	*/
		&z80::op_rb1ixd,			/* 0x8e	*/
		&z80::trap_ddcb,			/* 0x8f	*/
		&z80::trap_ddcb,			/* 0x90	*/
		&z80::trap_ddcb,			/* 0x91	*/
		&z80::trap_ddcb,			/* 0x92	*/
		&z80::trap_ddcb,			/* 0x93	*/
		&z80::trap_ddcb,			/* 0x94	*/
		&z80::trap_ddcb,			/* 0x95	*/
		&z80::op_rb2ixd,			/* 0x96	*/
		&z80::trap_ddcb,			/* 0x97	*/
		&z80::trap_ddcb,			/* 0x98	*/
		&z80::trap_ddcb,			/* 0x99	*/
		&z80::trap_ddcb,			/* 0x9a	*/
		&z80::trap_ddcb,			/* 0x9b	*/
		&z80::trap_ddcb,			/* 0x9c	*/
		&z80::trap_ddcb,			/* 0x9d	*/
		&z80::op_rb3ixd,			/* 0x9e	*/
		&z80::trap_ddcb,			/* 0x9f	*/
		&z80::trap_ddcb,			/* 0xa0	*/
		&z80::trap_ddcb,			/* 0xa1	*/
		&z80::trap_ddcb,			/* 0xa2	*/
		&z80::trap_ddcb,			/* 0xa3	*/
		&z80::trap_ddcb,			/* 0xa4	*/
		&z80::trap_ddcb,			/* 0xa5	*/
		&z80::op_rb4ixd,			/* 0xa6	*/
		&z80::trap_ddcb,			/* 0xa7	*/
		&z80::trap_ddcb,			/* 0xa8	*/
		&z80::trap_ddcb,			/* 0xa9	*/
		&z80::trap_ddcb,			/* 0xaa	*/
		&z80::trap_ddcb,			/* 0xab	*/
		&z80::trap_ddcb,			/* 0xac	*/
		&z80::trap_ddcb,			/* 0xad	*/
		&z80::op_rb5ixd,			/* 0xae	*/
		&z80::trap_ddcb,			/* 0xaf	*/
		&z80::trap_ddcb,			/* 0xb0	*/
		&z80::trap_ddcb,			/* 0xb1	*/
		&z80::trap_ddcb,			/* 0xb2	*/
		&z80::trap_ddcb,			/* 0xb3	*/
		&z80::trap_ddcb,			/* 0xb4	*/
		&z80::trap_ddcb,			/* 0xb5	*/
		&z80::op_rb6ixd,			/* 0xb6	*/
		&z80::trap_ddcb,			/* 0xb7	*/
		&z80::trap_ddcb,			/* 0xb8	*/
		&z80::trap_ddcb,			/* 0xb9	*/
		&z80::trap_ddcb,			/* 0xba	*/
		&z80::trap_ddcb,			/* 0xbb	*/
		&z80::trap_ddcb,			/* 0xbc	*/
		&z80::trap_ddcb,			/* 0xbd	*/
		&z80::op_rb7ixd,			/* 0xbe	*/
		&z80::trap_ddcb,			/* 0xbf	*/
		&z80::trap_ddcb,			/* 0xc0	*/
		&z80::trap_ddcb,			/* 0xc1	*/
		&z80::trap_ddcb,			/* 0xc2	*/
		&z80::trap_ddcb,			/* 0xc3	*/
		&z80::trap_ddcb,			/* 0xc4	*/
		&z80::trap_ddcb,			/* 0xc5	*/
		&z80::op_sb0ixd,			/* 0xc6	*/
		&z80::trap_ddcb,			/* 0xc7	*/
		&z80::trap_ddcb,			/* 0xc8	*/
		&z80::trap_ddcb,			/* 0xc9	*/
		&z80::trap_ddcb,			/* 0xca	*/
		&z80::trap_ddcb,			/* 0xcb	*/
		&z80::trap_ddcb,			/* 0xcc	*/
		&z80::trap_ddcb,			/* 0xcd	*/
		&z80::op_sb1ixd,			/* 0xce	*/
		&z80::trap_ddcb,			/* 0xcf	*/
		&z80::trap_ddcb,			/* 0xd0	*/
		&z80::trap_ddcb,			/* 0xd1	*/
		&z80::trap_ddcb,			/* 0xd2	*/
		&z80::trap_ddcb,			/* 0xd3	*/
		&z80::trap_ddcb,			/* 0xd4	*/
		&z80::trap_ddcb,			/* 0xd5	*/
		&z80::op_sb2ixd,			/* 0xd6	*/
		&z80::trap_ddcb,			/* 0xd7	*/
		&z80::trap_ddcb,			/* 0xd8	*/
		&z80::trap_ddcb,			/* 0xd9	*/
		&z80::trap_ddcb,			/* 0xda	*/
		&z80::trap_ddcb,			/* 0xdb	*/
		&z80::trap_ddcb,			/* 0xdc	*/
		&z80::trap_ddcb,			/* 0xdd	*/
		&z80::op_sb3ixd,			/* 0xde	*/
		&z80::trap_ddcb,			/* 0xdf	*/
		&z80::trap_ddcb,			/* 0xe0	*/
		&z80::trap_ddcb,			/* 0xe1	*/
		&z80::trap_ddcb,			/* 0xe2	*/
		&z80::trap_ddcb,			/* 0xe3	*/
		&z80::trap_ddcb,			/* 0xe4	*/
		&z80::trap_ddcb,			/* 0xe5	*/
		&z80::op_sb4ixd,			/* 0xe6	*/
		&z80::trap_ddcb,			/* 0xe7	*/
		&z80::trap_ddcb,			/* 0xe8	*/
		&z80::trap_ddcb,			/* 0xe9	*/
		&z80::trap_ddcb,			/* 0xea	*/
		&z80::trap_ddcb,			/* 0xeb	*/
		&z80::trap_ddcb,			/* 0xec	*/
		&z80::trap_ddcb,			/* 0xed	*/
		&z80::op_sb5ixd,			/* 0xee	*/
		&z80::trap_ddcb,			/* 0xef	*/
		&z80::trap_ddcb,			/* 0xf0	*/
		&z80::trap_ddcb,			/* 0xf1	*/
		&z80::trap_ddcb,			/* 0xf2	*/
		&z80::trap_ddcb,			/* 0xf3	*/
		&z80::trap_ddcb,			/* 0xf4	*/
		&z80::trap_ddcb,			/* 0xf5	*/
		&z80::op_sb6ixd,			/* 0xf6	*/
		&z80::trap_ddcb,			/* 0xf7	*/
		&z80::trap_ddcb,			/* 0xf8	*/
		&z80::trap_ddcb,			/* 0xf9	*/
		&z80::trap_ddcb,			/* 0xfa	*/
		&z80::trap_ddcb,			/* 0xfb	*/
		&z80::trap_ddcb,			/* 0xfc	*/
		&z80::trap_ddcb,			/* 0xfd	*/
		&z80::op_sb7ixd,			/* 0xfe	*/
		&z80::trap_ddcb			/* 0xff	*/
	};

const z80::optf1_t z80::op_fdcb[256] = {
		&z80::trap_fdcb,			/* 0x00	*/
		&z80::trap_fdcb,			/* 0x01	*/
		&z80::trap_fdcb,			/* 0x02	*/
		&z80::trap_fdcb,			/* 0x03	*/
		&z80::trap_fdcb,			/* 0x04	*/
		&z80::trap_fdcb,			/* 0x05	*/
		&z80::op_rlciyd,			/* 0x06	*/
		&z80::trap_fdcb,			/* 0x07	*/
		&z80::trap_fdcb,			/* 0x08	*/
		&z80::trap_fdcb,			/* 0x09	*/
		&z80::trap_fdcb,			/* 0x0a	*/
		&z80::trap_fdcb,			/* 0x0b	*/
		&z80::trap_fdcb,			/* 0x0c	*/
		&z80::trap_fdcb,			/* 0x0d	*/
		&z80::op_rrciyd,			/* 0x0e	*/
		&z80::trap_fdcb,			/* 0x0f	*/
		&z80::trap_fdcb,			/* 0x10	*/
		&z80::trap_fdcb,			/* 0x11	*/
		&z80::trap_fdcb,			/* 0x12	*/
		&z80::trap_fdcb,			/* 0x13	*/
		&z80::trap_fdcb,			/* 0x14	*/
		&z80::trap_fdcb,			/* 0x15	*/
		&z80::op_rliyd,			/* 0x16	*/
		&z80::trap_fdcb,			/* 0x17	*/
		&z80::trap_fdcb,			/* 0x18	*/
		&z80::trap_fdcb,			/* 0x19	*/
		&z80::trap_fdcb,			/* 0x1a	*/
		&z80::trap_fdcb,			/* 0x1b	*/
		&z80::trap_fdcb,			/* 0x1c	*/
		&z80::trap_fdcb,			/* 0x1d	*/
		&z80::op_rriyd,			/* 0x1e	*/
		&z80::trap_fdcb,			/* 0x1f	*/
		&z80::trap_fdcb,			/* 0x20	*/
		&z80::trap_fdcb,			/* 0x21	*/
		&z80::trap_fdcb,			/* 0x22	*/
		&z80::trap_fdcb,			/* 0x23	*/
		&z80::trap_fdcb,			/* 0x24	*/
		&z80::trap_fdcb,			/* 0x25	*/
		&z80::op_slaiyd,			/* 0x26	*/
		&z80::trap_fdcb,			/* 0x27	*/
		&z80::trap_fdcb,			/* 0x28	*/
		&z80::trap_fdcb,			/* 0x29	*/
		&z80::trap_fdcb,			/* 0x2a	*/
		&z80::trap_fdcb,			/* 0x2b	*/
		&z80::trap_fdcb,			/* 0x2c	*/
		&z80::trap_fdcb,			/* 0x2d	*/
		&z80::op_sraiyd,			/* 0x2e	*/
		&z80::trap_fdcb,			/* 0x2f	*/
		&z80::trap_fdcb,			/* 0x30	*/
		&z80::trap_fdcb,			/* 0x31	*/
		&z80::trap_fdcb,			/* 0x32	*/
		&z80::trap_fdcb,			/* 0x33	*/
		&z80::trap_fdcb,			/* 0x34	*/
		&z80::trap_fdcb,			/* 0x35	*/
		&z80::trap_fdcb,			/* 0x36	*/
		&z80::trap_fdcb,			/* 0x37	*/
		&z80::trap_fdcb,			/* 0x38	*/
		&z80::trap_fdcb,			/* 0x39	*/
		&z80::trap_fdcb,			/* 0x3a	*/
		&z80::trap_fdcb,			/* 0x3b	*/
		&z80::trap_fdcb,			/* 0x3c	*/
		&z80::trap_fdcb,			/* 0x3d	*/
		&z80::op_srliyd,			/* 0x3e	*/
		&z80::trap_fdcb,			/* 0x3f	*/
		&z80::trap_fdcb,			/* 0x40	*/
		&z80::trap_fdcb,			/* 0x41	*/
		&z80::trap_fdcb,			/* 0x42	*/
		&z80::trap_fdcb,			/* 0x43	*/
		&z80::trap_fdcb,			/* 0x44	*/
		&z80::trap_fdcb,			/* 0x45	*/
		&z80::op_tb0iyd,			/* 0x46	*/
		&z80::trap_fdcb,			/* 0x47	*/
		&z80::trap_fdcb,			/* 0x48	*/
		&z80::trap_fdcb,			/* 0x49	*/
		&z80::trap_fdcb,			/* 0x4a	*/
		&z80::trap_fdcb,			/* 0x4b	*/
		&z80::trap_fdcb,			/* 0x4c	*/
		&z80::trap_fdcb,			/* 0x4d	*/
		&z80::op_tb1iyd,			/* 0x4e	*/
		&z80::trap_fdcb,			/* 0x4f	*/
		&z80::trap_fdcb,			/* 0x50	*/
		&z80::trap_fdcb,			/* 0x51	*/
		&z80::trap_fdcb,			/* 0x52	*/
		&z80::trap_fdcb,			/* 0x53	*/
		&z80::trap_fdcb,			/* 0x54	*/
		&z80::trap_fdcb,			/* 0x55	*/
		&z80::op_tb2iyd,			/* 0x56	*/
		&z80::trap_fdcb,			/* 0x57	*/
		&z80::trap_fdcb,			/* 0x58	*/
		&z80::trap_fdcb,			/* 0x59	*/
		&z80::trap_fdcb,			/* 0x5a	*/
		&z80::trap_fdcb,			/* 0x5b	*/
		&z80::trap_fdcb,			/* 0x5c	*/
		&z80::trap_fdcb,			/* 0x5d	*/
		&z80::op_tb3iyd,			/* 0x5e	*/
		&z80::trap_fdcb,			/* 0x5f	*/
		&z80::trap_fdcb,			/* 0x60	*/
		&z80::trap_fdcb,			/* 0x61	*/
		&z80::trap_fdcb,			/* 0x62	*/
		&z80::trap_fdcb,			/* 0x63	*/
		&z80::trap_fdcb,			/* 0x64	*/
		&z80::trap_fdcb,			/* 0x65	*/
		&z80::op_tb4iyd,			/* 0x66	*/
		&z80::trap_fdcb,			/* 0x67	*/
		&z80::trap_fdcb,			/* 0x68	*/
		&z80::trap_fdcb,			/* 0x69	*/
		&z80::trap_fdcb,			/* 0x6a	*/
		&z80::trap_fdcb,			/* 0x6b	*/
		&z80::trap_fdcb,			/* 0x6c	*/
		&z80::trap_fdcb,			/* 0x6d	*/
		&z80::op_tb5iyd,			/* 0x6e	*/
		&z80::trap_fdcb,			/* 0x6f	*/
		&z80::trap_fdcb,			/* 0x70	*/
		&z80::trap_fdcb,			/* 0x71	*/
		&z80::trap_fdcb,			/* 0x72	*/
		&z80::trap_fdcb,			/* 0x73	*/
		&z80::trap_fdcb,			/* 0x74	*/
		&z80::trap_fdcb,			/* 0x75	*/
		&z80::op_tb6iyd,			/* 0x76	*/
		&z80::trap_fdcb,			/* 0x77	*/
		&z80::trap_fdcb,			/* 0x78	*/
		&z80::trap_fdcb,			/* 0x79	*/
		&z80::trap_fdcb,			/* 0x7a	*/
		&z80::trap_fdcb,			/* 0x7b	*/
		&z80::trap_fdcb,			/* 0x7c	*/
		&z80::trap_fdcb,			/* 0x7d	*/
		&z80::op_tb7iyd,			/* 0x7e	*/
		&z80::trap_fdcb,			/* 0x7f	*/
		&z80::trap_fdcb,			/* 0x80	*/
		&z80::trap_fdcb,			/* 0x81	*/
		&z80::trap_fdcb,			/* 0x82	*/
		&z80::trap_fdcb,			/* 0x83	*/
		&z80::trap_fdcb,			/* 0x84	*/
		&z80::trap_fdcb,			/* 0x85	*/
		&z80::op_rb0iyd,			/* 0x86	*/
		&z80::trap_fdcb,			/* 0x87	*/
		&z80::trap_fdcb,			/* 0x88	*/
		&z80::trap_fdcb,			/* 0x89	*/
		&z80::trap_fdcb,			/* 0x8a	*/
		&z80::trap_fdcb,			/* 0x8b	*/
		&z80::trap_fdcb,			/* 0x8c	*/
		&z80::trap_fdcb,			/* 0x8d	*/
		&z80::op_rb1iyd,			/* 0x8e	*/
		&z80::trap_fdcb,			/* 0x8f	*/
		&z80::trap_fdcb,			/* 0x90	*/
		&z80::trap_fdcb,			/* 0x91	*/
		&z80::trap_fdcb,			/* 0x92	*/
		&z80::trap_fdcb,			/* 0x93	*/
		&z80::trap_fdcb,			/* 0x94	*/
		&z80::trap_fdcb,			/* 0x95	*/
		&z80::op_rb2iyd,			/* 0x96	*/
		&z80::trap_fdcb,			/* 0x97	*/
		&z80::trap_fdcb,			/* 0x98	*/
		&z80::trap_fdcb,			/* 0x99	*/
		&z80::trap_fdcb,			/* 0x9a	*/
		&z80::trap_fdcb,			/* 0x9b	*/
		&z80::trap_fdcb,			/* 0x9c	*/
		&z80::trap_fdcb,			/* 0x9d	*/
		&z80::op_rb3iyd,			/* 0x9e	*/
		&z80::trap_fdcb,			/* 0x9f	*/
		&z80::trap_fdcb,			/* 0xa0	*/
		&z80::trap_fdcb,			/* 0xa1	*/
		&z80::trap_fdcb,			/* 0xa2	*/
		&z80::trap_fdcb,			/* 0xa3	*/
		&z80::trap_fdcb,			/* 0xa4	*/
		&z80::trap_fdcb,			/* 0xa5	*/
		&z80::op_rb4iyd,			/* 0xa6	*/
		&z80::trap_fdcb,			/* 0xa7	*/
		&z80::trap_fdcb,			/* 0xa8	*/
		&z80::trap_fdcb,			/* 0xa9	*/
		&z80::trap_fdcb,			/* 0xaa	*/
		&z80::trap_fdcb,			/* 0xab	*/
		&z80::trap_fdcb,			/* 0xac	*/
		&z80::trap_fdcb,			/* 0xad	*/
		&z80::op_rb5iyd,			/* 0xae	*/
		&z80::trap_fdcb,			/* 0xaf	*/
		&z80::trap_fdcb,			/* 0xb0	*/
		&z80::trap_fdcb,			/* 0xb1	*/
		&z80::trap_fdcb,			/* 0xb2	*/
		&z80::trap_fdcb,			/* 0xb3	*/
		&z80::trap_fdcb,			/* 0xb4	*/
		&z80::trap_fdcb,			/* 0xb5	*/
		&z80::op_rb6iyd,			/* 0xb6	*/
		&z80::trap_fdcb,			/* 0xb7	*/
		&z80::trap_fdcb,			/* 0xb8	*/
		&z80::trap_fdcb,			/* 0xb9	*/
		&z80::trap_fdcb,			/* 0xba	*/
		&z80::trap_fdcb,			/* 0xbb	*/
		&z80::trap_fdcb,			/* 0xbc	*/
		&z80::trap_fdcb,			/* 0xbd	*/
		&z80::op_rb7iyd,			/* 0xbe	*/
		&z80::trap_fdcb,			/* 0xbf	*/
		&z80::trap_fdcb,			/* 0xc0	*/
		&z80::trap_fdcb,			/* 0xc1	*/
		&z80::trap_fdcb,			/* 0xc2	*/
		&z80::trap_fdcb,			/* 0xc3	*/
		&z80::trap_fdcb,			/* 0xc4	*/
		&z80::trap_fdcb,			/* 0xc5	*/
		&z80::op_sb0iyd,			/* 0xc6	*/
		&z80::trap_fdcb,			/* 0xc7	*/
		&z80::trap_fdcb,			/* 0xc8	*/
		&z80::trap_fdcb,			/* 0xc9	*/
		&z80::trap_fdcb,			/* 0xca	*/
		&z80::trap_fdcb,			/* 0xcb	*/
		&z80::trap_fdcb,			/* 0xcc	*/
		&z80::trap_fdcb,			/* 0xcd	*/
		&z80::op_sb1iyd,			/* 0xce	*/
		&z80::trap_fdcb,			/* 0xcf	*/
		&z80::trap_fdcb,			/* 0xd0	*/
		&z80::trap_fdcb,			/* 0xd1	*/
		&z80::trap_fdcb,			/* 0xd2	*/
		&z80::trap_fdcb,			/* 0xd3	*/
		&z80::trap_fdcb,			/* 0xd4	*/
		&z80::trap_fdcb,			/* 0xd5	*/
		&z80::op_sb2iyd,			/* 0xd6	*/
		&z80::trap_fdcb,			/* 0xd7	*/
		&z80::trap_fdcb,			/* 0xd8	*/
		&z80::trap_fdcb,			/* 0xd9	*/
		&z80::trap_fdcb,			/* 0xda	*/
		&z80::trap_fdcb,			/* 0xdb	*/
		&z80::trap_fdcb,			/* 0xdc	*/
		&z80::trap_fdcb,			/* 0xdd	*/
		&z80::op_sb3iyd,			/* 0xde	*/
		&z80::trap_fdcb,			/* 0xdf	*/
		&z80::trap_fdcb,			/* 0xe0	*/
		&z80::trap_fdcb,			/* 0xe1	*/
		&z80::trap_fdcb,			/* 0xe2	*/
		&z80::trap_fdcb,			/* 0xe3	*/
		&z80::trap_fdcb,			/* 0xe4	*/
		&z80::trap_fdcb,			/* 0xe5	*/
		&z80::op_sb4iyd,			/* 0xe6	*/
		&z80::trap_fdcb,			/* 0xe7	*/
		&z80::trap_fdcb,			/* 0xe8	*/
		&z80::trap_fdcb,			/* 0xe9	*/
		&z80::trap_fdcb,			/* 0xea	*/
		&z80::trap_fdcb,			/* 0xeb	*/
		&z80::trap_fdcb,			/* 0xec	*/
		&z80::trap_fdcb,			/* 0xed	*/
		&z80::op_sb5iyd,			/* 0xee	*/
		&z80::trap_fdcb,			/* 0xef	*/
		&z80::trap_fdcb,			/* 0xf0	*/
		&z80::trap_fdcb,			/* 0xf1	*/
		&z80::trap_fdcb,			/* 0xf2	*/
		&z80::trap_fdcb,			/* 0xf3	*/
		&z80::trap_fdcb,			/* 0xf4	*/
		&z80::trap_fdcb,			/* 0xf5	*/
		&z80::op_sb6iyd,			/* 0xf6	*/
		&z80::trap_fdcb,			/* 0xf7	*/
		&z80::trap_fdcb,			/* 0xf8	*/
		&z80::trap_fdcb,			/* 0xf9	*/
		&z80::trap_fdcb,			/* 0xfa	*/
		&z80::trap_fdcb,			/* 0xfb	*/
		&z80::trap_fdcb,			/* 0xfc	*/
		&z80::trap_fdcb,			/* 0xfd	*/
		&z80::op_sb7iyd,			/* 0xfe	*/
		&z80::trap_fdcb			/* 0xff	*/
	};

	
z80::z80()
#ifdef WANT_DISASS
:fdis(stderr)
#endif
{
	reset();
};


/*
 *	This function builds the Z80 central processing unit.
 *	The opcode where PC points to is fetched from the memory
 *	and PC incremented by one. The opcode is used as an
 *	index to an array with function pointers, to execute a
 *	function which emulates this Z80 opcode.
 */
 void z80::run(void)
{
	do {
		switch (int_type) {
			case INT_NMI:	/* non maskable interrupt */
				(*this)[--STACK] = (BYTE)(PC >> 8);
				(*this)[--STACK] = (BYTE)PC ;
				nmiack();	// ACK the NMI
				hlt = FALSE;
				int_type = INT_NONE;
				IFF <<= 1;
				PC = 0x66;
				break;
			case INT_INT:	/* maskable interrupt */
				if (IFF != 3)
					break;
				hlt = FALSE;
				IFF = 0;
				int_type = INT_NONE;
				switch (int_mode) {
				case IM0:									
					m1(PC);
					t_states += (this->*op_sim[intrack(IM0)]) ();
					break;
				case IM1:
					(*this)[--STACK] = (BYTE)(PC >> 8);
					(*this)[--STACK] = (BYTE)PC;
					m1(PC);
					intrack(IM1);
					PC = 0x38;
					break;
				case IM2:
					{
					(*this)[--STACK] = (BYTE)(PC >> 8);
					(*this)[--STACK] = (BYTE)PC;
					m1(PC);
					WORD i = ((I << 8) | intrack(IM2)) & 0xfffe;;
					PC = (*this)[i] | ((*this)[i+1] << 8); 
					}
					break;
				}
				break;
			}
		m1(PC);
		t_states += (this->*op_sim[!hlt?(*this)[PC++]:0x00]) ();	/* execute next opcode or NOP if HLT*/
		R++;			/* increment refresh register */
		while(bus)
			Sleep(1);
	} while	(1);
}


int z80::op_nop(void)			/* NOP */
{
	return(4);
}

int z80::op_halt(void)		/* HALT */
{
	hlt = TRUE;
	halt(PC);
	return(op_nop());
}

int z80::op_scf(void)			/* SCF */
{
	F |= C_FLAG;
	F &= ~(N_FLAG |	H_FLAG);
	return(4);
}

int z80::op_ccf(void)			/* CCF */
{
	if (F &	C_FLAG)	{
		F |= H_FLAG;
		F &= ~C_FLAG;
	} else {
		F &= ~H_FLAG;
		F |= C_FLAG;
	}
	F &= ~N_FLAG;
	return(4);
}

int z80::op_cpl(void)			/* CPL */
{
	A = ~A;
	F |= H_FLAG | N_FLAG;
	return(4);
}

int z80::op_daa(void)			/* DAA */
{
	register int old_a;

	old_a =	A;
	if (F &	N_FLAG)	{		/* subtractions */
		if (((A	& 0x0f)	> 9) ||	(F & H_FLAG)) {
			(((old_a & 0x0f) - 6) <	0) ? (F	|= H_FLAG) : (F	&= ~H_FLAG);
			A = old_a -= 6;
		}
		if (((A	& 0xf0)	> 0x90)	|| (F &	C_FLAG)) {
			A -= 0x60;
			if (old_a - 0x60 < 0)
				F |= C_FLAG;
		}
	} else {			/* additions */
		if (((A	& 0x0f)	> 9) ||	(F & H_FLAG)) {
			(((old_a & 0x0f) + 6) >	0x0f) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
			A = old_a += 6;
		}
		if (((A	& 0xf0)	> 0x90)	|| (F &	C_FLAG)) {
			A += 0x60;
			if (old_a + 0x60 > 255)
				F |= C_FLAG;
		}
	}
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	return(4);
}

int z80::op_ei(void)			/* EI */
{
	IFF = 3;
	return(4);
}

int z80::op_di(void)			/* DI */
{
	IFF = 0;
	return(4);
}

int z80::op_in(void)			/* IN A,(n) */
{
	A = io_in((*this)[PC++]);
	return(11);
}

int z80::op_out(void)			/* OUT (n),A */
{
	io_out((*this)[PC++], A);
	return(11);
}

int z80::op_ldan(void)		/* LD A,n */
{
	A = (*this)[PC++];
	return(7);
}

int z80::op_ldbn(void)		/* LD B,n */
{
	B = (*this)[PC++];
	return(7);
}

int z80::op_ldcn(void)		/* LD C,n */
{
	C = (*this)[PC++];
	return(7);
}

int z80::op_lddn(void)		/* LD D,n */
{
	D = (*this)[PC++];
	return(7);
}

int z80::op_lden(void)		/* LD E,n */
{
	E = (*this)[PC++];
	return(7);
}

int z80::op_ldhn(void)		/* LD H,n */
{
	H = (*this)[PC++];
	return(7);
}

int z80::op_ldln(void)		/* LD L,n */
{
	L = (*this)[PC++];
	return(7);
}

int z80::op_ldabc(void)		/* LD A,(BC) */
{
	A = (*this)[(B << 8) + C];
	return(7);
}

int z80::op_ldade(void)		/* LD A,(DE) */
{
	A = (*this)[(D << 8) + E];
	return(7);
}

int z80::op_ldann(void)		/* LD A,(nn) */
{
	register unsigned i;

	i = (*this)[PC++];
	i += (*this)[PC++] << 8;
	A = (*this)[i];
	return(13);
}

int z80::op_ldbca(void)		/* LD (BC),A */
{
	(*this)[(B << 8) + C] =	A;
	return(7);
}

int z80::op_lddea(void)		/* LD (DE),A */
{
	(*this)[(D << 8) + E] =	A;
	return(7);
}

int z80::op_ldnna(void)		/* LD (nn),A */
{
	register unsigned i;

	i = (*this)[PC++];
	i += (*this)[PC++] << 8;
	(*this)[i] = A;
	return(13);
}

int z80::op_ldhla(void)		/* LD (HL),A */
{
	(*this)[(H << 8) + L] =	A;
	return(7);
}

int z80::op_ldhlb(void)		/* LD (HL),B */
{
	(*this)[(H << 8) + L] =	B;
	return(7);
}

int z80::op_ldhlc(void)		/* LD (HL),C */
{
	(*this)[(H << 8) + L] =	C;
	return(7);
}

int z80::op_ldhld(void)		/* LD (HL),D */
{
	(*this)[(H << 8) + L] =	D;
	return(7);
}

int z80::op_ldhle(void)		/* LD (HL),E */
{
	(*this)[(H << 8) + L] =	E;
	return(7);
}

int z80::op_ldhlh(void)		/* LD (HL),H */
{
	(*this)[(H << 8) + L] =	H;
	return(7);
}

int z80::op_ldhll(void)		/* LD (HL),L */
{
	(*this)[(H << 8) + L] =	L;
	return(7);
}

int z80::op_ldhl1(void)		/* LD (HL),n */
{
	(*this)[(H << 8) + L] =	(*this)[PC++];
	return(10);
}

int z80::op_ldaa(void)		/* LD A,A */
{
	return(4);
}

int z80::op_ldab(void)		/* LD A,B */
{
	A = B;
	return(4);
}

int z80::op_ldac(void)		/* LD A,C */
{
	A = C;
	return(4);
}

int z80::op_ldad(void)		/* LD A,D */
{
	A = D;
	return(4);
}

int z80::op_ldae(void)		/* LD A,E */
{
	A = E;
	return(4);
}

int z80::op_ldah(void)		/* LD A,H */
{
	A = H;
	return(4);
}

int z80::op_ldal(void)		/* LD A,L */
{
	A = L;
	return(4);
}

int z80::op_ldahl(void)		/* LD A,(HL) */
{
	A = (*this)[(H << 8) + L];
	return(7);
}

int z80::op_ldba(void)		/* LD B,A */
{
	B = A;
	return(4);
}

int z80::op_ldbb(void)		/* LD B,B */
{
	return(4);
}

int z80::op_ldbc(void)		/* LD B,C */
{
	B = C;
	return(4);
}

int z80::op_ldbd(void)		/* LD B,D */
{
	B = D;
	return(4);
}

int z80::op_ldbe(void)		/* LD B,E */
{
	B = E;
	return(4);
}

int z80::op_ldbh(void)		/* LD B,H */
{
	B = H;
	return(4);
}

int z80::op_ldbl(void)		/* LD B,L */
{
	B = L;
	return(4);
}

int z80::op_ldbhl(void)		/* LD B,(HL) */
{
	B = (*this)[(H << 8) + L];
	return(7);
}

int z80::op_ldca(void)		/* LD C,A */
{
	C = A;
	return(4);
}

int z80::op_ldcb(void)		/* LD C,B */
{
	C = B;
	return(4);
}

int z80::op_ldcc(void)		/* LD C,C */
{
	return(4);
}

int z80::op_ldcd(void)		/* LD C,D */
{
	C = D;
	return(4);
}

int z80::op_ldce(void)		/* LD C,E */
{
	C = E;
	return(4);
}

int z80::op_ldch(void)		/* LD C,H */
{
	C = H;
	return(4);
}

int z80::op_ldcl(void)		/* LD C,L */
{
	C = L;
	return(4);
}

int z80::op_ldchl(void)		/* LD C,(HL) */
{
	C = (*this)[(H << 8) + L];
	return(7);
}

int z80::op_ldda(void)		/* LD D,A */
{
	D = A;
	return(4);
}

int z80::op_lddb(void)		/* LD D,B */
{
	D = B;
	return(4);
}

int z80::op_lddc(void)		/* LD D,C */
{
	D = C;
	return(4);
}

int z80::op_lddd(void)		/* LD D,D */
{
	return(4);
}

int z80::op_ldde(void)		/* LD D,E */
{
	D = E;
	return(4);
}

int z80::op_lddh(void)		/* LD D,H */
{
	D = H;
	return(4);
}

int z80::op_lddl(void)		/* LD D,L */
{
	D = L;
	return(4);
}

int z80::op_lddhl(void)		/* LD D,(HL) */
{
	D = (*this)[(H << 8) + L];
	return(7);
}

int z80::op_ldea(void)		/* LD E,A */
{
	E = A;
	return(4);
}

int z80::op_ldeb(void)		/* LD E,B */
{
	E = B;
	return(4);
}

int z80::op_ldec(void)		/* LD E,C */
{
	E = C;
	return(4);
}

int z80::op_lded(void)		/* LD E,D */
{
	E = D;
	return(4);
}

int z80::op_ldee(void)		/* LD E,E */
{
	return(4);
}

int z80::op_ldeh(void)		/* LD E,H */
{
	E = H;
	return(4);
}

int z80::op_ldel(void)		/* LD E,L */
{
	E = L;
	return(4);
}

int z80::op_ldehl(void)		/* LD E,(HL) */
{
	E = (*this)[(H << 8) + L];
	return(7);
}

int z80::op_ldha(void)		/* LD H,A */
{
	H = A;
	return(4);
}

int z80::op_ldhb(void)		/* LD H,B */
{
	H = B;
	return(4);
}

int z80::op_ldhc(void)		/* LD H,C */
{
	H = C;
	return(4);
}

int z80::op_ldhd(void)		/* LD H,D */
{
	H = D;
	return(4);
}

int z80::op_ldhe(void)		/* LD H,E */
{
	H = E;
	return(4);
}

int z80::op_ldhh(void)		/* LD H,H */
{
	return(4);
}

int z80::op_ldhl(void)		/* LD H,L */
{
	H = L;
	return(4);
}

int z80::op_ldhhl(void)		/* LD H,(HL) */
{
	H = (*this)[(H << 8) + L];
	return(7);
}

int z80::op_ldla(void)		/* LD L,A */
{
	L = A;
	return(4);
}

int z80::op_ldlb(void)		/* LD L,B */
{
	L = B;
	return(4);
}

int z80::op_ldlc(void)		/* LD L,C */
{
	L = C;
	return(4);
}

int z80::op_ldld(void)		/* LD L,D */
{
	L = D;
	return(4);
}

int z80::op_ldle(void)		/* LD L,E */
{
	L = E;
	return(4);
}

int z80::op_ldlh(void)		/* LD L,H */
{
	L = H;
	return(4);
}

int z80::op_ldll(void)		/* LD L,L */
{
	return(4);
}

int z80::op_ldlhl(void)		/* LD L,(HL) */
{
	L = (*this)[(H << 8) + L];
	return(7);
}

int z80::op_ldbcnn(void)		/* LD BC,nn */
{
	C = (*this)[PC++];
	B = (*this)[PC++];
	return(10);
}

int z80::op_lddenn(void)		/* LD DE,nn */
{
	E = (*this)[PC++];
	D = (*this)[PC++];
	return(10);
}

int z80::op_ldhlnn(void)		/* LD HL,nn */
{
	L = (*this)[PC++];
	H = (*this)[PC++];
	return(10);
}

int z80::op_ldspnn(void)		/* LD SP,nn */
{
	STACK =	 (*this)[PC++];
	STACK += (*this)[PC++] << 8;
	return(10);
}

int z80::op_ldsphl(void)		/* LD SP,HL */
{
	STACK =	(H << 8) + L;
	return(6);
}

int z80::op_ldhlin(void)		/* LD HL,(nn) */
{
	register unsigned i;

	i = (*this)[PC++];
	i += (*this)[PC++] << 8;
	L = (*this)[i];
	H = (*this)[i +	1];
	return(16);
}

int z80::op_ldinhl(void)		/* LD (nn),HL */
{
	register unsigned i;

	i = (*this)[PC++];
	i += (*this)[PC++] << 8;
	(*this)[i] = L;
	(*this)[i + 1] = H;
	return(16);
}

int z80::op_incbc(void)		/* INC BC */
{
	C++;
	if (!C)
		B++;
	return(6);
}

int z80::op_incde(void)		/* INC DE */
{
	E++;
	if (!E)
		D++;
	return(6);
}

int z80::op_inchl(void)		/* INC HL */
{
	L++;
	if (!L)
		H++;
	return(6);
}

int z80::op_incsp(void)		/* INC SP */
{
	STACK++;
	return(6);
}

int z80::op_decbc(void)		/* DEC BC */
{
	C--;
	if (C == 0xff)
		B--;
	return(6);
}

int z80::op_decde(void)		/* DEC DE */
{
	E--;
	if (E == 0xff)
		D--;
	return(6);
}

int z80::op_dechl(void)		/* DEC HL */
{
	L--;
	if (L == 0xff)
		H--;
	return(6);
}

int z80::op_decsp(void)		/* DEC SP */
{
	STACK--;
	return(6);
}

int z80::op_adhlbc(void)		/* ADD HL,BC */
{
	register int carry;

	carry = (L + C > 255) ? 1 : 0;
	L += C;
	((H & 0xf) + (B & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(H + B + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	H += B + carry;
	F &= ~N_FLAG;
	return(11);
}

int z80::op_adhlde(void)		/* ADD HL,DE */
{
	register int carry;

	carry = (L + E > 255) ? 1 : 0;
	L += E;
	((H & 0xf) + (D & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(H + D + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	H += D + carry;
	F &= ~N_FLAG;
	return(11);
}

int z80::op_adhlhl(void)		/* ADD HL,HL */
{
	register int carry;

	carry = (L << 1 > 255) ? 1 : 0;
	L <<= 1;
	((H & 0xf) + (H & 0xf) + carry > 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(H + H + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	H += H + carry;
	F &= ~N_FLAG;
	return(11);
}

int z80::op_adhlsp(void)		/* ADD HL,SP */
{
	register int carry;
	BYTE spl = STACK & 0xff;
	BYTE sph = STACK >> 8;
	
	carry = (L + spl > 255) ? 1 : 0;
	L += spl;
	((H & 0xf) + (sph & 0xf) + carry > 0xf) ? (F |= H_FLAG)
						: (F &= ~H_FLAG);
	(H + sph + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	H += sph + carry;
	F &= ~N_FLAG;
	return(11);
}

int z80::op_anda(void)		/* AND A */
{
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(4);
}

int z80::op_andb(void)		/* AND B */
{
	A &= B;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(4);
}

int z80::op_andc(void)		/* AND C */
{
	A &= C;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(4);
}

int z80::op_andd(void)		/* AND D */
{
	A &= D;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(4);
}

int z80::op_ande(void)		/* AND E */
{
	A &= E;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(4);
}

int z80::op_andh(void)		/* AND H */
{
	A &= H;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(4);
}

int z80::op_andl(void)		/* AND L */
{
	A &= L;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(4);
}

int z80::op_andhl(void)		/* AND (HL) */
{
	A &= (*this)[(H	<< 8) +	L];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(7);
}

int z80::op_andn(void)		/* AND n */
{
	A &= (*this)[PC++];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= H_FLAG;
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(N_FLAG |	C_FLAG);
	return(7);
}

int z80::op_ora(void)			/* OR A */
{
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_orb(void)			/* OR B */
{
	A |= B;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_orc(void)			/* OR C */
{
	A |= C;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_ord(void)			/* OR D */
{
	A |= D;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_ore(void)			/* OR E */
{
	A |= E;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_orh(void)			/* OR H */
{
	A |= H;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_orl(void)			/* OR L */
{
	A |= L;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_orhl(void)		/* OR (HL)	*/
{
	A |= (*this)[(H	<< 8) +	L];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(7);
}

int z80::op_orn(void)			/* OR n */
{
	A |= (*this)[PC++];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(7);
}

int z80::op_xora(void)		/* XOR A */
{
	A = 0;
	F &= ~(S_FLAG |	H_FLAG | N_FLAG	| C_FLAG);
	F |= Z_FLAG | P_FLAG;
	return(4);
}

int z80::op_xorb(void)		/* XOR B */
{
	A ^= B;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_xorc(void)		/* XOR C */
{
	A ^= C;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_xord(void)		/* XOR D */
{
	A ^= D;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_xore(void)		/* XOR E */
{
	A ^= E;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_xorh(void)		/* XOR H */
{
	A ^= H;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_xorl(void)		/* XOR L */
{
	A ^= L;
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(4);
}

int z80::op_xorhl(void)		/* XOR (HL) */
{
	A ^= (*this)[(H << 8) + L];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(7);
}

int z80::op_xorn(void)		/* XOR n */
{
	A ^= (*this)[PC++];
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	(parrity[A]) ? (F &= ~P_FLAG) :	(F |= P_FLAG);
	F &= ~(H_FLAG |	N_FLAG | C_FLAG);
	return(7);
}

int z80::op_adda(void)		/* ADD A,A	*/
{
	register int i;

	((A & 0xf) + (A	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	((A << 1) > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	A = i =	((char)	A) << 1;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_addb(void)		/* ADD A,B	*/
{
	register int i;

	((A & 0xf) + (B	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + B > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) B;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_addc(void)		/* ADD A,C	*/
{
	register int i;

	((A & 0xf) + (C	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + C > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) C;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_addd(void)		/* ADD A,Dn*/
{
	register int i;

	((A & 0xf) + (D	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + D > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) D;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_adde(void)		/* ADD A,E */
{
	register int i;

	((A & 0xf) + (E	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + E > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) E;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_addh(void)		/* ADD A,H */
{
	register int i;

	((A & 0xf) + (H	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + H > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) H;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_addl(void)		/* ADD A,L */
{
	register int i;

	((A & 0xf) + (L	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + L > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) L;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_addhl(void)		/* ADD A,(HL) */
{
	register int i;
	register BYTE P;

	P = (*this)[(H << 8) + L];
	((A & 0xf) + (P	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + P > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(7);
}

int z80::op_addn(void)		/* ADD A,n */
{
	register int i;
	register BYTE P;

	P = (*this)[PC++];
	((A & 0xf) + (P	& 0xf) > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + P > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(7);
}

int z80::op_adca(void)		/* ADC A,A */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((A & 0xf) + (A	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	((A << 1) + carry > 255) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	A = i =	(((char) A) << 1) + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_adcb(void)		/* ADC A,B */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((A & 0xf) + (B	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + B + carry > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) B + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_adcc(void)		/* ADC A,C */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((A & 0xf) + (C	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + C + carry > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) C + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_adcd(void)		/* ADC A,D */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((A & 0xf) + (D	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + D + carry > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) D + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_adce(void)		/* ADC A,E */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((A & 0xf) + (E	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + E + carry > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) E + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_adch(void)		/* ADC A,H */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((A & 0xf) + (H	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + H + carry > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) H + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_adcl(void)		/* ADC A,L */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((A & 0xf) + (L	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + L + carry > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) L + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_adchl(void)		/* ADC A,(HL) */
{
	register int i,	carry;
	register BYTE P;

	P = (*this)[(H << 8) + L];
	carry =	(F & C_FLAG) ? 1 : 0;
	((A & 0xf) + (P	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + P + carry > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) P + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(7);
}

int z80::op_adcn(void)		/* ADC A,n */
{
	register int i,	carry;
	register BYTE P;

	carry =	(F & C_FLAG) ? 1 : 0;
	P = (*this)[PC++];
	((A & 0xf) + (P	& 0xf) + carry > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	(A + P + carry > 255) ?	(F |= C_FLAG) :	(F &= ~C_FLAG);
	A = i =	(char) A + (char) P + carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(7);
}

int z80::op_suba(void)		/* SUB A,A */
{
	A = 0;
	F &= ~(S_FLAG |	H_FLAG | P_FLAG	| C_FLAG);
	F |= Z_FLAG | N_FLAG;
	return(4);
}

int z80::op_subb(void)		/* SUB A,B */
{
	register int i;

	((B & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(B > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) B;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_subc(void)		/* SUB A,C */
{
	register int i;

	((C & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(C > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) C;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_subd(void)		/* SUB A,D */
{
	register int i;

	((D & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(D > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) D;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_sube(void)		/* SUB A,E */
{
	register int i;

	((E & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(E > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) E;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_subh(void)		/* SUB A,H */
{
	register int i;

	((H & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(H > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) H;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_subl(void)		/* SUB A,L */
{
	register int i;

	((L & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(L > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) L;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_subhl(void)		/* SUB A,(HL) */
{
	register int i;
	register BYTE P;

	P = (*this)[(H << 8) + L];
	((P & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(7);
}

int z80::op_subn(void)		/* SUB A,n */
{
	register int i;
	register BYTE P;

	P = (*this)[PC++];
	((P & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(7);
}

int z80::op_sbca(void)		/* SBC A,A */
{
	if (F &	C_FLAG)	{
		F |= S_FLAG | H_FLAG | N_FLAG |	C_FLAG;
		F &= ~(Z_FLAG |	P_FLAG);
		A = 255;
	} else {
		F |= Z_FLAG | N_FLAG;
		F &= ~(S_FLAG |	H_FLAG | P_FLAG	| C_FLAG);
		A = 0;
	}
	return(4);
}

int z80::op_sbcb(void)		/* SBC A,B */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((B & 0xf) + carry > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(B + carry > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) B - carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_sbcc(void)		/* SBC A,C */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((C & 0xf) + carry > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(C + carry > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) C - carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_sbcd(void)		/* SBC A,D */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((D & 0xf) + carry > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(D + carry > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) D - carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_sbce(void)		/* SBC A,E */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((E & 0xf) + carry > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(E + carry > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) E - carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_sbch(void)		/* SBC A,H */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((H & 0xf) + carry > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(H + carry > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) H - carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_sbcl(void)		/* SBC A,L */
{
	register int i,	carry;

	carry =	(F & C_FLAG) ? 1 : 0;
	((L & 0xf) + carry > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(L + carry > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) L - carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_sbchl(void)		/* SBC A,(HL) */
{
	register int i,	carry;
	register BYTE P;

	P = (*this)[(H << 8) + L];
	carry =	(F & C_FLAG) ? 1 : 0;
	((P & 0xf) + carry > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P + carry > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) P - carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(7);
}

int z80::op_sbcn(void)		/* SBC A,n */
{
	register int i,	carry;
	register BYTE P;

	P = (*this)[PC++];
	carry =	(F & C_FLAG) ? 1 : 0;
	((P & 0xf) + carry > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P + carry > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	A = i =	(char) A - (char) P - carry;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(7);
}

int z80::op_cpa(void)			/* CP A */
{
	F &= ~(S_FLAG |	H_FLAG | P_FLAG	| C_FLAG);
	F |= Z_FLAG | N_FLAG;
	return(4);
}

int z80::op_cpb(void)			/* CP B */
{
	register int i;

	((B & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(B > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	i = (char) A - (char) B;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_cpc(void)			/* CP C */
{
	register int i;

	((C & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(C > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	i = (char) A - (char) C;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_cpd(void)			/* CP D */
{
	register int i;

	((D & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(D > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	i = (char) A - (char) D;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_cpe(void)			/* CP E */
{
	register int i;

	((E & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(E > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	i = (char) A - (char) E;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_cph(void)			/* CP H */
{
	register int i;

	((H & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(H > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	i = (char) A - (char) H;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_cplr(void)		/* CP L */
{
	register int i;

	((L & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(L > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	i = (char) A - (char) L;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_cphl(void)		/* CP (HL) */
{
	register int i;
	register BYTE P;

	P = (*this)[(H << 8) + L];
	((P & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	i = (char) A - (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(7);
}

int z80::op_cpn(void)			/* CP n */
{
	register int i;
	register BYTE P;

	P = (*this)[PC++];
	((P & 0xf) > (A	& 0xf))	? (F |=	H_FLAG)	: (F &=	~H_FLAG);
	(P > A)	? (F |=	C_FLAG)	: (F &=	~C_FLAG);
	i = (char) A - (char) P;
	(i < -128 || i > 127) ?	(F |= P_FLAG) :	(F &= ~P_FLAG);
	(i & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(i) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(7);
}

int z80::op_inca(void)		/* INC A */
{
	((A & 0xf) + 1 > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	A++;
	(A == 128) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_incb(void)		/* INC B */
{
	((B & 0xf) + 1 > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	B++;
	(B == 128) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(B & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_incc(void)		/* INC C */
{
	((C & 0xf) + 1 > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	C++;
	(C == 128) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(C & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(C) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_incd(void)		/* INC D */
{
	((D & 0xf) + 1 > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	D++;
	(D == 128) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(D & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(D) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_ince(void)		/* INC E */
{
	((E & 0xf) + 1 > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	E++;
	(E == 128) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(E & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(E) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_inch(void)		/* INC H */
{
	((H & 0xf) + 1 > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	H++;
	(H == 128) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(H) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_incl(void)		/* INC L */
{
	((L & 0xf) + 1 > 0xf) ?	(F |= H_FLAG) :	(F &= ~H_FLAG);
	L++;
	(L == 128) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(L & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(L) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F &= ~N_FLAG;
	return(4);
}

int z80::op_incihl(void)		/* INC (HL) */
{
	register  WORD p;

	p = (H << 8) + L;
	(((*this)[p] & 0xf) + 1	> 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(*this)[p]++;
	((*this)[p] == 128) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	((*this)[p]  & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(*this)[p] ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	F &= ~N_FLAG;
	return(11);
}

int z80::op_deca(void)		/* DEC A */
{
	(((A - 1) & 0xf) == 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	A--;
	(A == 127) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(A & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(A) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_decb(void)		/* DEC B */
{
	(((B - 1) & 0xf) == 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	B--;
	(B == 127) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(B & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(B) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_decc(void)		/* DEC C */
{
	(((C - 1) & 0xf) == 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	C--;
	(C == 127) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(C & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(C) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_decd(void)		/* DEC D */
{
	(((D - 1) & 0xf) == 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	D--;
	(D == 127) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(D & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(D) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_dece(void)		/* DEC E */
{
	(((E - 1) & 0xf) == 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	E--;
	(E == 127) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(E & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(E) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_dech(void)		/* DEC H */
{
	(((H - 1) & 0xf) == 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	H--;
	(H == 127) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(H & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(H) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_decl(void)		/* DEC L */
{
	(((L - 1) & 0xf) == 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	L--;
	(L == 127) ? (F	|= P_FLAG) : (F	&= ~P_FLAG);
	(L & 128) ? (F |= S_FLAG) : (F &= ~S_FLAG);
	(L) ? (F &= ~Z_FLAG) : (F |= Z_FLAG);
	F |= N_FLAG;
	return(4);
}

int z80::op_decihl(void)		/* DEC (HL) */
{
	register WORD p;

	p = (H <<	8) + L;
	(((*this)[p] - 1 & 0xf) == 0xf) ? (F |= H_FLAG) : (F &= ~H_FLAG);
	(*this)[p]--;
	((*this)[p] == 127) ? (F |= P_FLAG) : (F &= ~P_FLAG);
	((*this)[p] & 128) ? (F	|= S_FLAG) : (F	&= ~S_FLAG);
	(*this)[p] ? (F &= ~Z_FLAG) :	(F |= Z_FLAG);
	F |= N_FLAG;
	return(11);
}

int z80::op_rlca(void)		/* RLCA */
{
	register int i;

	i = (A & 128) ?	1 : 0;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	A <<= 1;
	A |= i;
	return(4);
}

int z80::op_rrca(void)		/* RRCA */
{
	register int i;

	i = A &	1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	A >>= 1;
	if (i) A |= 128;
	return(4);
}

int z80::op_rla(void)			/* RLA */
{
	register int old_c_flag;

	old_c_flag = F & C_FLAG;
	(A & 128) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	A <<= 1;
	if (old_c_flag)	A |= 1;
	return(4);
}

int z80::op_rra(void)			/* RRA */
{
	register int i,	old_c_flag;

	old_c_flag = F & C_FLAG;
	i = A &	1;
	(i) ? (F |= C_FLAG) : (F &= ~C_FLAG);
	F &= ~(H_FLAG |	N_FLAG);
	A >>= 1;
	if (old_c_flag)	A |= 128;
	return(4);
}

int z80::op_exdehl(void)		/* EX DE,HL */
{
	register unsigned i;

	i = D;
	D = H;
	H = i;
	i = E;
	E = L;
	L = i;
	return(4);
}

int z80::op_exafaf(void)		/* EX AF,AF' */
{
	register unsigned i;

	i = A;
	A = A_;
	A_ = i;
	i = F;
	F = F_;
	F_ = i;
	return(4);
}

int z80::op_exx(void)			/* EXX */
{
	register unsigned i;

	i = B;
	B = B_;
	B_ = i;
	i = C;
	C = C_;
	C_ = i;
	i = D;
	D = D_;
	D_ = i;
	i = E;
	E = E_;
	E_ = i;
	i = H;
	H = H_;
	H_ = i;
	i = L;
	L = L_;
	L_ = i;
	return(4);
}

int z80::op_exsphl(void)		/* EX (SP),HL */
{
	register int i;

	i = (*this)[STACK];
	(*this)[STACK] = L;
	L = i;
	i = (*this)[STACK + 1];
	(*this)[STACK + 1] = H;
	H = i;
	return(19);
}

int z80::op_pushaf(void)		/* PUSH AF */
{
	(*this)[--STACK] = A;
	(*this)[--STACK] = F;
	return(11);
}

int z80::op_pushbc(void)		/* PUSH BC */
{
	(*this)[--STACK] = B;
	(*this)[--STACK] = C;
	return(11);
}

int z80::op_pushde(void)		/* PUSH DE */
{
	(*this)[--STACK] = D;
	(*this)[--STACK] = E;
	return(11);
}

int z80::op_pushhl(void)		/* PUSH HL */
{
	(*this)[--STACK] = H;
	(*this)[--STACK] = L;
	return(11);
}

int z80::op_popaf(void)		/* POP AF */
{
	F = (*this)[STACK++];
	A = (*this)[STACK++];
	return(10);
}

int z80::op_popbc(void)		/* POP BC */
{
	C = (*this)[STACK++];
	B = (*this)[STACK++];
	return(10);
}

int z80::op_popde(void)		/* POP DE */
{
	E = (*this)[STACK++];
	D = (*this)[STACK++];
	return(10);
}

int z80::op_pophl(void)		/* POP HL */
{
	L = (*this)[STACK++];
	H = (*this)[STACK++];
	return(10);
}

int z80::op_jp(void)			/* JP */
{
	register unsigned i;

	i = (*this)[PC++];
	i += (*this)[PC] << 8;
	PC = i;
	return(10);
}

int z80::op_jphl(void)		/* JP (HL) */
{
	PC = (H << 8) + L;
	return(4);
}

int z80::op_jr(void)			/* JR */
{
	PC += (char) (*this)[PC] + 1;
	return(12);
}

int z80::op_djnz(void)		/* DJNZ */
{
	if (--B) {
		PC += (char) (*this)[PC] + 1;
		return(13);
	} else {
		PC++;
		return(8);
	}
}

int z80::op_call(void)		/* CALL */
{
	register unsigned i;

	i = (*this)[PC++];
	i += (*this)[PC++] << 8;
	(*this)[--STACK] = (BYTE)(PC >> 8);
	(*this)[--STACK] = (BYTE)PC;
	PC = i;
	return(17);
}

int z80::op_ret(void)			/* RET */
{
	register unsigned i;

	i = (*this)[STACK++];
	i += (*this)[STACK++] << 8;
	PC = i;
	return(10);
}

int z80::op_jpz(void)			/* JP Z,nn */
{
	register unsigned i;

	if (F &	Z_FLAG)	{
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		PC = i;
	} else
		PC += 2;
	return(10);
}

int z80::op_jpnz(void)		/* JP NZ,nn */
{
	register unsigned i;

	if (!(F	& Z_FLAG)) {
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		PC = i;
	} else
		PC += 2;
	return(10);
}

int z80::op_jpc(void)			/* JP C,nn */
{
	register unsigned i;

	if (F &	C_FLAG)	{
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		PC = i;
	} else
		PC += 2;
	return(10);
}

int z80::op_jpnc(void)		/* JP NC,nn */
{
	register unsigned i;

	if (!(F	& C_FLAG)) {
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		PC = i;
	} else
		PC += 2;
	return(10);
}

int z80::op_jppe(void)		/* JP PE,nn */
{
	register unsigned i;

	if (F &	P_FLAG)	{
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		PC = i;
	} else
		PC += 2;
	return(10);
}

int z80::op_jppo(void)		/* JP PO,nn */
{
	register unsigned i;

	if (!(F	& P_FLAG)) {
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		PC = i;
	} else
		PC += 2;
	return(10);
}

int z80::op_jpm(void)			/* JP M,nn */
{
	register unsigned i;

	if (F &	S_FLAG)	{
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		PC = i;
	} else
		PC += 2;
	return(10);
}

int z80::op_jpp(void)			/* JP P,nn */
{
	register unsigned i;

	if (!(F	& S_FLAG)) {
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		PC = i;
	} else
		PC += 2;
	return(10);
}

int z80::op_calz(void)		/* CALL Z,nn */
{
	register unsigned i;

	if (F &	Z_FLAG)	{
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		(*this)[--STACK] = (BYTE)(PC >> 8);
		(*this)[--STACK] = (BYTE)(PC);
		PC = i;
		return(17);
	} else {
		PC += 2;
		return(10);
	}
}

int z80::op_calnz(void)		/* CALL NZ,nn */
{
	register unsigned i;

	if (!(F	& Z_FLAG)) {
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		(*this)[--STACK] = (BYTE)(PC >> 8);
		(*this)[--STACK] = (BYTE) PC;
		PC = i;
		return(17);
	} else {
		PC += 2;
		return(10);
	}
}

int z80::op_calc(void)		/* CALL C,nn */
{
	register unsigned i;

	if (F &	C_FLAG)	{
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		(*this)[--STACK] = PC >> 8;
		(*this)[--STACK] = (BYTE)PC;
		PC = i;
		return(17);
	} else {
		PC += 2;
		return(10);
	}
}

int z80::op_calnc(void)		/* CALL NC,nn */
{
	register unsigned i;

	if (!(F	& C_FLAG)) {
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		(*this)[--STACK] = PC >> 8;
		(*this)[--STACK] = (BYTE)PC;
		PC = i;
		return(17);
	} else {
		PC += 2;
		return(10);
	}
}

int z80::op_calpe(void)		/* CALL PE,nn */
{
	register unsigned i;

	if (F &	P_FLAG)	{
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		(*this)[--STACK] = PC >> 8;
		(*this)[--STACK] = (BYTE)PC;
		PC = i;
		return(17);
	} else {
		PC += 2;
		return(10);
	}
}

int z80::op_calpo(void)		/* CALL PO,nn */
{
	register unsigned i;

	if (!(F	& P_FLAG)) {
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		(*this)[--STACK] = PC >> 8;
		(*this)[--STACK] = (BYTE)PC;
		PC = i;
		return(17);
	} else {
		PC += 2;
		return(10);
	}
}

int z80::op_calm(void)		/* CALL M,nn */
{
	register unsigned i;

	if (F &	S_FLAG)	{
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		(*this)[--STACK] = PC >> 8;
		(*this)[--STACK] = (BYTE)PC;
		PC = i;
		return(17);
	} else {
		PC += 2;
		return(10);
	}
}

int z80::op_calp(void)		/* CALL P,nn */
{
	register unsigned i;

	if (!(F	& S_FLAG)) {
		i = (*this)[PC++];
		i += (*this)[PC++] << 8;
		(*this)[--STACK] = PC >> 8;
		(*this)[--STACK] = (BYTE)PC;
		PC = i;
		return(17);
	} else {
		PC += 2;
		return(10);
	}
}

int z80::op_retz(void)		/* RET Z */
{
	register unsigned i;

	if (F &	Z_FLAG)	{
		i = (*this)[STACK++];
		i += (*this)[STACK++] << 8;
		PC = i;
		return(11);
	} else {
		return(5);
	}
}

int z80::op_retnz(void)		/* RET NZ */
{
	register unsigned i;

	if (!(F	& Z_FLAG)) {
		i = (*this)[STACK++];
		i += (*this)[STACK++] << 8;
		PC = i;
		return(11);
	} else {
		return(5);
	}
}

int z80::op_retc(void)		/* RET C */
{
	register unsigned i;

	if (F &	C_FLAG)	{
		i = (*this)[STACK++];
		i += (*this)[STACK++] << 8;
		PC = i;
		return(11);
	} else {
		return(5);
	}
}

int z80::op_retnc(void)		/* RET NC */
{
	register unsigned i;

	if (!(F	& C_FLAG)) {
		i = (*this)[STACK++];
		i += (*this)[STACK++] << 8;
		PC = i;
		return(11);
	} else {
		return(5);
	}
}

int z80::op_retpe(void)		/* RET PE */
{
	register unsigned i;

	if (F &	P_FLAG)	{
		i = (*this)[STACK++];
		i += (*this)[STACK++] << 8;
		PC = i;
		return(11);
	} else {
		return(5);
	}
}

int z80::op_retpo(void)		/* RET PO */
{
	register unsigned i;

	if (!(F	& P_FLAG)) {
		i = (*this)[STACK++];
		i += (*this)[STACK++] << 8;
		PC = i;
		return(11);
	} else {
		return(5);
	}
}

int z80::op_retm(void)		/* RET M */
{
	register unsigned i;

	if (F &	S_FLAG)	{
		i = (*this)[STACK++];
		i += (*this)[STACK++] << 8;
		PC = i;
		return(11);
	} else {
		return(5);
	}
}

int z80::op_retp(void)		/* RET P */
{
	register unsigned i;

	if (!(F	& S_FLAG)) {
		i = (*this)[STACK++];
		i += (*this)[STACK++] << 8;
		PC = i;
		return(11);
	} else {
		return(5);
	}
}

int z80::op_jrz(void)			/* JR Z,n */
{
	if (F &	Z_FLAG)	{
		PC += (char) (*this)[PC] + 1;
		return(12);
	} else {
		PC++;
		return(7);
	}
}

int z80::op_jrnz(void)		/* JR NZ,n */
{
	if (!(F	& Z_FLAG)) {
		PC += (char) (*this)[PC] + 1;
		return(12);
	} else {
		PC++;
		return(7);
	}
}

int z80::op_jrc(void)			/* JR C,n */
{
	if (F &	C_FLAG)	{
		PC += (char) (*this)[PC] + 1;
		return(12);
	} else {
		PC++;
		return(7);
	}
}

int z80::op_jrnc(void)		/* JR NC,n */
{
	if (!(F	& C_FLAG)) {
		PC += (char) (*this)[PC] + 1;
		return(12);
	} else {
		PC++;
		return(7);
	}
}

int z80::op_rst00(void)		/* RST 00 */
{
	(*this)[--STACK] = PC >> 8;
	(*this)[--STACK] = (BYTE)PC;
	PC = 0;
	return(11);
}

int z80::op_rst08(void)		/* RST 08 */
{
	(*this)[--STACK] = PC >> 8;
	(*this)[--STACK] = (BYTE)PC;
	PC = 0x08;
	return(11);
}

int z80::op_rst10(void)		/* RST 10 */
{
	(*this)[--STACK] = PC >> 8;
	(*this)[--STACK] = (BYTE)PC;
	PC = 0x10;
	return(11);
}

int z80::op_rst18(void)		/* RST 18 */
{
	(*this)[--STACK] = PC >> 8;
	(*this)[--STACK] = (BYTE)PC;
	PC = 0x18;
	return(11);
}

int z80::op_rst20(void)		/* RST 20 */
{
	(*this)[--STACK] = PC >> 8;
	(*this)[--STACK] = (BYTE)PC;
	PC = 0x20;
	return(11);
}

int z80::op_rst28(void)		/* RST 28 */
{
	(*this)[--STACK] = PC >> 8;
	(*this)[--STACK] = (BYTE)PC;
	PC = 0x28;
	return(11);
}

int z80::op_rst30(void)		/* RST 30 */
{
	(*this)[--STACK] = PC >> 8;
	(*this)[--STACK] = (BYTE)PC;
	PC = 0x30;
	return(11);
}

int z80::op_rst38(void)		/* RST 38 */
{
	(*this)[--STACK] = PC >> 8;
	(*this)[--STACK] = (BYTE)PC;
	PC = 0x38;
	return(11);
}
/*
 *	This function saves the CPU and the memory into the file core.z80
 *
 */
void z80::save_core(void)
{
	int fd;

	if ((fd	= open("core.z80", O_WRONLY | O_CREAT, 0600)) == -1) {
		puts("can't open file core.z80");
		return;
	}
	write(fd, (char	*) &A, sizeof(A));
	write(fd, (char	*) &F, sizeof(F));
	write(fd, (char	*) &B, sizeof(B));
	write(fd, (char	*) &C, sizeof(C));
	write(fd, (char	*) &D, sizeof(D));
	write(fd, (char	*) &E, sizeof(E));
	write(fd, (char	*) &H, sizeof(H));
	write(fd, (char	*) &L, sizeof(L));
	write(fd, (char	*) &A_,	sizeof(A_));
	write(fd, (char	*) &F_,	sizeof(F_));
	write(fd, (char	*) &B_,	sizeof(B_));
	write(fd, (char	*) &C_,	sizeof(C_));
	write(fd, (char	*) &D_,	sizeof(D_));
	write(fd, (char	*) &E_,	sizeof(E_));
	write(fd, (char	*) &H_,	sizeof(H_));
	write(fd, (char	*) &L_,	sizeof(L_));
	write(fd, (char	*) &I, sizeof(I));
	write(fd, (char	*) &IFF, sizeof(IFF));
	write(fd, (char	*) &R, sizeof(R));
	write(fd, (char	*) &PC,	sizeof(PC));
	write(fd, (char	*) &STACK, sizeof(STACK));
	write(fd, (char	*) &IX,	sizeof(IX));
	write(fd, (char	*) &IY,	sizeof(IY));
	close(fd);
};

/*
 *	This function loads the CPU and memory from the file core.z80
 *
 */
void z80::load_core(void)
{
	int fd;

	if ((fd	= open("core.z80", O_RDONLY)) == -1) {
		puts("can't open file core.z80");
		return;
	}
	read(fd, (char *) &A, sizeof(A));
	read(fd, (char *) &F, sizeof(F));
	read(fd, (char *) &B, sizeof(B));
	read(fd, (char *) &C, sizeof(C));
	read(fd, (char *) &D, sizeof(D));
	read(fd, (char *) &E, sizeof(E));
	read(fd, (char *) &H, sizeof(H));
	read(fd, (char *) &L, sizeof(L));
	read(fd, (char *) &A_, sizeof(A_));
	read(fd, (char *) &F_, sizeof(F_));
	read(fd, (char *) &B_, sizeof(B_));
	read(fd, (char *) &C_, sizeof(C_));
	read(fd, (char *) &D_, sizeof(D_));
	read(fd, (char *) &E_, sizeof(E_));
	read(fd, (char *) &H_, sizeof(H_));
	read(fd, (char *) &L_, sizeof(L_));
	read(fd, (char *) &I, sizeof(I));
	read(fd, (char *) &IFF,	sizeof(IFF));
	read(fd, (char *) &R, sizeof(R));
	read(fd, (char *) &PC, sizeof(PC));
	read(fd, (char *) &STACK, sizeof(STACK));
	read(fd, (char *) &IX, sizeof(IX));
	read(fd, (char *) &IY, sizeof(IY));
	close(fd);
};

/*
 *	Table to get parrity as fast as possible
 */
const int z80::parrity[256] = {
		0 /* 00000000 */, 1 /* 00000001	*/, 1 /* 00000010 */,
		0 /* 00000011 */, 1 /* 00000100	*/, 0 /* 00000101 */,
		0 /* 00000110 */, 1 /* 00000111	*/, 1 /* 00001000 */,
		0 /* 00001001 */, 0 /* 00001010	*/, 1 /* 00001011 */,
		0 /* 00001100 */, 1 /* 00001101	*/, 1 /* 00001110 */,
		0 /* 00001111 */, 1 /* 00010000	*/, 0 /* 00010001 */,
		0 /* 00010010 */, 1 /* 00010011	*/, 0 /* 00010100 */,
		1 /* 00010101 */, 1 /* 00010110	*/, 0 /* 00010111 */,
		0 /* 00011000 */, 1 /* 00011001	*/, 1 /* 00011010 */,
		0 /* 00011011 */, 1 /* 00011100	*/, 0 /* 00011101 */,
		0 /* 00011110 */, 1 /* 00011111	*/, 1 /* 00100000 */,
		0 /* 00100001 */, 0 /* 00100010	*/, 1 /* 00100011 */,
		0 /* 00100100 */, 1 /* 00100101	*/, 1 /* 00100110 */,
		0 /* 00100111 */, 0 /* 00101000	*/, 1 /* 00101001 */,
		1 /* 00101010 */, 0 /* 00101011	*/, 1 /* 00101100 */,
		0 /* 00101101 */, 0 /* 00101110	*/, 1 /* 00101111 */,
		0 /* 00110000 */, 1 /* 00110001	*/, 1 /* 00110010 */,
		0 /* 00110011 */, 1 /* 00110100	*/, 0 /* 00110101 */,
		0 /* 00110110 */, 1 /* 00110111	*/, 1 /* 00111000 */,
		0 /* 00111001 */, 0 /* 00111010	*/, 1 /* 00111011 */,
		0 /* 00111100 */, 1 /* 00111101	*/, 1 /* 00111110 */,
		0 /* 00111111 */, 1 /* 01000000	*/, 0 /* 01000001 */,
		0 /* 01000010 */, 1 /* 01000011	*/, 0 /* 01000100 */,
		1 /* 01000101 */, 1 /* 01000110	*/, 0 /* 01000111 */,
		0 /* 01001000 */, 1 /* 01001001	*/, 1 /* 01001010 */,
		0 /* 01001011 */, 1 /* 01001100	*/, 0 /* 01001101 */,
		0 /* 01001110 */, 1 /* 01001111	*/, 0 /* 01010000 */,
		1 /* 01010001 */, 1 /* 01010010	*/, 0 /* 01010011 */,
		1 /* 01010100 */, 0 /* 01010101	*/, 0 /* 01010110 */,
		1 /* 01010111 */, 1 /* 01011000	*/, 0 /* 01011001 */,
		0 /* 01011010 */, 1 /* 01011011	*/, 0 /* 01011100 */,
		1 /* 01011101 */, 1 /* 01011110	*/, 0 /* 01011111 */,
		0 /* 01100000 */, 1 /* 01100001	*/, 1 /* 01100010 */,
		0 /* 01100011 */, 1 /* 01100100	*/, 0 /* 01100101 */,
		0 /* 01100110 */, 1 /* 01100111	*/, 1 /* 01101000 */,
		0 /* 01101001 */, 0 /* 01101010	*/, 1 /* 01101011 */,
		0 /* 01101100 */, 1 /* 01101101	*/, 1 /* 01101110 */,
		0 /* 01101111 */, 1 /* 01110000	*/, 0 /* 01110001 */,
		0 /* 01110010 */, 1 /* 01110011	*/, 0 /* 01110100 */,
		1 /* 01110101 */, 1 /* 01110110	*/, 0 /* 01110111 */,
		0 /* 01111000 */, 1 /* 01111001	*/, 1 /* 01111010 */,
		0 /* 01111011 */, 1 /* 01111100	*/, 0 /* 01111101 */,
		0 /* 01111110 */, 1 /* 01111111	*/,
		1 /* 10000000 */, 0 /* 10000001	*/, 0 /* 10000010 */,
		1 /* 10000011 */, 0 /* 10000100	*/, 1 /* 10000101 */,
		1 /* 10000110 */, 0 /* 10000111	*/, 0 /* 10001000 */,
		1 /* 10001001 */, 1 /* 10001010	*/, 0 /* 10001011 */,
		1 /* 10001100 */, 0 /* 10001101	*/, 0 /* 10001110 */,
		1 /* 10001111 */, 0 /* 10010000	*/, 1 /* 10010001 */,
		1 /* 10010010 */, 0 /* 10010011	*/, 1 /* 10010100 */,
		0 /* 10010101 */, 0 /* 10010110	*/, 1 /* 10010111 */,
		1 /* 10011000 */, 0 /* 10011001	*/, 0 /* 10011010 */,
		1 /* 10011011 */, 0 /* 10011100	*/, 1 /* 10011101 */,
		1 /* 10011110 */, 0 /* 10011111	*/, 0 /* 10100000 */,
		1 /* 10100001 */, 1 /* 10100010	*/, 0 /* 10100011 */,
		1 /* 10100100 */, 0 /* 10100101	*/, 0 /* 10100110 */,
		1 /* 10100111 */, 1 /* 10101000	*/, 0 /* 10101001 */,
		0 /* 10101010 */, 1 /* 10101011	*/, 0 /* 10101100 */,
		1 /* 10101101 */, 1 /* 10101110	*/, 0 /* 10101111 */,
		1 /* 10110000 */, 0 /* 10110001	*/, 0 /* 10110010 */,
		1 /* 10110011 */, 0 /* 10110100	*/, 1 /* 10110101 */,
		1 /* 10110110 */, 0 /* 10110111	*/, 0 /* 10111000 */,
		1 /* 10111001 */, 1 /* 10111010	*/, 0 /* 10111011 */,
		1 /* 10111100 */, 0 /* 10111101	*/, 0 /* 10111110 */,
		1 /* 10111111 */, 0 /* 11000000	*/, 1 /* 11000001 */,
		1 /* 11000010 */, 0 /* 11000011	*/, 1 /* 11000100 */,
		0 /* 11000101 */, 0 /* 11000110	*/, 1 /* 11000111 */,
		1 /* 11001000 */, 0 /* 11001001	*/, 0 /* 11001010 */,
		1 /* 11001011 */, 0 /* 11001100	*/, 1 /* 11001101 */,
		1 /* 11001110 */, 0 /* 11001111	*/, 1 /* 11010000 */,
		0 /* 11010001 */, 0 /* 11010010	*/, 1 /* 11010011 */,
		0 /* 11010100 */, 1 /* 11010101	*/, 1 /* 11010110 */,
		0 /* 11010111 */, 0 /* 11011000	*/, 1 /* 11011001 */,
		1 /* 11011010 */, 0 /* 11011011	*/, 1 /* 11011100 */,
		0 /* 11011101 */, 0 /* 11011110	*/, 1 /* 11011111 */,
		1 /* 11100000 */, 0 /* 11100001	*/, 0 /* 11100010 */,
		1 /* 11100011 */, 0 /* 11100100	*/, 1 /* 11100101 */,
		1 /* 11100110 */, 0 /* 11100111	*/, 0 /* 11101000 */,
		1 /* 11101001 */, 1 /* 11101010	*/, 0 /* 11101011 */,
		1 /* 11101100 */, 0 /* 11101101	*/, 0 /* 11101110 */,
		1 /* 11101111 */, 0 /* 11110000	*/, 1 /* 11110001 */,
		1 /* 11110010 */, 0 /* 11110011	*/, 1 /* 11110100 */,
		0 /* 11110101 */, 0 /* 11110110	*/, 1 /* 11110111 */,
		1 /* 11111000 */, 0 /* 11111001	*/, 0 /* 11111010 */,
		1 /* 11111011 */, 0 /* 11111100	*/, 1 /* 11111101 */,
		1 /* 11111110 */, 0 /* 11111111	*/
};
